@@ -31,6 +31,7 @@ import java.util.Enumeration;
 import java.util.Hashtable;
 import java.util.Vector;
 
+import org.dogtagpki.ct.CTEngine;
 import org.dogtagpki.server.ca.CAEngine;
 import org.dogtagpki.server.ca.ICAService;
 import org.dogtagpki.server.ca.ICRLIssuingPoint;
@@ -89,6 +90,7 @@ import com.netscape.cms.logging.SignedAuditLogger;
 import com.netscape.cms.profile.common.Profile;
 import com.netscape.cmscore.apps.CMS;
 import com.netscape.cmscore.apps.CMSEngine;
+import com.netscape.cmscore.apps.EngineConfig;
 import com.netscape.cmscore.connector.HttpConnector;
 import com.netscape.cmscore.connector.LocalConnector;
 import com.netscape.cmscore.connector.RemoteAuthority;
@@ -98,6 +100,7 @@ import com.netscape.cmscore.dbs.CertRecord;
 import com.netscape.cmscore.dbs.CertificateRepository;
 import com.netscape.cmscore.dbs.RevocationInfo;
 import com.netscape.cmscore.profile.ProfileSubsystem;
+import com.netscape.cmsutil.crypto.CryptoUtil;
 
 /**
  * Request Service for CertificateAuthority.
@@ -232,6 +235,8 @@ public class CAService implements ICAService, IService {
             throws EBaseException {
 
         CMSEngine engine = CMS.getCMSEngine();
+        EngineConfig cs = engine.getConfig();
+
         IConnector connector = null;
 
         if (config == null || config.size() <= 0) {
@@ -274,7 +279,17 @@ public class CAService implements ICAService, IService {
             String host = config.getString("host");
             int port = config.getInteger("port");
             String uri = config.getString("uri");
+
+            // Use client cert specified in KRA connector
             String nickname = config.getString("nickName", null);
+            if (nickname == null) {
+                // Use subsystem cert as client cert
+                nickname = cs.getString("ca.subsystem.nickname");
+
+                String tokenname = cs.getString("ca.subsystem.tokenname", "");
+                if (!CryptoUtil.isInternalToken(tokenname)) nickname = tokenname + ":" + nickname;
+            }
+
             int resendInterval = config.getInteger("resendInterval", -1);
             // Inserted by beomsuk
             int timeout = config.getInteger("timeout", 0);
@@ -285,9 +300,6 @@ public class CAService implements ICAService, IService {
             RemoteAuthority remauthority =
                     new RemoteAuthority(host, port, uri, timeout);
 
-            // Change end
-            if (nickname == null)
-                nickname = mCA.getNickname();
             // Changed by beomsuk
             //connector =
             //	new HttpConnector(mCA, nickname, remauthority, resendInterval);
@@ -615,6 +627,7 @@ public class CAService implements ICAService, IService {
 
         logger.debug("dnUTF8Encoding " + doUTF8);
 
+        CertificateExtensions exts = null;
         try {
             // check required fields in certinfo.
             if (certi.get(X509CertInfo.SUBJECT) == null ||
@@ -663,7 +676,6 @@ public class CAService implements ICAService, IService {
 
             // First find out if it is a CA cert
             boolean is_ca = false;
-            CertificateExtensions exts = null;
             BasicConstraintsExtension bc_ext = null;
 
             try {
@@ -847,7 +859,13 @@ public class CAService implements ICAService, IService {
             }
         }
 
-        logger.debug("About to ca.sign cert.");
+        /*
+         * handle possible Certificate Transparency processing
+         */
+        CTEngine ctEngine = new CTEngine();
+        ctEngine.process(certi, mCA, aid, algname);
+
+        logger.debug("CAService: issueX509Cert: About to ca.sign cert.");
         cert = ca.sign(certi, algname);
         return cert;
     }
