@@ -531,32 +531,33 @@ public class CAProcessor extends Processor {
         return authenticate(httpReq, authMgr);
     }
 
-    public static void saveAuthToken(IAuthToken token, IRequest req) {
-        if (token != null && req != null)
-            req.setExtData(IRequest.AUTH_TOKEN, token);
+    public void saveAuthToken(IAuthToken token, IRequest request) {
+
+        logger.info("CAProcessor: saving authentication token into request:");
+        request.setExtData(IRequest.AUTH_TOKEN, token);
 
         // # 56230 - expose auth token parameters to the policy predicate
-        if (token != null && req != null) {
-            Enumeration<String> e = token.getElements();
-            while (e.hasMoreElements()) {
-                String n = e.nextElement();
-                String[] x1 = token.getInStringArray(n);
-                if (x1 != null) {
-                    for (int i = 0; i < x1.length; i++) {
-                        logger.debug("Setting " + IRequest.AUTH_TOKEN + "-" + n +
-                                "(" + i + ")=" + x1[i]);
-                        req.setExtData(IRequest.AUTH_TOKEN + "-" + n + "(" + i + ")",
-                                x1[i]);
-                    }
-                } else {
-                    String x = token.getInString(n);
-                    if (x != null) {
-                        logger.debug("Setting " + IRequest.AUTH_TOKEN + "-" + n + "=" + x);
-                        req.setExtData(IRequest.AUTH_TOKEN + "-" + n, x);
-                    }
+        Enumeration<String> e = token.getElements();
+
+        while (e.hasMoreElements()) {
+            String name = e.nextElement();
+            logger.info("- " + name + ":");
+
+            String[] values = token.getInStringArray(name);
+            if (values != null) {
+                for (int i = 0; i < values.length; i++) {
+                    logger.debug("  - " + values[i]);
+                    request.setExtData(IRequest.AUTH_TOKEN + "-" + name + "(" + i + ")", values[i]);
                 }
-            } // while
-        } // if
+
+            } else {
+                String value = token.getInString(name);
+                if (value != null) {
+                    logger.debug("  - " + value);
+                    request.setExtData(IRequest.AUTH_TOKEN + "-" + name, value);
+                }
+            }
+        }
     }
 
     public IAuthToken authenticate(HttpServletRequest httpReq, String authMgrName)
