@@ -41,7 +41,11 @@ import org.mozilla.jss.asn1.OBJECT_IDENTIFIER;
 import org.mozilla.jss.asn1.SEQUENCE;
 import org.mozilla.jss.asn1.SET;
 import org.mozilla.jss.asn1.UTF8String;
+import org.mozilla.jss.netscape.security.pkcs.ContentInfo;
+import org.mozilla.jss.netscape.security.pkcs.PKCS7;
+import org.mozilla.jss.netscape.security.pkcs.SignerInfo;
 import org.mozilla.jss.netscape.security.util.Utils;
+import org.mozilla.jss.netscape.security.x509.AlgorithmId;
 import org.mozilla.jss.netscape.security.x509.X509CertImpl;
 import org.mozilla.jss.pkix.cmc.LraPopWitness;
 import org.mozilla.jss.pkix.cmc.OtherInfo;
@@ -70,8 +74,8 @@ import com.netscape.cms.logging.Logger;
 import com.netscape.cms.logging.SignedAuditLogger;
 import com.netscape.cms.profile.IProfileAuthenticator;
 import com.netscape.cms.profile.common.EnrollProfile;
-import com.netscape.cms.profile.common.ProfileInput;
 import com.netscape.cms.profile.common.Profile;
+import com.netscape.cms.profile.common.ProfileInput;
 import com.netscape.cms.servlet.common.AuthCredentials;
 import com.netscape.cms.servlet.common.CMCOutputTemplate;
 import com.netscape.cms.servlet.common.CMSRequest;
@@ -895,7 +899,7 @@ public class ProfileSubmitCMCServlet extends ProfileServlet {
                         notify.notify(reqs[k]);
                     }
 
-                    logger.warn("ProfileSubmitCMCServlet: submit " + e.getMessage(), e);
+                    logger.warn("ProfileSubmitCMCServlet: submit " + e.getMessage());
                     errorCode = "2";
                     errorReason = CMS.getUserMessage(locale,
                                 "CMS_PROFILE_DEFERRED",
@@ -1057,7 +1061,16 @@ public class ProfileSubmitCMCServlet extends ProfileServlet {
             if (cert_request_type.equals("pkcs10") || cert_request_type.equals("crmf")) {
 
                 if (outputFormat != null && outputFormat.equals("pkcs7")) {
-                    byte[] pkcs7 = engine.getPKCS7(locale, reqs[0]);
+                    X509CertImpl cert = reqs[0].getExtDataInCert(EnrollProfile.REQUEST_ISSUED_CERT);
+                    X509Certificate[] certChain = engine.getCertChain(cert);
+
+                    PKCS7 p7 = new PKCS7(new AlgorithmId[0],
+                            new ContentInfo(new byte[0]),
+                            certChain,
+                            new SignerInfo[0]);
+
+                    byte[] pkcs7 = p7.getBytes();
+
                     response.setContentType("application/pkcs7-mime");
                     response.setContentLength(pkcs7.length);
                     try {
@@ -1082,6 +1095,10 @@ public class ProfileSubmitCMCServlet extends ProfileServlet {
                 template.createFullResponse(response, reqs, cert_request_type,
                         error_codes);
             }
+
+        } catch (Exception e) {
+            throw new EBaseException(e);
+
         } finally {
             SessionContext.releaseContext();
         }
