@@ -18,26 +18,19 @@
 
 package org.dogtagpki.server.ca;
 
-import java.io.ByteArrayOutputStream;
 import java.security.cert.X509Certificate;
-import java.util.Locale;
+import java.util.Arrays;
 
 import org.apache.commons.lang.StringUtils;
-import org.mozilla.jss.netscape.security.pkcs.ContentInfo;
-import org.mozilla.jss.netscape.security.pkcs.PKCS7;
-import org.mozilla.jss.netscape.security.pkcs.SignerInfo;
-import org.mozilla.jss.netscape.security.x509.AlgorithmId;
 import org.mozilla.jss.netscape.security.x509.CertificateChain;
-import org.mozilla.jss.netscape.security.x509.X509CertImpl;
 
 import com.netscape.ca.CertificateAuthority;
 import com.netscape.certsrv.base.EBaseException;
-import com.netscape.certsrv.request.IRequest;
-import com.netscape.cms.profile.common.EnrollProfile;
 import com.netscape.cmscore.apps.CMS;
 import com.netscape.cmscore.apps.CMSEngine;
 import com.netscape.cmscore.apps.EngineConfig;
 import com.netscape.cmscore.base.ConfigStorage;
+import com.netscape.cmscore.cert.CertUtils;
 import com.netscape.cmscore.cert.CrossCertPairSubsystem;
 import com.netscape.cmscore.profile.ProfileSubsystem;
 import com.netscape.cmscore.selftests.SelfTestSubsystem;
@@ -78,36 +71,21 @@ public class CAEngine extends CMSEngine {
         }
     }
 
-    public byte[] getPKCS7(Locale locale, IRequest req) {
-        try {
-            X509CertImpl cert = req.getExtDataInCert(EnrollProfile.REQUEST_ISSUED_CERT);
+    public X509Certificate[] getCertChain(X509Certificate cert) throws Exception {
 
-            if (cert == null) {
-                return null;
-            }
+        CertificateAuthority ca = (CertificateAuthority) getSubsystem(CertificateAuthority.ID);
+        CertificateChain caChain = ca.getCACertChain();
+        X509Certificate[] caCerts = caChain.getChain();
 
-            ICertificateAuthority ca = (ICertificateAuthority) getSubsystem(ICertificateAuthority.ID);
-            CertificateChain cachain = ca.getCACertChain();
-            X509Certificate[] cacerts = cachain.getChain();
-
-            X509CertImpl[] userChain = new X509CertImpl[cacerts.length + 1];
-            userChain[0] = cert;
-            for (int n = 0; n < cacerts.length; n++) {
-                userChain[n + 1] = (X509CertImpl) cacerts[n];
-            }
-
-            PKCS7 p7 = new PKCS7(new AlgorithmId[0],
-                    new ContentInfo(new byte[0]),
-                    userChain,
-                    new SignerInfo[0]);
-
-            ByteArrayOutputStream bos = new ByteArrayOutputStream();
-            p7.encodeSignedData(bos);
-            return bos.toByteArray();
-
-        } catch (Exception e) {
-            return null;
+        if (CertUtils.certInCertChain(caCerts, cert)) {
+            return Arrays.copyOf(caCerts, caCerts.length);
         }
+
+        X509Certificate[] certChain = new X509Certificate[caCerts.length + 1];
+        certChain[0] = cert;
+        System.arraycopy(caCerts, 0, certChain, 1, caCerts.length);
+
+        return certChain;
     }
 
     public void startupSubsystems() throws EBaseException {
