@@ -26,7 +26,6 @@ import java.security.InvalidKeyException;
 import java.security.Principal;
 import java.security.PublicKey;
 import java.security.SecureRandom;
-import java.security.cert.CertificateEncodingException;
 import java.security.cert.X509Certificate;
 import java.util.ArrayList;
 import java.util.Date;
@@ -37,7 +36,8 @@ import java.util.Map;
 import javax.ws.rs.core.Response;
 
 import org.apache.catalina.realm.GenericPrincipal;
-import org.dogtagpki.server.ca.ICertificateAuthority;
+import org.dogtag.util.cert.CertUtil;
+import org.dogtagpki.server.ca.CAEngine;
 import org.jboss.resteasy.plugins.providers.atom.Link;
 import org.mozilla.jss.netscape.security.pkcs.ContentInfo;
 import org.mozilla.jss.netscape.security.pkcs.PKCS7;
@@ -52,6 +52,7 @@ import org.mozilla.jss.netscape.security.x509.X509CertImpl;
 import org.mozilla.jss.netscape.security.x509.X509ExtensionException;
 import org.mozilla.jss.netscape.security.x509.X509Key;
 
+import com.netscape.ca.CertificateAuthority;
 import com.netscape.certsrv.base.BadRequestException;
 import com.netscape.certsrv.base.EBaseException;
 import com.netscape.certsrv.base.PKIException;
@@ -81,7 +82,6 @@ import com.netscape.cms.servlet.processors.CAProcessor;
 import com.netscape.cmscore.apps.CMS;
 import com.netscape.cmscore.apps.CMSEngine;
 import com.netscape.cmscore.cert.CertPrettyPrint;
-import com.netscape.cmscore.cert.CertUtils;
 import com.netscape.cmscore.security.JssSubsystem;
 import com.netscape.cmsutil.ldap.LDAPUtil;
 
@@ -93,7 +93,7 @@ public class CertService extends PKIService implements CertResource {
 
     public static org.slf4j.Logger logger = org.slf4j.LoggerFactory.getLogger(CertService.class);
 
-    ICertificateAuthority authority;
+    CertificateAuthority authority;
     ICertificateRepository repo;
     SecureRandom random;
 
@@ -105,7 +105,7 @@ public class CertService extends PKIService implements CertResource {
         CMSEngine engine = CMS.getCMSEngine();
         JssSubsystem jssSubsystem = engine.getJSSSubsystem();
 
-        authority = (ICertificateAuthority) engine.getSubsystem(ICertificateAuthority.ID);
+        authority = (CertificateAuthority) engine.getSubsystem(CertificateAuthority.ID);
         if (authority.noncesEnabled()) {
             random = jssSubsystem.getRandomNumberGenerator();
         }
@@ -139,11 +139,11 @@ public class CertService extends PKIService implements CertResource {
 
         try {
             certData = getCert(data, generateNonce);
+
         } catch (EDBRecordNotFoundException e) {
             throw new CertNotFoundException(id);
-        } catch (EBaseException e) {
-            throw new PKIException(e.getMessage(), e);
-        } catch (CertificateEncodingException e) {
+
+        } catch (Exception e) {
             throw new PKIException(e.getMessage(), e);
         }
 
@@ -500,7 +500,10 @@ public class CertService extends PKIService implements CertResource {
         return createOKResponse(infos);
     }
 
-    public CertData getCert(CertRetrievalRequest data, boolean generateNonce) throws EBaseException, CertificateEncodingException {
+    public CertData getCert(CertRetrievalRequest data, boolean generateNonce) throws Exception {
+
+        CAEngine engine = CAEngine.getInstance();
+
         CertId certId = data.getCertId();
 
         //find the cert in question
@@ -517,13 +520,25 @@ public class CertService extends PKIService implements CertResource {
         Principal subjectDN = cert.getSubjectDN();
         if (subjectDN != null) certData.setSubjectDN(subjectDN.toString());
 
-        String base64 = CertUtils.getEncodedCert(cert);
+        String base64 = CertUtil.toPEM(cert);
         certData.setEncoded(base64);
 
         CertPrettyPrint print = new CertPrettyPrint(cert);
         certData.setPrettyPrint(print.toString(getLocale(headers)));
 
-        String p7Str = getCertChainData(cert);
+        X509Certificate[] certChain = engine.getCertChain(cert);
+
+        PKCS7 pkcs7 = new PKCS7(
+                new AlgorithmId[0],
+                new ContentInfo(new byte[0]),
+                certChain,
+                new SignerInfo[0]);
+
+        ByteArrayOutputStream bos = new ByteArrayOutputStream();
+        pkcs7.encodeSignedData(bos, false);
+        byte[] p7Bytes = bos.toByteArray();
+
+        String p7Str = Utils.base64encode(p7Bytes, true);
         certData.setPkcs7CertChain(p7Str);
 
         Date notBefore = cert.getNotBefore();
@@ -605,74 +620,4 @@ public class CertService extends PKIService implements CertResource {
 
         return info;
     }
-
-    private String getCertChainData(X509CertImpl x509cert) {
-        X509Certificate mCACerts[];
-
-        if (x509cert == null) {
-            return null;
-        }
-
-        try {
-            mCACerts = authority.getCACertChain().getChain();
-        } catch (Exception e) {
-            mCACerts = null;
-        }
-
-        X509CertImpl[] certsInChain = new X509CertImpl[1];
-
-        int mCACertsLength = 0;
-        boolean certAlreadyInChain = false;
-        int certsInChainLength = 0;
-        if (mCACerts != null) {
-            mCACertsLength = mCACerts.length;
-            for (int i = 0; i < mCACertsLength; i++) {
-                if (x509cert.equals(mCACerts[i])) {
-                    certAlreadyInChain = true;
-                    break;
-                }
-            }
-
-            if (certAlreadyInChain == true) {
-                certsInChainLength = mCACertsLength;
-            } else {
-                certsInChainLength = mCACertsLength + 1;
-            }
-
-            certsInChain = new X509CertImpl[certsInChainLength];
-
-        }
-
-        certsInChain[0] = x509cert;
-
-        if (mCACerts != null) {
-            int curCount = 1;
-            for (int i = 0; i < mCACertsLength; i++) {
-                if (!x509cert.equals(mCACerts[i])) {
-                    certsInChain[curCount] = (X509CertImpl) mCACerts[i];
-                    curCount++;
-                }
-
-            }
-        }
-
-        String p7Str;
-
-        try {
-            PKCS7 p7 = new PKCS7(new AlgorithmId[0],
-                    new ContentInfo(new byte[0]),
-                    certsInChain,
-                    new SignerInfo[0]);
-            ByteArrayOutputStream bos = new ByteArrayOutputStream();
-
-            p7.encodeSignedData(bos, false);
-            byte[] p7Bytes = bos.toByteArray();
-
-            p7Str = Utils.base64encode(p7Bytes, true);
-        } catch (Exception e) {
-            p7Str = null;
-        }
-
-        return p7Str;
-    }
 }
