@@ -183,7 +183,8 @@ class NSSDatabase(object):
                  password_file=None,
                  internal_password=None,
                  internal_password_file=None,
-                 passwords=None):
+                 passwords=None,
+                 password_conf=None):
 
         if not directory:
             directory = os.path.join(
@@ -221,6 +222,7 @@ class NSSDatabase(object):
             self.internal_password_file = self.password_file
 
         self.passwords = passwords
+        self.password_conf = password_conf
 
     def create(self, enable_trust_policy=False):
 
@@ -509,7 +511,7 @@ class NSSDatabase(object):
         finally:
             shutil.rmtree(tmpdir)
 
-    def add_ca_cert(self, cert_file, trust_attributes=None):
+    def add_ca_cert(self, cert_file, trust_attributes='CT,C,C'):
 
         # Import CA certificate into internal token with automatically
         # assigned nickname.
@@ -532,8 +534,8 @@ class NSSDatabase(object):
             cmd.extend(['-C', self.internal_password_file])
 
         cmd.extend([
-            'client-cert-import',
-            '--ca-cert', cert_file
+            'nss-cert-import',
+            '--cert', cert_file
         ])
 
         if trust_attributes:
@@ -1143,6 +1145,110 @@ class NSSDatabase(object):
         logger.debug('Command: %s', ' '.join(map(str, cmd)))
         subprocess.check_call(cmd)
 
+    def get_trust(self, nickname, token=None):
+        """
+        Get trust of certificate from NSSDB
+
+        :param nickname: Nickname of the cert
+        :type nickname: str
+        :param token: Token where cert resides
+        :type token: str
+        :return: Trust flag
+        :rtype: str
+        """
+        tmpdir = tempfile.mkdtemp()
+        try:
+            token = self.get_effective_token(token)
+            password_file = self.get_password_file(tmpdir, token)
+            cmd = [
+                'certutil',
+                '-L',
+                '-d', self.directory
+            ]
+            fullname = nickname
+
+            if token:
+                cmd.extend(['-h', token])
+                fullname = token + ':' + fullname
+
+            if password_file:
+                cmd.extend(['-f', password_file])
+
+            logger.debug('Command: %s', ' '.join(map(str, cmd)))
+
+            p = subprocess.Popen(cmd,
+                                 stdout=subprocess.PIPE,
+                                 stderr=subprocess.PIPE)
+
+            output, error = p.communicate()
+
+            if error:
+                # certutil returned an error
+                # raise exception unless its not cert not found
+                if error.startswith(b'certutil: Could not find cert: '):
+                    return None
+
+                raise Exception('Could not find certificate: %s: %s' % (fullname, error.strip()))
+
+            if p.returncode != 0:
+                logger.warning('certutil returned non-zero exit code (bug #1539996)')
+
+            re_compile = re.compile(r'^' + fullname + r'\s+(\S+)$', re.MULTILINE)
+            cert_trust = re.search(re_compile, output.decode()).group(1)
+
+            return cert_trust
+
+        finally:
+            shutil.rmtree(tmpdir)
+
+    def show_cert(self, nickname, token=None):
+
+        tmpdir = tempfile.mkdtemp()
+        try:
+            token = self.get_effective_token(token)
+            password_file = self.get_password_file(tmpdir, token)
+
+            cmd = [
+                'certutil',
+                '-L',
+                '-d', self.directory
+            ]
+
+            fullname = nickname
+
+            if token:
+                cmd.extend(['-h', token])
+                fullname = token + ':' + fullname
+
+            if password_file:
+                cmd.extend(['-f', password_file])
+
+            cmd.extend(['-n', fullname])
+
+            logger.debug('Command: %s', ' '.join(map(str, cmd)))
+
+            p = subprocess.Popen(cmd,
+                                 stdout=subprocess.PIPE,
+                                 stderr=subprocess.PIPE)
+
+            output, error = p.communicate()
+
+            if error:
+                # certutil returned an error
+                # raise exception unless its not cert not found
+                if error.startswith(b'certutil: Could not find cert: '):
+                    return None
+
+                raise Exception('Could not find certificate: %s: %s' % (fullname, error.strip()))
+
+            if p.returncode != 0:
+                logger.warning('certutil returned non-zero exit code (bug #1539996)')
+
+            print(output.decode('ascii'))
+
+        finally:
+            shutil.rmtree(tmpdir)
+
     def get_cert(self, nickname, token=None, output_format='pem',
                  output_text=False):
 
@@ -1216,10 +1322,9 @@ class NSSDatabase(object):
         finally:
             shutil.rmtree(tmpdir)
 
-    def get_cert_info(self, nickname):
+    def get_cert_info(self, nickname, token=None):
 
-        cert_pem = self.get_cert(
-            nickname=nickname)
+        cert_pem = self.get_cert(nickname=nickname, token=token)
 
         if not cert_pem:
             return None
@@ -1237,6 +1342,7 @@ class NSSDatabase(object):
 
         cert['not_before'] = self.convert_time_to_millis(cert_obj.not_valid_before)
         cert['not_after'] = self.convert_time_to_millis(cert_obj.not_valid_after)
+        cert['trust_flags'] = self.get_trust(nickname=nickname, token=token)
 
         return cert
 
@@ -1245,6 +1351,35 @@ class NSSDatabase(object):
         epoch = datetime.datetime.utcfromtimestamp(0)
         return (date - epoch).total_seconds() * 1000
 
+    def export_cert_from_db(self,
+                            nickname,
+                            output_file,
+                            include_chain=False,
+                            output_format=None):
+        cmd = [
+            'pki',
+            '-d', self.directory
+        ]
+
+        if self.password_file:
+            cmd.extend(['-C', self.password_file])
+
+        if self.token:
+            cmd.extend(['--token', self.token])
+
+        cmd.extend(['nss-cert-export'])
+
+        if include_chain:
+            cmd.extend(['--with-chain'])
+
+        if output_format:
+            cmd.extend(['--format', output_format])
+
+        cmd.extend([nickname, output_file])
+
+        logger.debug('Command: %s', ' '.join(map(str, cmd)))
+        subprocess.check_call(cmd)
+
     def export_cert(self,
                     nickname,
                     pkcs12_file,
@@ -1387,13 +1522,18 @@ class NSSDatabase(object):
                 )
 
             elif file_type == 'pkcs7':  # import PKCS #7 cert chain
-                chain, nicks = self.import_pkcs7(
+                self.import_pkcs7(
                     pkcs7_file=cert_chain_file,
                     nickname=nickname,
                     token=token,
-                    trust_attributes=trust_attributes,
-                    output_format='base64')
-                return chain, nicks
+                    trust_attributes=trust_attributes)
+
+                with open(cert_chain_file, 'r') as f:
+                    pkcs7_data = f.read()
+
+                base64_data = convert_pkcs7(pkcs7_data, 'pem', 'base64')
+
+                return base64_data, [nickname]
 
             else:  # import PKCS #7 data without header/footer
                 with open(cert_chain_file, 'r') as f:
@@ -1411,24 +1551,54 @@ class NSSDatabase(object):
                 with open(tmp_cert_chain_file, 'w') as f:
                     f.write(pkcs7_data)
 
-                chain, nicks = self.import_pkcs7(
+                self.import_pkcs7(
                     pkcs7_file=tmp_cert_chain_file,
                     nickname=nickname,
                     token=token,
                     trust_attributes=trust_attributes)
 
-                return base64_data, nicks
+                return base64_data, [nickname]
 
         finally:
             shutil.rmtree(tmpdir)
 
     def import_pkcs7(
             self,
-            pkcs7_file,
-            nickname,
+            pkcs7_data=None,
+            pkcs7_file=None,
+            nickname=None,
             token=None,
-            trust_attributes=None,
-            output_format='pem'):
+            trust_attributes=None):
+
+        if not nickname:
+
+            # Import certificate chain without nickname
+
+            cmd = ['pki', '-d', self.directory]
+
+            if self.internal_password_file:
+                cmd.extend(['-C', self.internal_password_file])
+
+            cmd.extend(['pkcs7-import'])
+
+            if pkcs7_file:
+                cmd.extend(['--pkcs7', pkcs7_file])
+
+            if trust_attributes:
+                cmd.extend(['--trust', trust_attributes])
+
+            logger.debug('Command: %s', ' '.join(map(str, cmd)))
+
+            if pkcs7_data:
+                data = pkcs7_data.encode('utf-8')
+            else:
+                data = None
+
+            subprocess.run(cmd, input=data, check=True)
+
+            return
+
+        # Import certificate chain with nickname
 
         tmpdir = tempfile.mkdtemp()
 
@@ -1473,12 +1643,6 @@ class NSSDatabase(object):
         finally:
             shutil.rmtree(tmpdir)
 
-        # convert PKCS #7 data to the requested format
-        with open(pkcs7_file, 'r') as f:
-            data = f.read()
-
-        return convert_pkcs7(data, 'pem', output_format), [nickname]
-
     def import_pkcs12(self, pkcs12_file,
                       pkcs12_password=None,
                       pkcs12_password_file=None,
@@ -1506,7 +1670,10 @@ class NSSDatabase(object):
                 '-d', self.directory
             ]
 
-            if self.password_file:
+            if self.password_conf:
+                cmd.extend(['-f', self.password_conf])
+
+            elif self.password_file:
                 cmd.extend(['-C', self.password_file])
 
             if self.token:
@@ -1514,8 +1681,8 @@ class NSSDatabase(object):
 
             cmd.extend([
                 'pkcs12-import',
-                '--pkcs12-file', pkcs12_file,
-                '--pkcs12-password-file', password_file
+                '--pkcs12', pkcs12_file,
+                '--password-file', password_file
             ])
 
             if no_user_certs:
@@ -1527,6 +1694,12 @@ class NSSDatabase(object):
             if overwrite:
                 cmd.extend(['--overwrite'])
 
+            if logger.isEnabledFor(logging.DEBUG):
+                cmd.append('--debug')
+
+            elif logger.isEnabledFor(logging.INFO):
+                cmd.append('-v')
+
             logger.debug('Command: %s', ' '.join(map(str, cmd)))
             subprocess.check_call(cmd)
 
@@ -1573,8 +1746,8 @@ class NSSDatabase(object):
             cmd.extend(['pkcs12-export'])
 
             cmd.extend([
-                '--pkcs12-file', pkcs12_file,
-                '--pkcs12-password-file', password_file
+                '--pkcs12', pkcs12_file,
+                '--password-file', password_file
             ])
 
             if cert_encryption:
@@ -1607,6 +1780,13 @@ class NSSDatabase(object):
         finally:
             shutil.rmtree(tmpdir)
 
+    def extract_ca_cert(self, ca_path, nickname):
+        # Build a chain containing the certificate we're trying to
+        # export. OpenSSL gets confused if we don't have a key for
+        # the end certificate: rh-bz#1246371
+        self.export_cert_from_db(nickname, ca_path, include_chain=True,
+                                 output_format="PEM")
+
     @staticmethod
     def __generate_key_args(key_type=None, key_size=None, curve=None):
         """
