@@ -299,19 +299,6 @@ class SecurityDomainClient(object):
         return InstallToken.from_json(response.json())
 
 
-class ConfigurationRequest(object):
-    """
-    Class used to represent a configuration request to be submitted to the
-    Java installation servlet during the execution of pkispawn.
-
-    This class is the python equivalent of the Java class:
-    com.netscape.certsrv.system.ConfigurationRequest
-    """
-
-    def __init__(self):
-        self.isClone = "false"
-
-
 class CloneSetupRequest(object):
     def __init__(self):
         self.domainInfo = None
@@ -326,7 +313,6 @@ class DatabaseSetupRequest(object):
 class CertificateSetupRequest(object):
     def __init__(self):
         self.installToken = None
-        self.generateServerCert = "true"
         self.clone = 'false'
 
 
@@ -413,23 +399,6 @@ class SystemConfigClient(object):
             self.setup_db_user_url = '/' + subsystem + self.setup_db_user_url
             self.finalize_config_url = '/' + subsystem + self.finalize_config_url
 
-    def configure(self, request):
-        """
-        Contacts the server and invokes the Java configuration REST API to
-        configure a Dogtag subsystem.
-
-        :param request: Configuration request containing all the input needed to
-            configure the subsystem
-        :type request: ConfigurationRequest
-        """
-        data = json.dumps(request, cls=pki.encoder.CustomTypeEncoder)
-        headers = {'Content-type': 'application/json',
-                   'Accept': 'application/json'}
-        self.connection.post(
-            self.configure_url,
-            data,
-            headers)
-
     def setupClone(self, request):
         """
         Set up clone.
@@ -578,7 +547,6 @@ class SystemStatusClient(object):
         return response.text
 
 
-pki.encoder.NOTYPES['ConfigurationRequest'] = ConfigurationRequest
 pki.encoder.NOTYPES['DomainInfo'] = DomainInfo
 pki.encoder.NOTYPES['SecurityDomainSubsystem'] = SecurityDomainSubsystem
 pki.encoder.NOTYPES['SecurityDomainHost'] = SecurityDomainHost
