@@ -81,7 +81,7 @@ public class KeyClient extends Client {
     private Version getServerVersion() {
         Version ret = null;
         try {
-            Info info = client.infoClient.getInfo();
+            Info info = client.getInfo();
             String version = info.getVersion();
             ret = new Version(version);
         } catch (Exception e) {
@@ -990,7 +990,11 @@ public class KeyClient extends Client {
         data.setKeyAlgorithm(keyAlgorithm);
         data.setKeySize(keySize);
         data.setUsages(usages);
-        data.setTransWrappedSessionKey(Utils.base64encode(transWrappedSessionKey, false));
+        if (transWrappedSessionKey !=null) {
+            // Caller by default calls in with transWrappedSessionKey null;
+            // And the underlying Java code does not like null;
+            data.setTransWrappedSessionKey(Utils.base64encode(transWrappedSessionKey, false));
+        }
         data.setRealm(realm);
 
         return submitRequest(data);
