@@ -23,6 +23,7 @@ import javax.ws.rs.core.Response;
 
 import com.netscape.certsrv.client.Client;
 import com.netscape.certsrv.client.PKIClient;
+import com.netscape.certsrv.client.SubsystemClient;
 
 /**
  * @author Endi S. Dewata
@@ -36,6 +37,10 @@ public class UserClient extends Client {
         init();
     }
 
+    public UserClient(SubsystemClient subsystemClient) throws URISyntaxException {
+        this(subsystemClient.client, subsystemClient.getName());
+    }
+
     public void init() throws URISyntaxException {
         userClient = createProxy(UserResource.class);
     }
