@@ -20,10 +20,9 @@ package com.netscape.cmstools.ca;
 
 import java.io.BufferedReader;
 import java.io.InputStreamReader;
-import java.util.Arrays;
 
 import org.apache.commons.cli.CommandLine;
-import org.dogtagpki.cli.CLI;
+import org.dogtagpki.cli.CommandCLI;
 
 import com.netscape.certsrv.ca.CACertClient;
 import com.netscape.certsrv.cert.CertData;
@@ -35,15 +34,15 @@ import com.netscape.cmstools.cli.MainCLI;
 /**
  * @author Endi S. Dewata
  */
-public class CACertReleaseHoldCLI extends CLI {
+public class CACertReleaseHoldCLI extends CommandCLI {
+
+    public static org.slf4j.Logger logger = org.slf4j.LoggerFactory.getLogger(CACertReleaseHoldCLI.class);
 
     public CACertCLI certCLI;
 
     public CACertReleaseHoldCLI(CACertCLI certCLI) {
         super("release-hold", "Place certificate off-hold", certCLI);
         this.certCLI = certCLI;
-
-        createOptions();
     }
 
     public void printHelp() {
@@ -54,14 +53,7 @@ public class CACertReleaseHoldCLI extends CLI {
         options.addOption(null, "force", false, "Force");
     }
 
-    public void execute(String[] args) throws Exception {
-        // Always check for "--help" prior to parsing
-        if (Arrays.asList(args).contains("--help")) {
-            printHelp();
-            return;
-        }
-
-        CommandLine cmd = parser.parse(options, args);
+    public void execute(CommandLine cmd) throws Exception {
 
         String[] cmdArgs = cmd.getArgs();
 
@@ -70,6 +62,10 @@ public class CACertReleaseHoldCLI extends CLI {
         }
 
         CertId certID = new CertId(cmdArgs[0]);
+
+        MainCLI mainCLI = (MainCLI) getRoot();
+        mainCLI.init();
+
         CACertClient certClient = certCLI.getCertClient();
 
         if (!cmd.hasOption("force")) {
@@ -92,8 +88,8 @@ public class CACertReleaseHoldCLI extends CLI {
 
         CertRequestInfo certRequestInfo = certClient.unrevokeCert(certID);
 
-        if (verbose) {
-            CACertCLI.printCertRequestInfo(certRequestInfo);
+        if (logger.isInfoEnabled()) {
+            CACertRequestCLI.printCertRequestInfo(certRequestInfo);
         }
 
         if (certRequestInfo.getRequestStatus() == RequestStatus.COMPLETE) {
