@@ -37,6 +37,7 @@ import java.util.List;
 import java.util.Map;
 
 import javax.ws.rs.ProcessingException;
+import javax.ws.rs.core.Response;
 
 import org.apache.commons.cli.CommandLine;
 import org.apache.commons.cli.Option;
@@ -45,7 +46,6 @@ import org.apache.commons.lang.StringUtils;
 import org.dogtagpki.cli.CLI;
 import org.dogtagpki.cli.CLIException;
 import org.dogtagpki.common.Info;
-import org.dogtagpki.common.InfoClient;
 import org.dogtagpki.nss.NSSDatabase;
 import org.dogtagpki.util.logging.PKILogger;
 import org.dogtagpki.util.logging.PKILogger.Level;
@@ -53,11 +53,11 @@ import org.mozilla.jss.CryptoManager;
 import org.mozilla.jss.NotInitializedException;
 import org.mozilla.jss.crypto.CryptoToken;
 import org.mozilla.jss.ssl.SSLCertificateApprovalCallback;
-import org.mozilla.jss.ssl.SSLVersion;
-import org.mozilla.jss.ssl.SSLVersionRange;
+import org.mozilla.jss.ssl.SSLSocket;
 import org.mozilla.jss.util.IncorrectPasswordException;
 import org.mozilla.jss.util.Password;
 
+import com.netscape.certsrv.base.PKIException;
 import com.netscape.certsrv.ca.CAClient;
 import com.netscape.certsrv.client.ClientConfig;
 import com.netscape.certsrv.client.PKIClient;
@@ -78,6 +78,7 @@ import com.netscape.cmstools.tks.TKSCLI;
 import com.netscape.cmstools.tps.TPSCLI;
 import com.netscape.cmstools.user.ProxyUserCLI;
 import com.netscape.cmsutil.crypto.CryptoUtil;
+import com.netscape.cmsutil.password.PlainPasswordFile;
 
 /**
  * @author Endi S. Dewata
@@ -86,15 +87,17 @@ public class MainCLI extends CLI {
 
     public ClientConfig config = new ClientConfig();
 
+    NSSDatabase nssdb;
+
     public Collection<Integer> rejectedCertStatuses = new HashSet<Integer>();
     public Collection<Integer> ignoredCertStatuses = new HashSet<Integer>();
 
     public boolean ignoreBanner;
-    public File certDatabase;
 
     String output;
 
     boolean initialized;
+    boolean optionsParsed;
 
     public MainCLI() throws Exception {
         super("pki", "PKI command-line interface");
@@ -125,8 +128,12 @@ public class MainCLI extends CLI {
         createOptions();
     }
 
-    public File getNSSDatabase() {
-        return certDatabase;
+    public ClientConfig getConfig() {
+        return config;
+    }
+
+    public NSSDatabase getNSSDatabase() {
+        return nssdb;
     }
 
     public String getFullModuleName(String moduleName) {
@@ -417,6 +424,21 @@ public class MainCLI extends CLI {
             config.setNSSPasswords(nssPasswords);
         }
 
+        PlainPasswordFile passwordStore = new PlainPasswordFile();
+
+        if (nssPassword != null) {
+            String token = tokenName;
+            if (token == null) token = "internal";
+            passwordStore.putPassword(token, nssPassword);
+
+        } else if (nssPasswordConfig != null) {
+            passwordStore.init(nssPasswordConfig);
+        }
+
+        logger.info("NSS database: " + config.getNSSDatabase());
+        nssdb = new NSSDatabase(config.getNSSDatabase());
+        nssdb.setPasswordStore(passwordStore);
+
         // store user name
         config.setUsername(username);
 
@@ -440,16 +462,11 @@ public class MainCLI extends CLI {
 
         ignoreBanner = cmd.hasOption("ignore-banner");
 
-        this.certDatabase = new File(config.getNSSDatabase());
-        logger.info("NSS database: " + this.certDatabase.getAbsolutePath());
-
         String messageFormat = cmd.getOptionValue("message-format");
         config.setMessageFormat(messageFormat);
         logger.info("Message format: " + messageFormat);
-    }
 
-    public ClientConfig getConfig() {
-        return config;
+        optionsParsed = true;
     }
 
     public void convertCertStatusList(String list, Collection<Integer> statuses) throws Exception {
@@ -475,15 +492,22 @@ public class MainCLI extends CLI {
             return;
         }
 
-        NSSDatabase nssdb = new NSSDatabase(certDatabase);
+        if (!optionsParsed) {
+            throw new Exception("Unable to call MainCLI.init() without first calling MainCLI.parseOptions()");
+        }
 
         if (!nssdb.exists()) {
-            // Create a default NSS database without password
-            nssdb.create();
+            // Create the NSS DB with the specified password, if one has been
+            // specified.
+            if (config.getNSSPassword() != null) {
+                nssdb.create(config.getNSSPassword());
+            } else {
+                nssdb.create();
+            }
         }
 
         logger.info("Initializing NSS");
-        CryptoManager.initialize(certDatabase.getAbsolutePath());
+        CryptoManager.initialize(nssdb.getPath().toString());
 
         CryptoManager manager;
         try {
@@ -547,35 +571,7 @@ public class MainCLI extends CLI {
         CryptoToken token = CryptoUtil.getKeyStorageToken(tokenName);
         manager.setThreadToken(token);
 
-        // See default SSL configuration in /usr/share/pki/etc/pki.conf.
-
-        String streamVersionMin = System.getenv("SSL_STREAM_VERSION_MIN");
-        String streamVersionMax = System.getenv("SSL_STREAM_VERSION_MAX");
-
-        SSLVersionRange streamRange = CryptoUtil.boundSSLStreamVersionRange(
-                streamVersionMin == null ? SSLVersion.TLS_1_0 : SSLVersion.valueOf(streamVersionMin),
-                streamVersionMax == null ? SSLVersion.TLS_1_2 : SSLVersion.valueOf(streamVersionMax)
-        );
-        CryptoUtil.setSSLStreamVersionRange(streamRange.getMinVersion(), streamRange.getMaxVersion());
-
-        String datagramVersionMin = System.getenv("SSL_DATAGRAM_VERSION_MIN");
-        String datagramVersionMax = System.getenv("SSL_DATAGRAM_VERSION_MAX");
-
-        SSLVersionRange datagramRange = CryptoUtil.boundSSLDatagramVersionRange(
-                datagramVersionMin == null ? SSLVersion.TLS_1_1 : SSLVersion.valueOf(datagramVersionMin),
-                datagramVersionMax == null ? SSLVersion.TLS_1_2 : SSLVersion.valueOf(datagramVersionMax)
-        );
-        CryptoUtil.setSSLDatagramVersionRange(datagramRange.getMinVersion(), datagramRange.getMaxVersion());
-
-        String defaultCiphers = System.getenv("SSL_DEFAULT_CIPHERS");
-        if (defaultCiphers == null || Boolean.parseBoolean(defaultCiphers)) {
-            CryptoUtil.setDefaultSSLCiphers();
-        } else {
-            CryptoUtil.unsetSSLCiphers();
-        }
-
-        String ciphers = System.getenv("SSL_CIPHERS");
-        CryptoUtil.setSSLCiphers(ciphers);
+        SSLSocket.enablePostHandshakeAuthDefault(true);
 
         initialized = true;
     }
@@ -584,9 +580,9 @@ public class MainCLI extends CLI {
 
         if (client != null) return client;
 
-        logger.info("Initializing PKIClient");
+        logger.info("Connecting to " + config.getServerURL());
 
-        client = new PKIClient(config, null);
+        client = new PKIClient(config);
         client.setRejectedCertStatuses(rejectedCertStatuses);
         client.setIgnoredCertStatuses(ignoredCertStatuses);
 
@@ -598,13 +594,15 @@ public class MainCLI extends CLI {
             connection.setOutput(file);
         }
 
-        if (!ignoreBanner) {
+        try {
+            Info info = client.getInfo();
+
+            logger.info("Server Name: " + info.getName());
+            logger.info("Server Version: " + info.getVersion());
 
-            InfoClient infoClient = new InfoClient(client);
-            Info info = infoClient.getInfo();
             String banner = info.getBanner();
 
-            if (banner != null) {
+            if (banner != null && !ignoreBanner) {
 
                 System.out.println(banner);
                 System.out.println();
@@ -618,6 +616,12 @@ public class MainCLI extends CLI {
                     throw new CLIException();
                 }
             }
+
+        } catch (PKIException e) {
+            if (e.getCode() != Response.Status.NOT_FOUND.getStatusCode()) {
+                throw e;
+            }
+            logger.warn("Unable to get server info: " + e.getMessage());
         }
 
         return client;
