@@ -19,25 +19,27 @@
 package com.netscape.cmstools.kra;
 
 import org.dogtagpki.cli.CLI;
+import org.dogtagpki.kra.KRASystemCertClient;
 import org.mozilla.jss.CryptoManager;
 import org.mozilla.jss.crypto.X509Certificate;
+import org.mozilla.jss.netscape.security.util.Cert;
+import org.mozilla.jss.netscape.security.util.Utils;
 
 import com.netscape.certsrv.client.PKIClient;
 import com.netscape.certsrv.key.KeyClient;
 import com.netscape.certsrv.key.KeyInfo;
 import com.netscape.certsrv.key.KeyRequestInfo;
-import com.netscape.certsrv.system.SystemCertClient;
 import com.netscape.certsrv.util.NSSCryptoProvider;
 import com.netscape.cmstools.cli.MainCLI;
 import com.netscape.cmstools.cli.SubsystemCLI;
-import org.mozilla.jss.netscape.security.util.Cert;
-import org.mozilla.jss.netscape.security.util.Utils;
 
 /**
  * @author Endi S. Dewata
  */
 public class KRAKeyCLI extends CLI {
 
+    public static org.slf4j.Logger logger = org.slf4j.LoggerFactory.getLogger(KRAKeyCLI.class);
+
     public KeyClient keyClient;
 
     public KRAKeyCLI(CLI parent) {
@@ -96,38 +98,31 @@ public class KRAKeyCLI extends CLI {
         // create new key client
         keyClient = new KeyClient(client, subsystem);
 
-        // if security database password is specified,
-        // prepare key client for archival/retrieval
-        if (client.getConfig().getNSSPassword() != null) {
+        // create crypto provider for key client
+        keyClient.setCrypto(new NSSCryptoProvider(client.getConfig()));
 
-            // create crypto provider for key client
-            keyClient.setCrypto(new NSSCryptoProvider(client.getConfig()));
+        CryptoManager manager = CryptoManager.getInstance();
+        X509Certificate transportCert;
 
-            CryptoManager manager = CryptoManager.getInstance();
-            X509Certificate transportCert;
+        if (transportNickname == null) {
+            // download transport cert
+            KRASystemCertClient systemCertClient = new KRASystemCertClient(client, subsystem);
+            String pemCert = systemCertClient.getTransportCert().getEncoded();
+            String b64Cert = pemCert.substring(Cert.HEADER.length(), pemCert.indexOf(Cert.FOOTER));
+            byte[] binCert = Utils.base64decode(b64Cert);
 
-            if (transportNickname == null) {
-                // download transport cert
-                SystemCertClient systemCertClient = new SystemCertClient(client, subsystem);
-                String pemCert = systemCertClient.getTransportCert().getEncoded();
-                String b64Cert = pemCert.substring(Cert.HEADER.length(), pemCert.indexOf(Cert.FOOTER));
-                byte[] binCert = Utils.base64decode(b64Cert);
+            transportCert = manager.importCACertPackage(binCert);
 
-                transportCert = manager.importCACertPackage(binCert);
-
-            } else {
-                // load transport cert
-                transportCert = manager.findCertByNickname(transportNickname);
-            }
-
-            if (verbose) {
-                System.out.println("Transport cert: " + transportCert.getNickname());
-            }
-
-            // set transport cert for key client
-            keyClient.setTransportCert(transportCert);
+        } else {
+            // load transport cert
+            transportCert = manager.findCertByNickname(transportNickname);
         }
 
+        logger.info("Transport cert: " + transportCert.getNickname());
+
+        // set transport cert for key client
+        keyClient.setTransportCert(transportCert);
+
         return keyClient;
     }
 
