@@ -1710,8 +1710,12 @@ public class KeyRecoveryAuthority implements IAuthority, IKeyService, IKeyRecove
         return agents;
     }
 
-    public KeyPair generateKeyPair(String alg, int keySize, String keyCurve,
+   public KeyPair generateKeyPair(String alg, int keySize, String keyCurve,
             PQGParams pqg, KeyPairGeneratorSpi.Usage[] usageList) throws EBaseException {
+        return generateKeyPair(alg, keySize, keyCurve, pqg, usageList, false);
+    }
+    public KeyPair generateKeyPair(String alg, int keySize, String keyCurve,
+            PQGParams pqg, KeyPairGeneratorSpi.Usage[] usageList, boolean temp) throws EBaseException {
         KeyPairAlgorithm kpAlg = null;
 
         if (alg.equals("RSA"))
@@ -1722,7 +1726,7 @@ public class KeyRecoveryAuthority implements IAuthority, IKeyService, IKeyRecove
             kpAlg = KeyPairAlgorithm.DSA;
 
         try {
-            KeyPair kp = generateKeyPair(kpAlg, keySize, keyCurve, pqg, usageList);
+            KeyPair kp = generateKeyPair(kpAlg, keySize, keyCurve, pqg, usageList, temp);
 
             return kp;
         } catch (InvalidParameterException e) {
@@ -1745,6 +1749,13 @@ public class KeyRecoveryAuthority implements IAuthority, IKeyService, IKeyRecove
             KeyPairGeneratorSpi.Usage[] usageList )
             throws NoSuchAlgorithmException, TokenException, InvalidAlgorithmParameterException,
             InvalidParameterException, PQGParamGenException {
+        return generateKeyPair(kpAlg, keySize, keyCurve, pqg, usageList, true);
+    }
+    public KeyPair generateKeyPair(
+            KeyPairAlgorithm kpAlg, int keySize, String keyCurve, PQGParams pqg,
+            KeyPairGeneratorSpi.Usage[] usageList, boolean temp)
+            throws NoSuchAlgorithmException, TokenException, InvalidAlgorithmParameterException,
+            InvalidParameterException, PQGParamGenException {
 
         CMSEngine engine = CMS.getCMSEngine();
         CryptoToken token = getKeygenToken();
@@ -1765,7 +1776,7 @@ public class KeyRecoveryAuthority implements IAuthority, IKeyService, IKeyRecove
         KeyPairGenerator kpGen = token.getKeyPairGenerator(kpAlg);
         EngineConfig config = engine.getConfig();
         IConfigStore kgConfig = config.getSubStore("kra.keygen");
-        boolean tp = false;
+        boolean tp = temp;
         boolean sp = false;
         boolean ep = false;
         if ((kgConfig != null) && (!kgConfig.equals(""))) {
