@@ -27,11 +27,13 @@ import grp
 import inspect
 import logging
 import os
+import pathlib
 import pwd
 import re
 import shutil
 import subprocess
 import tempfile
+import socket
 
 import ldap
 import ldap.filter
@@ -52,6 +54,15 @@ LIB_SYSTEMD_DIR = '/lib/systemd'
 
 SUBSYSTEM_TYPES = ['ca', 'kra', 'ocsp', 'tks', 'tps']
 
+DEFAULT_DIR_MODE = 0o0770
+DEFAULT_FILE_MODE = 0o0660
+DEFAULT_LINK_MODE = 0o0777
+
+SCHEMA_FILES = [
+    '/usr/share/pki/server/conf/schema.ldif',
+    '/usr/share/pki/acme/database/ldap/schema.ldif'
+]
+
 logger = logging.getLogger(__name__)
 
 parser = etree.XMLParser(remove_blank_text=True)
@@ -110,6 +121,7 @@ class PKIServer(object):
 
         self.config = {}
         self.passwords = {}
+        self.subsystems = {}
 
     def __repr__(self):
         return self.name
@@ -217,6 +229,10 @@ class PKIServer(object):
 
     @property
     def nssdb_dir(self):
+        return os.path.join(self.conf_dir, 'alias')
+
+    @property
+    def nssdb_link(self):
         return os.path.join(self.base_dir, 'alias')
 
     @property
@@ -224,10 +240,13 @@ class PKIServer(object):
         return os.path.join(self.conf_dir, 'jss.conf')
 
     def is_valid(self):
+        return self.exists()
+
+    def exists(self):
         return os.path.exists(self.base_dir)
 
     def validate(self):
-        if not self.is_valid():
+        if not self.exists():
             raise pki.PKIException('Invalid instance: ' + self.name, None)
 
     def is_active(self):
@@ -261,11 +280,30 @@ class PKIServer(object):
         logger.debug('Command: %s', ' '.join(cmd))
         subprocess.check_call(cmd)
 
-    def run(self, command='start', jdb=False, as_current_user=False, gdb=False):
-        p = self.execute(command, jdb=jdb, as_current_user=as_current_user, gdb=gdb)
+    def run(self, command='start',
+            as_current_user=False,
+            with_jdb=False,
+            with_gdb=False,
+            with_valgrind=False,
+            agentpath=None):
+
+        p = self.execute(
+            command,
+            as_current_user=as_current_user,
+            with_jdb=with_jdb,
+            with_gdb=with_gdb,
+            with_valgrind=with_valgrind,
+            agentpath=agentpath)
+
         p.wait()
 
-    def execute(self, command, jdb=False, as_current_user=False, gdb=False):
+    def execute(
+            self, command,
+            as_current_user=False,
+            with_jdb=False,
+            with_gdb=False,
+            with_valgrind=False,
+            agentpath=None):
 
         logger.debug('Environment variables:')
         for name in self.config:
@@ -282,6 +320,7 @@ class PKIServer(object):
             if current_user != self.user:
                 prefix.extend(['sudo', '-u', self.user])
 
+        java_path = os.getenv('PKI_JAVA_PATH')
         java_home = self.config.get('JAVA_HOME')
         java_opts = self.config.get('JAVA_OPTS')
         security_manager = self.config.get('SECURITY_MANAGER')
@@ -293,22 +332,33 @@ class PKIServer(object):
         ]
 
         cmd = prefix
-        if gdb:
+
+        if with_valgrind:
+            cmd.extend(['valgrind', '--trace-children=yes', '--tool=massif'])
+
+        if with_gdb:
             cmd.extend(['gdb', '--args'])
 
-        if jdb:
+        if with_jdb:
             cmd.extend(['jdb'])
 
         else:
-            if java_home:
+            if os.path.exists(java_path):
+                cmd.extend([java_path])
+            elif os.path.exists(java_home + '/jre/bin/java'):
+                cmd.extend([java_home + '/jre/bin/java'])
+            elif os.path.exists(java_home + '/bin/java'):
                 cmd.extend([java_home + '/bin/java'])
             else:
-                cmd.extend(['java'])
+                cmd.extend(['/usr/bin/env', 'java'])
 
-            if os.path.exists('/usr/lib/abrt-java-connector/libabrt-java-connector.so'):
-                cmd.extend([
-                    '-agentpath:/usr/lib/abrt-java-connector/libabrt-java-connector.so=abrt=on,'
-                ])
+        if agentpath:
+            cmd.extend(['-agentpath:%s' % agentpath])
+
+        elif os.path.exists('/usr/lib/abrt-java-connector/libabrt-java-connector.so'):
+            cmd.extend([
+                '-agentpath:/usr/lib/abrt-java-connector/libabrt-java-connector.so=abrt=on,'
+            ])
 
         cmd.extend([
             '-classpath', os.pathsep.join(classpath),
@@ -329,15 +379,28 @@ class PKIServer(object):
         if java_opts:
             cmd.extend(java_opts.split())
 
+        if with_valgrind:
+            cmd.extend(['-Djava.compiler=NONE'])
+
         cmd.extend(['org.apache.catalina.startup.Bootstrap', command])
 
         logger.debug('Command: %s', ' '.join(cmd))
 
         return subprocess.Popen(cmd, env=self.config)
 
-    def makedirs(self, path, force=False):
+    def touch(self, path):
+        pathlib.Path(path).touch()
+        os.chown(path, self.uid, self.gid)
+        os.chmod(path, DEFAULT_FILE_MODE)
+
+    def makedirs(self, path, exist_ok=None, force=False):
         pki.util.makedirs(
-            path, uid=self.uid, gid=self.gid, force=force)
+            path,
+            mode=DEFAULT_DIR_MODE,
+            exist_ok=exist_ok,
+            uid=self.uid,
+            gid=self.gid,
+            force=force)
 
     def symlink(self, source, dest, force=False):
         pki.util.symlink(
@@ -345,53 +408,95 @@ class PKIServer(object):
 
     def copy(self, source, dest, force=False):
         pki.util.copy(
-            source, dest, uid=self.uid, gid=self.gid, force=force)
+            source,
+            dest,
+            uid=self.uid,
+            gid=self.gid,
+            dir_mode=DEFAULT_DIR_MODE,
+            file_mode=DEFAULT_FILE_MODE,
+            force=force)
+
+    def copydirs(self, source, dest, force=False):
+        pki.util.copydirs(
+            source,
+            dest,
+            uid=self.uid,
+            gid=self.gid,
+            mode=DEFAULT_DIR_MODE,
+            force=force)
 
     def copyfile(self, source, dest, slots=None, params=None, force=False):
         pki.util.copyfile(
-            source, dest, slots=slots, params=params,
-            uid=self.uid, gid=self.gid, force=force)
+            source,
+            dest,
+            slots=slots,
+            params=params,
+            uid=self.uid,
+            gid=self.gid,
+            mode=DEFAULT_FILE_MODE,
+            force=force)
+
+    def store_properties(self, filename, properties):
+        pki.util.store_properties(filename, properties)
+        pki.util.chown(filename, self.uid, self.gid)
 
     def create(self, force=False):
 
+        logger.info('Creating %s', self.base_dir)
         self.makedirs(self.base_dir, force=force)
 
+        logger.info('Creating %s', self.bin_dir)
         bin_dir = os.path.join(Tomcat.SHARE_DIR, 'bin')
         self.symlink(bin_dir, self.bin_dir, force=force)
 
+        logger.info('Creating %s', self.conf_dir)
         self.makedirs(self.conf_dir, force=force)
 
+        logger.info('Creating %s', self.catalina_policy)
         catalina_policy = os.path.join(Tomcat.CONF_DIR, 'catalina.policy')
         self.copy(catalina_policy, self.catalina_policy, force=force)
 
+        logger.info('Creating %s', self.catalina_properties)
         catalina_properties = os.path.join(
             PKIServer.SHARE_DIR, 'server', 'conf', 'catalina.properties')
         self.symlink(catalina_properties, self.catalina_properties, force=force)
 
+        logger.info('Creating %s', self.context_xml)
         context_xml = os.path.join(Tomcat.CONF_DIR, 'context.xml')
         self.symlink(context_xml, self.context_xml, force=force)
 
+        logger.info('Creating %s', self.logging_properties)
         logging_properties = os.path.join(Tomcat.CONF_DIR, 'logging.properties')
         self.copy(logging_properties, self.logging_properties, force=force)
 
         self.create_server_xml()
 
+        logger.info('Creating %s', self.tomcat_conf)
         self.copy(Tomcat.TOMCAT_CONF, self.tomcat_conf, force=force)
 
         with open(self.tomcat_conf, 'a') as f:
             f.write('\nPKI_VERSION=%s\n' % pki.specification_version())
 
+        logger.info('Creating %s', self.web_xml)
         web_xml = os.path.join(Tomcat.CONF_DIR, 'web.xml')
         self.symlink(web_xml, self.web_xml, force=force)
 
         conf_d_dir = os.path.join(self.conf_dir, 'conf.d')
+        logger.info('Creating %s', conf_d_dir)
         self.makedirs(conf_d_dir, force=force)
 
         self.create_libs(force=force)
 
+        logger.info('Creating %s', self.temp_dir)
         self.makedirs(self.temp_dir, force=force)
+
+        logger.info('Creating %s', self.webapps_dir)
         self.makedirs(self.webapps_dir, force=force)
+
+        logger.info('Creating %s', self.work_dir)
         self.makedirs(self.work_dir, force=force)
+
+        logger.info('Creating %s', self.log_dir)
         self.makedirs(self.log_dir, force=force)
 
         document = etree.parse(self.server_xml, parser)
@@ -399,14 +504,19 @@ class PKIServer(object):
 
         for engine in server.findall('Service/Engine'):
             engine_name = engine.get('name')
+
             engine_dir = os.path.join(self.conf_dir, engine_name)
+            logger.info('Creating %s', engine_dir)
             self.makedirs(engine_dir, force=force)
 
             for host in engine.findall('Host'):
                 host_name = host.get('name')
+
                 host_dir = os.path.join(engine_dir, host_name)
+                logger.info('Creating %s', host_dir)
                 self.makedirs(host_dir, force=force)
 
+        logger.info('Creating %s', self.service_conf)
         service_conf = os.path.join(SYSCONFIG_DIR, 'tomcat')
         self.copy(service_conf, self.service_conf, force=force)
 
@@ -421,6 +531,7 @@ class PKIServer(object):
         self.remove_lockout_realm(document)
         self.remove_default_user_database(document)
 
+        logger.info('Creating %s', self.server_xml)
         with open(self.server_xml, 'wb') as f:
             document.write(f, pretty_print=True, encoding='utf-8')
 
@@ -484,20 +595,19 @@ class PKIServer(object):
     def create_libs(self, force=False):
 
         logger.info('Creating %s', self.lib_dir)
-
         lib_dir = os.path.join(PKIServer.SHARE_DIR, 'server', 'lib')
         self.symlink(lib_dir, self.lib_dir, force=force)
 
-        logger.info('Creating %s', self.common_lib_dir)
-
+        logger.info('Creating %s', self.common_dir)
         self.makedirs(self.common_dir, force=force)
 
+        logger.info('Creating %s', self.common_lib_dir)
         common_lib_dir = os.path.join(PKIServer.SHARE_DIR, 'server', 'common', 'lib')
         self.symlink(common_lib_dir, self.common_lib_dir, force=force)
 
     def create_nssdb(self, force=False):
 
-        logger.info('Creating NSS database: %s', self.nssdb_dir)
+        logger.info('Creating %s', self.nssdb_dir)
 
         if force and os.path.exists(self.nssdb_dir):
             logger.warning('NSS database already exists: %s', self.nssdb_dir)
@@ -505,6 +615,9 @@ class PKIServer(object):
 
         self.makedirs(self.nssdb_dir, force=force)
 
+        logger.info('Creating %s', self.nssdb_link)
+        self.symlink(self.nssdb_dir, self.nssdb_link, force=force)
+
         password = self.passwords.get(pki.nssdb.INTERNAL_TOKEN_NAME)
 
         nssdb = pki.nssdb.NSSDatabase(
@@ -562,7 +675,7 @@ class PKIServer(object):
             document = etree.parse(context_xml, parser)
             context = document.getroot()
 
-            doc_base = context.get('docBase', None)
+            doc_base = context.get('docBase')
             webapp['docBase'] = doc_base
 
             webapps.append(webapp)
@@ -612,14 +725,14 @@ class PKIServer(object):
             context = document.getroot()
             context.set('docBase', doc_base)
 
-        # write deployment descriptor
+        logger.info('Creating %s', context_xml)
         with open(context_xml, 'wb') as f:
             # xml as UTF-8 encoded bytes
             document.write(f, pretty_print=True, encoding='utf-8')
 
         # set deployment descriptor ownership and permission
         os.chown(context_xml, self.uid, self.gid)
-        os.chmod(context_xml, 0o0660)
+        os.chmod(context_xml, DEFAULT_FILE_MODE)
 
     def undeploy_webapp(self, webapp_id, force=False):
 
@@ -629,32 +742,54 @@ class PKIServer(object):
             'localhost',
             webapp_id + '.xml')
 
+        logger.info('Removing %s', context_xml)
         pki.util.remove(context_xml, force=force)
 
     def remove(self, force=False):
 
+        logger.info('Removing %s', self.service_conf)
         pki.util.remove(self.service_conf, force=force)
+
+        logger.info('Removing %s', self.log_dir)
         pki.util.rmtree(self.log_dir, force=force)
+
+        logger.info('Removing %s', self.work_dir)
         pki.util.rmtree(self.work_dir, force=force)
+
+        logger.info('Removing %s', self.webapps_dir)
         pki.util.rmtree(self.webapps_dir, force=force)
+
+        logger.info('Removing %s', self.temp_dir)
         pki.util.rmtree(self.temp_dir, force=force)
 
         self.remove_libs(force=force)
 
+        logger.info('Removing %s', self.conf_dir)
         pki.util.rmtree(self.conf_dir, force=force)
+
+        logger.info('Removing %s', self.bin_dir)
         pki.util.unlink(self.bin_dir, force=force)
+
+        logger.info('Removing %s', self.base_dir)
         pki.util.rmtree(self.base_dir, force=force)
 
     def remove_libs(self, force=False):
 
+        logger.info('Removing %s', self.common_lib_dir)
         pki.util.unlink(self.common_lib_dir, force=force)
+
+        logger.info('Removing %s', self.common_dir)
         pki.util.rmtree(self.common_dir, force=force)
+
+        logger.info('Removing %s', self.lib_dir)
         pki.util.unlink(self.lib_dir, force=force)
 
     def remove_nssdb(self, force=False):
 
-        logger.info('Removing NSS database: %s', self.nssdb_dir)
+        logger.info('Removing %s', self.nssdb_link)
+        pki.util.unlink(self.nssdb_link, force=force)
 
+        logger.info('Removing %s', self.nssdb_dir)
         pki.util.rmtree(self.nssdb_dir, force=force)
 
     def load(self):
@@ -663,6 +798,7 @@ class PKIServer(object):
 
         self.load_config()
         self.load_passwords()
+        self.load_subsystems()
 
     def load_config(self):
 
@@ -694,9 +830,25 @@ class PKIServer(object):
             pki.util.load_properties(self.password_conf, self.passwords)
 
     def store_passwords(self):
+        self.store_properties(self.password_conf, self.passwords)
 
-        pki.util.store_properties(self.password_conf, self.passwords)
-        pki.util.chown(self.password_conf, self.uid, self.gid)
+    def load_subsystems(self):
+
+        for subsystem_name in SUBSYSTEM_TYPES:
+
+            subsystem_dir = os.path.join(self.base_dir, subsystem_name)
+            if not os.path.exists(subsystem_dir):
+                continue
+
+            subsystem = pki.server.subsystem.PKISubsystemFactory.create(self, subsystem_name)
+            subsystem.load()
+            self.subsystems[subsystem_name] = subsystem
+
+    def get_subsystems(self):
+        return list(self.subsystems.values())
+
+    def get_subsystem(self, subsystem_name):
+        return self.subsystems.get(subsystem_name)
 
     def load_jss_config(self):
 
@@ -709,9 +861,7 @@ class PKIServer(object):
         return jss_config
 
     def store_jss_config(self, jss_config):
-
-        pki.util.store_properties(self.jss_conf, jss_config)
-        pki.util.chown(self.jss_conf, self.uid, self.gid)
+        self.store_properties(self.jss_conf, jss_config)
 
     def get_server_config(self):
         server_config = ServerConfiguration(self.server_xml)
@@ -791,9 +941,25 @@ class PKIServer(object):
         return subsystem_name, cert_tag
 
     @staticmethod
-    def setup_password_authentication(username, password, subsystem_name='ca', secure_port='8443'):
-        """Return a PKIConnection, logged in using username and password."""
-        connection = pki.client.PKIConnection('https', os.environ['HOSTNAME'], secure_port)
+    def build_ca_files(client_nssdb):
+        if not client_nssdb:
+            return None
+
+        ca_cert = os.path.join(client_nssdb, "ca.crt")
+        if os.path.exists(ca_cert):
+            return ca_cert
+
+        return None
+
+    @staticmethod
+    def setup_password_authentication(username, password, subsystem_name='ca', secure_port='8443',
+                                      client_nssdb=None):
+        """
+        Return a PKIConnection, logged in using username and password.
+        """
+        ca_cert = PKIServer.build_ca_files(client_nssdb)
+        connection = pki.client.PKIConnection('https', socket.getfqdn(), secure_port,
+                                              cert_paths=ca_cert)
         connection.authenticate(username, password)
         account_client = pki.account.AccountClient(connection, subsystem=subsystem_name)
         account_client.login()
@@ -837,10 +1003,6 @@ class PKIServer(object):
         if not client_cert:
             raise PKIServerException('Client cert nickname is required.')
 
-        # Create a PKIConnection object that stores the details of subsystem.
-        connection = pki.client.PKIConnection('https', os.environ['HOSTNAME'], secure_port,
-                                              subsystem_name)
-
         # Create a p12 file using
         # pk12util -o <p12 file name> -n <cert nick name> -d <NSS db path>
         # -W <pkcs12 password> -K <NSS db pass>
@@ -859,8 +1021,19 @@ class PKIServer(object):
             'pkcs12',
             '-in', temp_auth_p12,
             '-out', temp_auth_cert,
-            '-nodes',
+            '-nodes'
+        ]
 
+        # The PEM file containing the CA certificate. Created from a p12 file
+        # using the command:
+        # openssl pkcs12 -in <p12_file_path> -out /tmp/auth.pem -nodes -cacerts -nokeys
+        cmd_generate_ca = [
+            'openssl', 'pkcs12',
+            '-in', temp_auth_p12,
+            '-out', os.path.join(client_nssdb, "ca.crt"),
+            '-nodes',
+            '-cacerts',
+            '-nokeys'
         ]
 
         if client_nssdb_pass_file:
@@ -868,11 +1041,13 @@ class PKIServer(object):
             cmd_generate_pk12.extend(['-k', client_nssdb_pass_file,
                                       '-w', client_nssdb_pass_file])
             cmd_generate_pem.extend(['-passin', 'file:' + client_nssdb_pass_file])
+            cmd_generate_ca.extend(['-passin', 'file:' + client_nssdb_pass_file])
         else:
             # Use the same password for the generated pk12 file
             cmd_generate_pk12.extend(['-K', client_nssdb_pass,
                                       '-W', client_nssdb_pass])
             cmd_generate_pem.extend(['-passin', 'pass:' + client_nssdb_pass])
+            cmd_generate_ca.extend(['-passin', 'pass:' + client_nssdb_pass])
 
         # Generate temp_auth_p12 file
         res_pk12 = subprocess.check_output(cmd_generate_pk12,
@@ -885,6 +1060,24 @@ class PKIServer(object):
                                           stderr=subprocess.STDOUT).decode('utf-8')
         logger.debug('Result of pem generation: %s', res_pem)
 
+        # When we generate the .p12 file, we can extract the ca certificate.
+        # We remove it when it already exists. This ensures we always have
+        # an up-to-date CA certificate.
+        ca_cert = PKIServer.build_ca_files(client_nssdb)
+        if ca_cert and os.path.exists(ca_cert):
+            os.remove(ca_cert)
+
+        # Export the CA each time. This ensures it is always up to date when
+        # trying to connect.
+        res_ca = subprocess.check_output(cmd_generate_ca,
+                                         stderr=subprocess.STDOUT).decode('utf-8')
+        logger.debug('Result of CA generation: %s', res_ca)
+        ca_cert = PKIServer.build_ca_files(client_nssdb)
+
+        # Create a PKIConnection object that stores the details of subsystem.
+        connection = pki.client.PKIConnection('https', socket.getfqdn(), secure_port,
+                                              subsystem_name, cert_paths=ca_cert)
+
         # Bind the authentication with the connection object
         connection.set_authentication_cert(temp_auth_cert)
 
