@@ -179,6 +179,7 @@ class PKIServerCLI(pki.cli.CLI):
                 subsystem_type += ' (Security Domain)'
             print('    Type:                %s' % subsystem_type)
 
+            print('    SD Name:             %s' % ca.config['securitydomain.name'])
             url = 'https://%s:%s' % (
                 ca.config['securitydomain.host'],
                 ca.config['securitydomain.httpsadminport'])
@@ -209,6 +210,7 @@ class PKIServerCLI(pki.cli.CLI):
                 subsystem_type += ' (Standalone)'
             print('    Type:                %s' % subsystem_type)
 
+            print('    SD Name:             %s' % kra.config['securitydomain.name'])
             url = 'https://%s:%s' % (
                 kra.config['securitydomain.host'],
                 kra.config['securitydomain.httpsadminport'])
@@ -235,6 +237,7 @@ class PKIServerCLI(pki.cli.CLI):
                 subsystem_type += ' (Standalone)'
             print('    Type:                %s' % subsystem_type)
 
+            print('    SD Name:             %s' % ocsp.config['securitydomain.name'])
             url = 'https://%s:%s' % (
                 ocsp.config['securitydomain.host'],
                 ocsp.config['securitydomain.httpsadminport'])
@@ -263,6 +266,7 @@ class PKIServerCLI(pki.cli.CLI):
                 subsystem_type += ' Clone'
             print('    Type:                %s' % subsystem_type)
 
+            print('    SD Name:             %s' % tks.config['securitydomain.name'])
             url = 'https://%s:%s' % (
                 tks.config['securitydomain.host'],
                 tks.config['securitydomain.httpsadminport'])
@@ -287,6 +291,7 @@ class PKIServerCLI(pki.cli.CLI):
                 subsystem_type += ' Clone'
             print('    Type:                %s' % subsystem_type)
 
+            print('    SD Name:             %s' % tps.config['securitydomain.name'])
             url = 'https://%s:%s' % (
                 tps.config['securitydomain.host'],
                 tps.config['securitydomain.httpsadminport'])
@@ -313,6 +318,8 @@ class CreateCLI(pki.cli.CLI):
     def print_help(self):
         print('Usage: pki-server create [OPTIONS] [<instance ID>]')
         print()
+        print('      --user <name>             User.')
+        print('      --group <name>            Group.')
         print('      --with-maven-deps         Install Maven dependencies.')
         print('      --force                   Force creation.')
         print('  -v, --verbose                 Run in verbose mode.')
@@ -324,6 +331,7 @@ class CreateCLI(pki.cli.CLI):
 
         try:
             opts, args = getopt.gnu_getopt(argv, 'v', [
+                'user=', 'group=',
                 'with-maven-deps', 'force',
                 'verbose', 'debug', 'help'])
 
@@ -333,11 +341,19 @@ class CreateCLI(pki.cli.CLI):
             sys.exit(1)
 
         instance_name = 'pki-tomcat'
+        user = None
+        group = None
         with_maven_deps = False
         force = False
 
-        for o, _ in opts:
-            if o == '--with-maven-deps':
+        for o, a in opts:
+            if o == '--user':
+                user = a
+
+            elif o == '--group':
+                group = a
+
+            elif o == '--with-maven-deps':
                 with_maven_deps = True
 
             elif o == '--force':
@@ -363,13 +379,20 @@ class CreateCLI(pki.cli.CLI):
 
         instance = pki.server.instance.PKIServerFactory.create(instance_name)
 
-        if not force and instance.is_valid():
+        if not force and instance.exists():
             logger.error('Instance already exists: %s', instance_name)
             sys.exit(1)
 
-        logging.info('Creating instance: %s', instance_name)
+        logger.info('Creating instance: %s', instance_name)
+
+        if user:
+            instance.user = user
+
+        if group:
+            instance.group = group
 
         instance.with_maven_deps = with_maven_deps
+
         instance.create(force=force)
 
 
@@ -426,11 +449,11 @@ class RemoveCLI(pki.cli.CLI):
 
         instance = pki.server.instance.PKIServerFactory.create(instance_name)
 
-        if not force and not instance.is_valid():
+        if not force and not instance.exists():
             logger.error('Invalid instance: %s', instance_name)
             sys.exit(1)
 
-        logging.info('Removing instance: %s', instance_name)
+        logger.info('Removing instance: %s', instance_name)
 
         instance.stop()
         instance.remove(force=force)
@@ -483,7 +506,7 @@ class StatusCLI(pki.cli.CLI):
 
         instance = pki.server.instance.PKIServerFactory.create(instance_name)
 
-        if not instance.is_valid():
+        if not instance.exists():
             logger.error('Invalid instance: %s', instance_name)
             sys.exit(1)
 
@@ -539,7 +562,7 @@ class StartCLI(pki.cli.CLI):
 
         instance = pki.server.instance.PKIServerFactory.create(instance_name)
 
-        if not instance.is_valid():
+        if not instance.exists():
             logger.error('Invalid instance: %s', instance_name)
             sys.exit(1)
 
@@ -597,7 +620,7 @@ class StopCLI(pki.cli.CLI):
 
         instance = pki.server.instance.PKIServerFactory.create(instance_name)
 
-        if not instance.is_valid():
+        if not instance.exists():
             logger.error('Invalid instance: %s', instance_name)
             sys.exit(1)
 
@@ -655,7 +678,7 @@ class RestartCLI(pki.cli.CLI):
 
         instance = pki.server.instance.PKIServerFactory.create(instance_name)
 
-        if not instance.is_valid():
+        if not instance.exists():
             logger.error('Invalid instance: %s', instance_name)
             sys.exit(1)
 
@@ -671,8 +694,10 @@ class RunCLI(pki.cli.CLI):
         print('Usage: pki-server run [OPTIONS] [<instance ID>]')
         print()
         print('      --as-current-user         Run as current user.')
-        print('      --gdb                     Run under the GNU Debugger.')
-        print('      --jdb                     Run under the Java Debugger.')
+        print('      --with-jdb                Run with Java debugger.')
+        print('      --with-gdb                Run with GNU debugger.')
+        print('      --with-valgrind           Run with Valgrind.')
+        print('      --agentpath <value>       Java agent path.')
         print('  -v, --verbose                 Run in verbose mode.')
         print('      --debug                   Run in debug mode.')
         print('      --help                    Show help message.')
@@ -682,7 +707,9 @@ class RunCLI(pki.cli.CLI):
 
         try:
             opts, args = getopt.gnu_getopt(argv, 'v', [
-                'as-current-user', 'jdb',
+                'as-current-user',
+                'with-jdb', 'with-gdb', 'with-valgrind',
+                'agentpath=',
                 'verbose', 'debug', 'help'])
 
         except getopt.GetoptError as e:
@@ -692,18 +719,26 @@ class RunCLI(pki.cli.CLI):
 
         instance_name = 'pki-tomcat'
         as_current_user = False
-        jdb = False
-        gdb = False
+        with_jdb = False
+        with_gdb = False
+        with_valgrind = False
+        agentpath = None
 
-        for o, _ in opts:
+        for o, a in opts:
             if o == '--as-current-user':
                 as_current_user = True
 
-            elif o == '--gdb':
-                gdb = True
+            elif o == '--with-jdb':
+                with_jdb = True
 
-            elif o == '--jdb':
-                jdb = True
+            elif o == '--with-gdb':
+                with_gdb = True
+
+            elif o == '--with-valgrind':
+                with_valgrind = True
+
+            elif o == '--agentpath':
+                agentpath = a
 
             elif o in ('-v', '--verbose'):
                 logging.getLogger().setLevel(logging.INFO)
@@ -725,14 +760,19 @@ class RunCLI(pki.cli.CLI):
 
         instance = pki.server.instance.PKIServerFactory.create(instance_name)
 
-        if not instance.is_valid():
+        if not instance.exists():
             logger.error('Invalid instance: %s', instance_name)
             sys.exit(1)
 
         instance.load()
 
         try:
-            instance.run(gdb=gdb, jdb=jdb, as_current_user=as_current_user)
+            instance.run(
+                as_current_user=as_current_user,
+                with_jdb=with_jdb,
+                with_gdb=with_gdb,
+                with_valgrind=with_valgrind,
+                agentpath=agentpath)
 
         except KeyboardInterrupt:
-            logging.debug('Server stopped')
+            logger.debug('Server stopped')
