@@ -37,6 +37,7 @@ import pki.cli
 import pki.server.cli.audit
 import pki.server.cli.config
 import pki.server.cli.db
+import pki.server.cli.range
 import pki.server.instance
 
 logger = logging.getLogger(__name__)
@@ -60,6 +61,7 @@ class KRACLI(pki.cli.CLI):
         self.add_module(KRACloneCLI())
         self.add_module(pki.server.cli.config.SubsystemConfigCLI(self))
         self.add_module(KRADBCLI(self))
+        self.add_module(pki.server.cli.range.RangeCLI(self))
 
 
 class KRACloneCLI(pki.cli.CLI):
@@ -84,6 +86,7 @@ class KRAClonePrepareCLI(pki.cli.CLI):
         print('      --pkcs12-file <path>           PKCS #12 file to store certificates and keys.')
         print('      --pkcs12-password <password>   Password for the PKCS #12 file.')
         print('      --pkcs12-password-file <path>  File containing the PKCS #12 password.')
+        print('      --no-key                       Do not include private key.')
         print('  -v, --verbose                      Run in verbose mode.')
         print('      --debug                        Run in debug mode.')
         print('      --help                         Show help message.')
@@ -94,6 +97,7 @@ class KRAClonePrepareCLI(pki.cli.CLI):
         try:
             opts, _ = getopt.gnu_getopt(argv, 'i:v', [
                 'instance=', 'pkcs12-file=', 'pkcs12-password=', 'pkcs12-password-file=',
+                'no-key',
                 'verbose', 'debug', 'help'])
 
         except getopt.GetoptError as e:
@@ -104,6 +108,7 @@ class KRAClonePrepareCLI(pki.cli.CLI):
         instance_name = 'pki-tomcat'
         pkcs12_file = None
         pkcs12_password = None
+        no_key = False
 
         for o, a in opts:
             if o in ('-i', '--instance'):
@@ -119,6 +124,9 @@ class KRAClonePrepareCLI(pki.cli.CLI):
                 with io.open(a, 'rb') as f:
                     pkcs12_password = f.read()
 
+            elif o == '--no-key':
+                no_key = True
+
             elif o == '--debug':
                 logging.getLogger().setLevel(logging.DEBUG)
 
@@ -145,7 +153,7 @@ class KRAClonePrepareCLI(pki.cli.CLI):
             sys.exit(1)
 
         instance = pki.server.instance.PKIInstance(instance_name)
-        if not instance.is_valid():
+        if not instance.exists():
             logger.error('Invalid instance %s.', instance_name)
             sys.exit(1)
         instance.load()
@@ -163,13 +171,13 @@ class KRAClonePrepareCLI(pki.cli.CLI):
                 f.write(pkcs12_password)
 
             subsystem.export_system_cert(
-                'subsystem', pkcs12_file, pkcs12_password_file)
+                'subsystem', pkcs12_file, pkcs12_password_file, no_key=no_key)
             subsystem.export_system_cert(
-                'transport', pkcs12_file, pkcs12_password_file, append=True)
+                'transport', pkcs12_file, pkcs12_password_file, no_key=no_key, append=True)
             subsystem.export_system_cert(
-                'storage', pkcs12_file, pkcs12_password_file, append=True)
+                'storage', pkcs12_file, pkcs12_password_file, no_key=no_key, append=True)
             subsystem.export_system_cert(
-                'audit_signing', pkcs12_file, pkcs12_password_file, append=True)
+                'audit_signing', pkcs12_file, pkcs12_password_file, no_key=no_key, append=True)
             instance.export_external_certs(
                 pkcs12_file, pkcs12_password_file, append=True)
 
@@ -265,7 +273,7 @@ class KRADBVLVFindCLI(pki.cli.CLI):
                 sys.exit(1)
 
         instance = pki.server.instance.PKIInstance(instance_name)
-        if not instance.is_valid():
+        if not instance.exists():
             logger.error('Invalid instance %s.', instance_name)
             sys.exit(1)
         instance.load()
@@ -383,7 +391,7 @@ class KRADBVLVAddCLI(pki.cli.CLI):
                 sys.exit(1)
 
         instance = pki.server.instance.PKIInstance(instance_name)
-        if not instance.is_valid():
+        if not instance.exists():
             logger.error('Invalid instance %s.', instance_name)
             sys.exit(1)
         instance.load()
@@ -494,7 +502,7 @@ class KRADBVLVDeleteCLI(pki.cli.CLI):
 
         instance = pki.server.instance.PKIInstance(instance_name)
 
-        if not instance.is_valid():
+        if not instance.exists():
             logger.error('Invalid instance %s.', instance_name)
             sys.exit(1)
 
@@ -619,7 +627,7 @@ class KRADBVLVReindexCLI(pki.cli.CLI):
                 sys.exit(1)
 
         instance = pki.server.instance.PKIInstance(instance_name)
-        if not instance.is_valid():
+        if not instance.exists():
             logger.error('Invalid instance %s.', instance_name)
             sys.exit(1)
         instance.load()
