@@ -30,6 +30,7 @@ import sys
 from lxml import etree
 
 import pki.cli
+import pki.nssdb
 import pki.server.instance
 import pki.util
 
@@ -100,7 +101,7 @@ class MigrateCLI(pki.cli.CLI):
 
             instance = pki.server.instance.PKIInstance(instance_name)
 
-            if not instance.is_valid():
+            if not instance.exists():
                 logger.error('Invalid instance %s.', instance_name)
                 sys.exit(1)
 
@@ -118,8 +119,7 @@ class MigrateCLI(pki.cli.CLI):
         self.migrate_nssdb(instance)
         self.migrate_tomcat(instance, tomcat_version)
         self.migrate_subsystems(instance, tomcat_version)
-
-        self.print_message('%s instance migrated' % instance.name)
+        self.migrate_service(instance)
 
     def migrate_nssdb(self, instance):
 
@@ -137,6 +137,20 @@ class MigrateCLI(pki.cli.CLI):
         finally:
             nssdb.close()
 
+        ca_path = os.path.join(instance.nssdb_dir, 'ca.crt')
+        token = pki.nssdb.INTERNAL_TOKEN_NAME
+        nickname = instance.get_sslserver_cert_nickname()
+        if ':' in nickname:
+            token = nickname.split(':', 1)[0]
+
+        # Re-open NSS DB with correct token name
+        nssdb = instance.open_nssdb(token=token)
+
+        try:
+            nssdb.extract_ca_cert(ca_path, nickname)
+        finally:
+            nssdb.close()
+
     def migrate_tomcat(self, instance, tomcat_version):
 
         logger.info('Migrating %s instance to Tomcat %s',
@@ -164,7 +178,10 @@ class MigrateCLI(pki.cli.CLI):
 
         document = etree.parse(filename, self.parser)
 
-        if tomcat_version >= pki.util.Version('8.5.0'):
+        if tomcat_version >= pki.util.Version('9.0.31'):
+            self.migrate_server_xml_to_tomcat9031(instance, document)
+
+        elif tomcat_version >= pki.util.Version('8.5.0'):
             self.migrate_server_xml_to_tomcat85(instance, document)
 
         elif tomcat_version >= pki.util.Version('8.0.0'):
@@ -483,8 +500,44 @@ class MigrateCLI(pki.cli.CLI):
             full_name = instance.get_sslserver_cert_nickname()
             certificate.set('certificateKeyAlias', full_name)
 
+    def migrate_server_xml_to_tomcat9031(self, instance, document):
+
+        self.migrate_server_xml_to_tomcat85(instance, document)
+
+        server = document.getroot()
+
+        # Migrate requiredSecret -> secret on AJP connectors
+
+        services = server.findall('Service')
+        for service in services:
+
+            children = list(service)
+            for child in children:
+                if isinstance(child, etree._Comment):  # pylint: disable=protected-access
+                    if 'protocol="AJP/1.3"' in child.text:
+                        child.text = re.sub(r'requiredSecret=',
+                                            r'secret=',
+                                            child.text,
+                                            flags=re.MULTILINE)
+
+        connectors = server.findall('Service/Connector')
+        for connector in connectors:
+            if connector.get('protocol') != 'AJP/1.3':
+                # Only modify AJP connectors.
+                continue
+            if connector.get('secret'):
+                # Nothing to migrate because the secret attribute already
+                # exists.
+                continue
+            if connector.get('requiredSecret') is None:
+                # No requiredSecret field either; nothing to do.
+                continue
+
+            connector.set('secret', connector.get('requiredSecret'))
+            connector.attrib.pop('requiredSecret', None)
+
     def migrate_subsystems(self, instance, tomcat_version):
-        for subsystem in instance.subsystems:
+        for subsystem in instance.get_subsystems():
             self.migrate_subsystem(subsystem, tomcat_version)
 
     def migrate_subsystem(self, subsystem, tomcat_version):
@@ -549,3 +602,45 @@ class MigrateCLI(pki.cli.CLI):
 
         os.symlink(source, dest)
         os.lchown(dest, instance.uid, instance.gid)
+
+    def migrate_service(self, instance):
+        self.migrate_service_java_home(instance)
+
+    def migrate_service_java_home(self, instance):
+        # When JAVA_HOME in the Tomcat service config differs from the
+        # value in /usr/share/pki/etc/pki.conf, update the value in
+        # the service config.
+
+        if "JAVA_HOME" not in os.environ or not os.environ["JAVA_HOME"]:
+            logger.debug("Refusing to migrate JAVA_HOME with missing environment variable")
+            return
+
+        java_home = os.environ['JAVA_HOME']
+
+        # Update in /etc/sysconfig/<instance>
+        result = self.update_java_home_in_config(instance.service_conf, java_home)
+        self.write_config(instance.service_conf, result)
+
+        # Update in /etc/pki/<instance>/tomcat.conf
+        result = self.update_java_home_in_config(instance.tomcat_conf, java_home)
+        self.write_config(instance.tomcat_conf, result)
+
+    def update_java_home_in_config(self, path, java_home):
+        result = []
+
+        target = "JAVA_HOME="
+
+        with open(path, 'r') as conf_fp:
+            for line in conf_fp:
+                if not line.startswith(target):
+                    result.append(line)
+                else:
+                    new_line = target + '"' + java_home + '"\n'
+                    result.append(new_line)
+
+        return result
+
+    def write_config(self, path, output):
+        with open(path, 'w') as conf_fp:
+            for line in output:
+                print(line, end='', file=conf_fp)
