@@ -72,6 +72,7 @@ class TKSClonePrepareCLI(pki.cli.CLI):
         print('      --pkcs12-file <path>           PKCS #12 file to store certificates and keys.')
         print('      --pkcs12-password <password>   Password for the PKCS #12 file.')
         print('      --pkcs12-password-file <path>  File containing the PKCS #12 password.')
+        print('      --no-key                       Do not include private key.')
         print('  -v, --verbose                      Run in verbose mode.')
         print('      --debug                        Run in debug mode.')
         print('      --help                         Show help message.')
@@ -82,6 +83,7 @@ class TKSClonePrepareCLI(pki.cli.CLI):
         try:
             opts, _ = getopt.gnu_getopt(argv, 'i:v', [
                 'instance=', 'pkcs12-file=', 'pkcs12-password=', 'pkcs12-password-file=',
+                'no-key',
                 'verbose', 'debug', 'help'])
 
         except getopt.GetoptError as e:
@@ -92,6 +94,7 @@ class TKSClonePrepareCLI(pki.cli.CLI):
         instance_name = 'pki-tomcat'
         pkcs12_file = None
         pkcs12_password = None
+        no_key = False
 
         for o, a in opts:
             if o in ('-i', '--instance'):
@@ -107,6 +110,9 @@ class TKSClonePrepareCLI(pki.cli.CLI):
                 with io.open(a, 'rb') as f:
                     pkcs12_password = f.read()
 
+            elif o == '--no-key':
+                no_key = True
+
             elif o == '--debug':
                 logging.getLogger().setLevel(logging.DEBUG)
 
@@ -133,7 +139,7 @@ class TKSClonePrepareCLI(pki.cli.CLI):
             sys.exit(1)
 
         instance = pki.server.instance.PKIInstance(instance_name)
-        if not instance.is_valid():
+        if not instance.exists():
             logger.error('Invalid instance %s.', instance_name)
             sys.exit(1)
 
@@ -152,9 +158,9 @@ class TKSClonePrepareCLI(pki.cli.CLI):
                 f.write(pkcs12_password)
 
             subsystem.export_system_cert(
-                'subsystem', pkcs12_file, pkcs12_password_file)
+                'subsystem', pkcs12_file, pkcs12_password_file, no_key=no_key)
             subsystem.export_system_cert(
-                'audit_signing', pkcs12_file, pkcs12_password_file, append=True)
+                'audit_signing', pkcs12_file, pkcs12_password_file, no_key=no_key, append=True)
             instance.export_external_certs(
                 pkcs12_file, pkcs12_password_file, append=True)
 
