@@ -245,11 +245,18 @@ class PKIDeployer:
 
         logger.info('Connecting to security domain at %s', sd_url)
 
+        ca_cert = os.path.join(self.mdict['pki_server_database_path'],
+                               "ca.crt")
+        if not os.path.exists(ca_cert):
+            if os.path.exists(self.mdict['pki_cert_chain_path']):
+                ca_cert = self.mdict['pki_cert_chain_path']
+
         self.sd_connection = pki.client.PKIConnection(
             protocol='https',
             hostname=sd_hostname,
             port=sd_port,
-            trust_env=False)
+            trust_env=False,
+            cert_paths=ca_cert)
 
         return self.sd_connection
 
@@ -289,7 +296,7 @@ class PKIDeployer:
         hostname = self.mdict['pki_hostname']
         subsystem = self.mdict['pki_subsystem']
 
-        logger.info('Getting token for installing %s on %s', subsystem, hostname)
+        logger.info('Getting install token')
 
         sd_client = pki.system.SecurityDomainClient(self.sd_connection)
         self.install_token = sd_client.get_install_token(hostname, subsystem)
@@ -316,6 +323,56 @@ class PKIDeployer:
 
         self.get_install_token()
 
+    def setup_cert(self, client, tag):
+
+        request = pki.system.CertificateSetupRequest()
+
+        request.pin = self.mdict['pki_one_time_pin']
+        request.installToken = self.install_token
+
+        # Process existing CA installation like external CA
+        request.external = config.str2bool(self.mdict['pki_external']) or \
+            config.str2bool(self.mdict['pki_existing'])
+        request.standAlone = config.str2bool(self.mdict['pki_standalone'])
+        request.clone = config.str2bool(self.mdict['pki_clone'])
+        request.masterURL = self.mdict['pki_clone_uri']
+
+        request.tag = tag
+        self.config_client.set_system_cert_info(request, tag)
+
+        if tag != 'sslserver' and request.clone:
+            logger.info('%s certificate is already set up', tag)
+            return None
+
+        system_list = self.instance.tomcat_instance_subsystems()
+
+        if tag == 'sslserver' and len(system_list) > 1:
+            logger.info('sslserver certificate is already set up')
+            return None
+
+        if tag == 'subsystem' and len(system_list) > 1:
+            logger.info('subsystem certificate is already set up')
+            return None
+
+        return client.setupCert(request)
+
+    def setup_admin(self, client):
+
+        request = pki.system.AdminSetupRequest()
+        request.pin = self.mdict['pki_one_time_pin']
+        request.installToken = self.install_token
+
+        self.config_client.set_admin_parameters(request)
+
+        response = client.setupAdmin(request)
+
+        if config.str2bool(self.mdict['pki_external']) \
+                or config.str2bool(self.mdict['pki_standalone']) \
+                or not config.str2bool(self.mdict['pki_import_admin_cert']):
+
+            admin_cert = response['adminCert']['cert']
+            self.config_client.process_admin_cert(admin_cert)
+
     def backup_keys(self, instance, subsystem):
 
         tmpdir = tempfile.mkdtemp()
@@ -366,8 +423,7 @@ class PKIDeployer:
         ]
 
         logger.debug('Command: %s', ' '.join(cmd))
-        # TODO: Replace stdout/stderr with capture_output in Python 3.7.
-        result = subprocess.run(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, check=False)
+        result = subprocess.run(cmd, stdout=subprocess.PIPE, check=False)
 
         if result.returncode == 0:
             return json.loads(result.stdout.decode())
@@ -401,8 +457,7 @@ class PKIDeployer:
         ]
 
         logger.debug('Command: %s', ' '.join(cmd))
-        # TODO: Replace stdout/stderr with capture_output in Python 3.7.
-        result = subprocess.run(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, check=False)
+        result = subprocess.run(cmd, stdout=subprocess.PIPE, check=False)
 
         if result.returncode == 0:
             return json.loads(result.stdout.decode())
@@ -430,8 +485,7 @@ class PKIDeployer:
         ]
 
         logger.debug('Command: %s', ' '.join(cmd))
-        # TODO: Replace stdout/stderr with capture_output in Python 3.7.
-        result = subprocess.run(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, check=False)
+        result = subprocess.run(cmd, stdout=subprocess.PIPE, check=False)
 
         if result.returncode == 0:
             return json.loads(result.stdout.decode())
@@ -460,8 +514,7 @@ class PKIDeployer:
         ]
 
         logger.debug('Command: %s', ' '.join(cmd))
-        # TODO: Replace stdout/stderr with capture_output in Python 3.7.
-        result = subprocess.run(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, check=False)
+        result = subprocess.run(cmd, stdout=subprocess.PIPE, check=False)
 
         if result.returncode == 0:
             return json.loads(result.stdout.decode())
@@ -490,8 +543,7 @@ class PKIDeployer:
         ]
 
         logger.debug('Command: %s', ' '.join(cmd))
-        # TODO: Replace stdout/stderr with capture_output in Python 3.7.
-        result = subprocess.run(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, check=False)
+        result = subprocess.run(cmd, stdout=subprocess.PIPE, check=False)
 
         if result.returncode == 0:
             return json.loads(result.stdout.decode())
