@@ -26,11 +26,9 @@ import getpass
 import ldap
 import logging
 import os
-import random
-import string
 import xml.etree.ElementTree as ET
 
-from six.moves import input, range  # pylint: disable=W0622,F0401
+from six.moves import input  # pylint: disable=W0622,F0401
 from six.moves import configparser  # pylint: disable=F0401
 from six.moves.urllib.parse import urlparse  # pylint: disable=F0401,E0611
 
@@ -179,9 +177,13 @@ class PKIConfigParser:
             dest='help', action='help',
             help='show this help message and exit')
         self.optional.add_argument(
-            '-v',
+            '-v', '--verbose',
             dest='pki_verbosity', action='count', default=0,
-            help='display verbose information (details below)')
+            help='Run in verbose mode')
+        self.optional.add_argument(
+            '--debug',
+            dest='debug', action='store_true',
+            help='Run in debug mode')
 
         self.indent = 0
         self.authdb_connection = None
@@ -190,23 +192,18 @@ class PKIConfigParser:
         self.mdict = deployer.mdict
         self.slots_dict = deployer.slots
 
-    # PKI Deployment Helper Functions
     def process_command_line_arguments(self):
 
-        # Parse command-line options
         args = self.arg_parser.parse_args()
 
-        # Process 'Mandatory' command-line options
+        if args.debug:
+            logging.getLogger().setLevel(logging.DEBUG)
+
+        elif args.pki_verbosity == 1:
+            logging.getLogger().setLevel(logging.INFO)
 
-        # Process 'Optional' command-line options
-        #    '-v'
-        if args.pki_verbosity == 1:
-            config.pki_log_level = logging.INFO
         elif args.pki_verbosity >= 2:
-            config.pki_log_level = logging.DEBUG
-        else:
-            # Set default log levels
-            config.pki_log_level = logging.WARNING
+            logging.getLogger().setLevel(logging.DEBUG)
 
         return args
 
@@ -285,7 +282,8 @@ class PKIConfigParser:
             'nss_default_db_type': self.deployer.nss_db_type,
             'java_home': java_home,
             'home_dir': os.path.expanduser("~"),
-            'pki_hostname': self.deployer.hostname})
+            'pki_hostname': self.deployer.hostname,
+            'pki_random_ajp_secret': pki.generate_password()})
 
         # Make keys case-sensitive!
         self.deployer.main_config.optionxform = str
@@ -535,11 +533,15 @@ class PKIConfigParser:
 
     def get_server_status(self, system_type, system_uri):
         parse = urlparse(self.mdict[system_uri])
+        # Because this is utilized exclusively during pkispawn, we can safely
+        # ignore validating the certificate; it might not yet have been
+        # configured anyways.
         conn = pki.client.PKIConnection(
             protocol=parse.scheme,
             hostname=parse.hostname,
             port=str(parse.port),
-            trust_env=False)
+            trust_env=False,
+            verify=False)
         client = pki.system.SystemStatusClient(conn, subsystem=system_type)
         response = client.get_status()
         root = ET.fromstring(response)
@@ -614,36 +616,6 @@ class PKIConfigParser:
                     self.mdict['pki_subsystem_configuration_path'],
                     "CS.cfg")
 
-            if config.str2bool(self.mdict['pki_external_step_two']) or\
-               config.str2bool(self.mdict['pki_skip_installation']):
-                # For CA (External CA Step 2) and Stand-alone PKI (Step 2),
-                # use the 'pki_one_time_pin' established during the setup
-                # of (Step 1)
-                #
-                # Similarly, if the only code being processed is for
-                # configuration, re-use the 'pki_one_time_pin' generated
-                # during the installation phase
-                #
-                if os.path.exists(self.mdict['pki_target_cs_cfg'])\
-                   and\
-                   os.path.isfile(self.mdict['pki_target_cs_cfg']):
-                    cs_cfg = self.read_simple_configuration_file(
-                        self.mdict['pki_target_cs_cfg'])
-                    self.mdict['pki_one_time_pin'] = \
-                        cs_cfg.get('preop.pin')
-                else:
-                    logger.error(
-                        log.PKI_FILE_MISSING_OR_NOT_A_FILE_1,
-                        self.mdict['pki_target_cs_cfg'])
-                    raise Exception(log.PKI_FILE_MISSING_OR_NOT_A_FILE_1)
-            else:
-                # Generate a one-time pin to be used prior to configuration
-                # and add this to the "sensitive" key value pairs read in from
-                # the configuration file
-                self.mdict['pki_one_time_pin'] = \
-                    ''.join(random.choice(string.ascii_letters + string.digits)
-                            for x in range(20))
-
             self.mdict['pki_target_server_xml'] = \
                 os.path.join(
                     self.mdict['pki_instance_configuration_path'],
@@ -670,6 +642,12 @@ class PKIConfigParser:
                     self.mdict['pki_tomcat_webapps_subsystem_path'],
                     "WEB-INF",
                     "web.xml.orig")
+
+            self.mdict['pki_target_registry_cfg'] = \
+                os.path.join(
+                    self.mdict['pki_subsystem_configuration_path'],
+                    "registry.cfg")
+
             # subystem-specific slot substitution name/value pairs
             if self.mdict['pki_subsystem'] == "CA":
                 self.mdict['pki_target_flatfile_txt'] = \
@@ -680,10 +658,6 @@ class PKIConfigParser:
                     os.path.join(
                         self.mdict['pki_subsystem_configuration_path'],
                         "proxy.conf")
-                self.mdict['pki_target_registry_cfg'] = \
-                    os.path.join(
-                        self.mdict['pki_subsystem_configuration_path'],
-                        "registry.cfg")
                 # '*.profile'
                 self.mdict['pki_target_admincert_profile'] = \
                     os.path.join(
@@ -744,6 +718,8 @@ class PKIConfigParser:
                 self.mdict['pki_ajp_port']
             self.mdict['PKI_AJP_HOST_SLOT'] = \
                 self.mdict['pki_ajp_host']
+            self.mdict['PKI_AJP_SECRET_SLOT'] = \
+                self.mdict['pki_ajp_secret']
             self.mdict['PKI_AJP_REDIRECT_PORT_SLOT'] = \
                 self.mdict['pki_https_port']
             self.mdict['PKI_CA_HOSTNAME_SLOT'] = \
@@ -831,8 +807,6 @@ class PKIConfigParser:
                     "<!--"
             self.mdict['PKI_TMPDIR_SLOT'] = \
                 self.mdict['pki_tomcat_tmpdir_path']
-            self.mdict['PKI_RANDOM_NUMBER_SLOT'] = \
-                self.mdict['pki_one_time_pin']
             self.mdict['PKI_SECURE_PORT_SLOT'] = \
                 self.mdict['pki_https_port']
             self.mdict['PKI_SECURE_PORT_CONNECTOR_NAME_SLOT'] = \
@@ -860,8 +834,6 @@ class PKIConfigParser:
                 "<!-- Shared Ports:  Unsecure Port Connector -->"
             self.mdict['PKI_USER_SLOT'] = \
                 self.mdict['pki_user']
-            self.mdict['PKI_WEB_SERVER_TYPE_SLOT'] = \
-                "tomcat"
             self.mdict['PKI_WEBAPPS_NAME_SLOT'] = \
                 "webapps"
 
@@ -1087,7 +1059,6 @@ class PKIConfigParser:
 
                 # Stand-alone PKI
                 self.mdict['pki_security_domain_type'] = "new"
-                self.mdict['pki_issuing_ca'] = "External CA"
 
             elif self.deployer.subsystem_name != "CA" or \
                     config.str2bool(self.mdict['pki_clone']) or \
@@ -1103,7 +1074,6 @@ class PKIConfigParser:
 
                 # External CA
                 self.mdict['pki_security_domain_type'] = "new"
-                self.mdict['pki_issuing_ca'] = "External CA"
 
             else:
                 # PKI CA (master) and Subordinate CA (new)
