@@ -25,6 +25,7 @@ from cryptography.x509.oid import NameOID
 import logging
 import os
 import shutil
+import subprocess
 import tempfile
 import urllib.parse
 
@@ -432,7 +433,7 @@ class PkiScriptlet(pkiscriptlet.AbstractBasePkiScriptlet):
 
             logger.info('Creating temp SSL server cert for %s', deployer.mdict['pki_hostname'])
 
-            # TODO: replace with pki-server create-cert sslserver --temp
+            # TODO: replace with pki-server create-cert --temp sslserver
 
             # NOTE:  ALWAYS create the temporary sslserver certificate
             #        in the software DB regardless of whether the
@@ -526,6 +527,23 @@ class PkiScriptlet(pkiscriptlet.AbstractBasePkiScriptlet):
             nssdb.close()
             shutil.rmtree(tmpdir)
 
+    def get_cert_chain(self, instance, url):
+
+        cmd = [
+            'pki',
+            '-d', instance.nssdb_dir,
+            '-f', instance.password_conf,
+            '-U', url,
+            '--ignore-cert-status', 'UNTRUSTED_ISSUER',
+            'ca-cert-signing-export',
+            '--pkcs7'
+        ]
+
+        logger.debug('Command: %s', ' '.join(cmd))
+        output = subprocess.check_output(cmd)
+
+        return output.decode()
+
     def spawn(self, deployer):
 
         external = deployer.configuration_file.external
@@ -547,9 +565,10 @@ class PkiScriptlet(pkiscriptlet.AbstractBasePkiScriptlet):
         if PKISPAWN_STARTUP_TIMEOUT_SECONDS <= 0:
             PKISPAWN_STARTUP_TIMEOUT_SECONDS = 60
 
-        instance = pki.server.instance.PKIInstance(deployer.mdict['pki_instance_name'])
+        instance = self.instance
         instance.load()
 
+        subsystems = instance.get_subsystems()
         subsystem = instance.get_subsystem(deployer.mdict['pki_subsystem'].lower())
 
         # configure internal database
@@ -622,6 +641,7 @@ class PkiScriptlet(pkiscriptlet.AbstractBasePkiScriptlet):
         existing = deployer.configuration_file.existing
         step_two = deployer.configuration_file.external_step_two
         clone = deployer.configuration_file.clone
+        master_url = deployer.mdict['pki_clone_uri']
 
         try:
             if existing or (external or standalone) and step_two:
@@ -635,6 +655,35 @@ class PkiScriptlet(pkiscriptlet.AbstractBasePkiScriptlet):
 
                 self.validate_system_certs(deployer, nssdb, subsystem)
 
+            elif len(subsystems) > 1:
+
+                for s in subsystems:
+
+                    # find a subsystem that is already installed
+                    if s.name == subsystem.name:
+                        continue
+
+                    # import cert/request data from the existing subsystem
+                    # into the new subsystem being installed
+
+                    logger.info('Importing sslserver cert data from %s', s.type)
+                    subsystem.config['%s.sslserver.cert' % subsystem.name] = \
+                        s.config['%s.sslserver.cert' % s.name]
+
+                    logger.info('Importing subsystem cert data from %s', s.type)
+                    subsystem.config['%s.subsystem.cert' % subsystem.name] = \
+                        s.config['%s.subsystem.cert' % s.name]
+
+                    logger.info('Importing sslserver request data from %s', s.type)
+                    subsystem.config['%s.sslserver.certreq' % subsystem.name] = \
+                        s.config['%s.sslserver.certreq' % s.name]
+
+                    logger.info('Importing subsystem request data from %s', s.type)
+                    subsystem.config['%s.subsystem.certreq' % subsystem.name] = \
+                        s.config['%s.subsystem.certreq' % s.name]
+
+                    break
+
             else:  # self-signed CA
 
                 # To be implemented in ticket #1692.
@@ -704,39 +753,63 @@ class PkiScriptlet(pkiscriptlet.AbstractBasePkiScriptlet):
             subsystem.config['securitydomain.httpsadminport'] = securePort
 
         subsystem.config['service.securityDomainPort'] = securePort
-        subsystem.config['securitydomain.store'] = 'ldap'
 
         hierarchy = subsystem.config.get('hierarchy.select')
+        issuing_ca = deployer.mdict['pki_issuing_ca']
+
         if not (subsystem.type == 'CA' and hierarchy == 'Root'):
 
-            issuing_ca = deployer.mdict['pki_issuing_ca']
-
-            if issuing_ca == 'External CA':
-
-                logger.info('Using external CA')
-
-                subsystem.config['preop.ca.type'] = 'otherca'
-                subsystem.config['preop.ca.pkcs7'] = ''
-                subsystem.config['preop.ca.certchain.size'] = '0'
-
-                if subsystem.type == 'CA':
-                    subsystem.config['preop.cert.signing.type'] = 'remote'
-
-            else:
+            if not external and not standalone:
 
                 logger.info('Using CA at %s', issuing_ca)
 
                 url = urllib.parse.urlparse(issuing_ca)
 
                 subsystem.config['preop.ca.url'] = issuing_ca
-                subsystem.config['preop.ca.type'] = 'sdca'
                 subsystem.config['preop.ca.hostname'] = url.hostname
                 subsystem.config['preop.ca.httpsport'] = str(url.port)
                 subsystem.config['preop.ca.httpsadminport'] = str(url.port)
 
-                if subsystem.type == 'CA':
-                    subsystem.config['preop.cert.signing.type'] = 'remote'
-                    subsystem.config['preop.cert.signing.profile'] = 'caInstallCACert'
+        system_certs_imported = \
+            deployer.mdict['pki_server_pkcs12_path'] != '' or \
+            deployer.mdict['pki_clone_pkcs12_path'] != ''
+
+        if not (subsystem.type == 'CA' and hierarchy == 'Root'):
+
+            if external or standalone:
+                subsystem.config['preop.ca.pkcs7'] = ''
+
+            elif not clone and not system_certs_imported:
+
+                logger.info('Retrieving CA certificate chain from %s', issuing_ca)
+
+                pem_chain = self.get_cert_chain(instance, issuing_ca)
+                base64_chain = pki.nssdb.convert_pkcs7(pem_chain, 'pem', 'base64')
+                subsystem.config['preop.ca.pkcs7'] = base64_chain
+
+                logger.info('Importing CA certificate chain')
+
+                nssdb = instance.open_nssdb()
+                try:
+                    nssdb.import_pkcs7(pkcs7_data=pem_chain, trust_attributes='CT,C,C')
+                finally:
+                    nssdb.close()
+
+        if subsystem.type == 'CA' and clone and not system_certs_imported:
+
+            logger.info('Retrieving CA certificate chain from %s', master_url)
+
+            pem_chain = self.get_cert_chain(instance, master_url)
+            base64_chain = pki.nssdb.convert_pkcs7(pem_chain, 'pem', 'base64')
+            subsystem.config['preop.clone.pkcs7'] = base64_chain
+
+            logger.info('Importing CA certificate chain')
+
+            nssdb = instance.open_nssdb()
+            try:
+                nssdb.import_pkcs7(pkcs7_data=pem_chain, trust_attributes='CT,C,C')
+            finally:
+                nssdb.close()
 
         subsystem.save()
 
@@ -803,6 +876,16 @@ class PkiScriptlet(pkiscriptlet.AbstractBasePkiScriptlet):
             setup_db_manager=setup_db_manager,
             setup_vlv_indexes=setup_vlv_indexes)
 
+        if clone:
+
+            if subsystem.type in ['CA', 'KRA']:
+
+                logger.info('Updating ranges for %s clone', subsystem.type)
+                subsystem.update_ranges(master_url, deployer.install_token)
+
+            logger.info('Updating configuration for %s clone', subsystem.type)
+            subsystem.update_config(master_url, deployer.install_token)
+
         # Start/Restart this Tomcat PKI Process
         # Optionally prepare to enable a java debugger
         # (e. g. - 'eclipse'):
@@ -840,20 +923,24 @@ class PkiScriptlet(pkiscriptlet.AbstractBasePkiScriptlet):
         if config.str2bool(deployer.mdict['pki_enable_java_debugger']):
             config.wait_to_attach_an_external_java_debugger()
 
+        ca_cert = os.path.join(instance.nssdb_dir, "ca.crt")
+
         connection = pki.client.PKIConnection(
             protocol='https',
             hostname=deployer.mdict['pki_hostname'],
             port=deployer.mdict['pki_https_port'],
-            trust_env=False)
+            trust_env=False,
+            cert_paths=ca_cert)
 
         client = pki.system.SystemConfigClient(
             connection,
             subsystem=deployer.mdict['pki_subsystem_type'])
 
-        logger.info('Configuring %s subsystem', subsystem.type)
+        # If pki_one_time_pin is not already defined, load from CS.cfg
+        if 'pki_one_time_pin' not in deployer.mdict:
+            deployer.mdict['pki_one_time_pin'] = subsystem.config['preop.pin']
 
-        request = deployer.config_client.create_config_request()
-        client.configure(request)
+        logger.info('Configuring %s subsystem', subsystem.type)
 
         if clone:
 
@@ -874,10 +961,7 @@ class PkiScriptlet(pkiscriptlet.AbstractBasePkiScriptlet):
         for tag in subsystem.config['preop.cert.list'].split(','):
 
             logger.info('Setting up %s certificate', tag)
-
-            cert_setup_request = deployer.config_client.create_certificate_setup_request(tag)
-            cert_setup_request.installToken = deployer.install_token
-            cert = client.setupCert(cert_setup_request)
+            cert = deployer.setup_cert(client, tag)
 
             if not cert:
                 continue
@@ -893,15 +977,7 @@ class PkiScriptlet(pkiscriptlet.AbstractBasePkiScriptlet):
         if not clone:
 
             logger.info('Setting up admin user')
-
-            admin_setup_request = deployer.config_client.create_admin_setup_request()
-            admin_setup_request.installToken = deployer.install_token
-            admin_setup_response = client.setupAdmin(admin_setup_request)
-
-            if external or standalone \
-                    or not config.str2bool(deployer.mdict['pki_import_admin_cert']):
-                admin_cert = admin_setup_response['adminCert']['cert']
-                deployer.config_client.process_admin_cert(admin_cert)
+            deployer.setup_admin(client)
 
         if config.str2bool(deployer.mdict['pki_backup_keys']):
 
