@@ -22,6 +22,7 @@ from __future__ import absolute_import
 from __future__ import print_function
 import logging
 import os
+import subprocess
 
 import pki.nssdb
 import pki.pkcs12
@@ -45,11 +46,15 @@ class PkiScriptlet(pkiscriptlet.AbstractBasePkiScriptlet):
             logger.info('Skipping NSS database creation')
             return
 
-        logger.info('Creating NSS database')
-
-        instance = pki.server.instance.PKIInstance(deployer.mdict['pki_instance_name'])
+        instance = self.instance
         instance.load()
 
+        usePSSForRSASigningAlg = \
+            deployer.mdict['pki_use_pss_rsa_signing_algorithm']
+
+        if usePSSForRSASigningAlg == "True":
+            self.set_signing_algs_for_pss(deployer)
+
         subsystem = instance.get_subsystem(deployer.mdict['pki_subsystem'].lower())
 
         if config.str2bool(deployer.mdict['pki_hsm_enable']):
@@ -68,6 +73,7 @@ class PkiScriptlet(pkiscriptlet.AbstractBasePkiScriptlet):
         deployer.file.modify(deployer.mdict['pki_shared_password_conf'])
 
         if not os.path.isdir(deployer.mdict['pki_server_database_path']):
+            logger.info('Creating %s', deployer.mdict['pki_server_database_path'])
             instance.makedirs(deployer.mdict['pki_server_database_path'], force=True)
 
         nssdb = pki.nssdb.NSSDatabase(
@@ -76,6 +82,8 @@ class PkiScriptlet(pkiscriptlet.AbstractBasePkiScriptlet):
 
         try:
             if not nssdb.exists():
+                logger.info('Creating NSS database: %s',
+                            deployer.mdict['pki_server_database_path'])
                 nssdb.create()
         finally:
             nssdb.close()
@@ -100,13 +108,13 @@ class PkiScriptlet(pkiscriptlet.AbstractBasePkiScriptlet):
         pki.util.chown(
             deployer.mdict['pki_server_database_path'],
             deployer.mdict['pki_uid'],
-            deployer.mdict['pki_uid'])
+            deployer.mdict['pki_gid'])
         pki.util.chmod(
             deployer.mdict['pki_server_database_path'],
             config.PKI_DEPLOYMENT_DEFAULT_SECURITY_DATABASE_PERMISSIONS)
         os.chmod(
             deployer.mdict['pki_server_database_path'],
-            config.PKI_DEPLOYMENT_DEFAULT_DIR_PERMISSIONS)
+            pki.server.DEFAULT_DIR_MODE)
 
         # import system certificates before starting the server
 
@@ -136,6 +144,7 @@ class PkiScriptlet(pkiscriptlet.AbstractBasePkiScriptlet):
 
         # import CA certificates from PKCS #12 file for cloning
         pki_clone_pkcs12_path = deployer.mdict['pki_clone_pkcs12_path']
+        pki_server_database_path = deployer.mdict['pki_server_database_path']
 
         if pki_clone_pkcs12_path:
 
@@ -145,7 +154,7 @@ class PkiScriptlet(pkiscriptlet.AbstractBasePkiScriptlet):
                 raise Exception('Missing pki_clone_pkcs12_password property.')
 
             nssdb = pki.nssdb.NSSDatabase(
-                directory=deployer.mdict['pki_server_database_path'],
+                directory=pki_server_database_path,
                 password_file=deployer.mdict['pki_shared_pfile'])
 
             try:
@@ -179,13 +188,50 @@ class PkiScriptlet(pkiscriptlet.AbstractBasePkiScriptlet):
                     trust_attributes='u,u,Pu')
 
                 print('Imported certificates into %s:' %
-                      deployer.mdict['pki_server_database_path'])
+                      pki_server_database_path)
 
                 nssdb.show_certs()
 
             finally:
                 nssdb.close()
 
+            # Export CA certificate to PEM file; same command as in
+            # PKIServer.setup_cert_authentication().
+            # openssl pkcs12 -in <p12_file_path> -out /tmp/auth.pem -nodes -nokeys
+            pki_ca_crt_path = os.path.join(pki_server_database_path, 'ca.crt')
+            cmd_export_ca = [
+                'openssl', 'pkcs12',
+                '-in', pki_clone_pkcs12_path,
+                '-out', pki_ca_crt_path,
+                '-nodes',
+                '-nokeys',
+                '-passin', 'pass:' + pki_clone_pkcs12_password
+            ]
+            res_ca = subprocess.check_output(cmd_export_ca,
+                                             stderr=subprocess.STDOUT).decode('utf-8')
+            logger.debug('Result of CA certificate export: %s', res_ca)
+
+            # At this point, we're running as root. However, the subsystem
+            # will eventually start up as non-root and will attempt to do a
+            # migration. If we don't fix the permissions now, migration will
+            # fail and subsystem won't start up.
+            pki.util.chmod(pki_ca_crt_path, 0o644)
+            pki.util.chown(pki_ca_crt_path, deployer.mdict['pki_uid'],
+                           deployer.mdict['pki_gid'])
+
+        ca_cert_path = deployer.mdict.get('pki_cert_chain_path')
+        if ca_cert_path and os.path.exists(ca_cert_path):
+            destination = os.path.join(instance.nssdb_dir, "ca.crt")
+            if not os.path.exists(destination):
+                # When we're passed a CA certificate file and we don't already
+                # have a CA file for some reason, we need to copy the passed
+                # file as the root CA certificate to establish trust in the
+                # Python code. It doesn't import it into the NSS DB for Java
+                # code, but so far we haven't had any issues with certificate
+                # validation there. This is only usually necessary when
+                # installing a non-CA subsystem on a fresh system.
+                instance.copyfile(ca_cert_path, destination)
+
         if len(deployer.instance.tomcat_instance_subsystems()) < 2:
 
             # Check to see if a secure connection is being used for the DS
@@ -239,13 +285,16 @@ class PkiScriptlet(pkiscriptlet.AbstractBasePkiScriptlet):
             tokenname = deployer.mdict['pki_%s_token' % deploy_tag]
             subsystem.config['%s.%s.tokenname' % (subsystem.name, config_tag)] = tokenname
 
+            fullname = nickname
+            if pki.nssdb.normalize_token(tokenname):
+                fullname = tokenname + ':' + nickname
+
+            subsystem.config['%s.cert.%s.nickname' % (subsystem.name, config_tag)] = fullname
+
             # store subject DN
             subject_dn = deployer.mdict['pki_%s_subject_dn' % deploy_tag]
             subsystem.config['preop.cert.%s.dn' % config_tag] = subject_dn
 
-            keytype = deployer.mdict['pki_%s_key_type' % deploy_tag]
-            subsystem.config['preop.cert.%s.keytype' % config_tag] = keytype
-
             keyalgorithm = deployer.mdict['pki_%s_key_algorithm' % deploy_tag]
             subsystem.config['preop.cert.%s.keyalgorithm' % config_tag] = keyalgorithm
 
@@ -253,14 +302,42 @@ class PkiScriptlet(pkiscriptlet.AbstractBasePkiScriptlet):
                 'pki_%s_signing_algorithm' % deploy_tag, keyalgorithm)
             subsystem.config['preop.cert.%s.signingalgorithm' % config_tag] = signingalgorithm
 
+            if subsystem.name == 'ca':
+                if config_tag == 'signing':
+                    subsystem.config['ca.signing.defaultSigningAlgorithm'] = signingalgorithm
+                    subsystem.config['ca.crl.MasterCRL.signingAlgorithm'] = signingalgorithm
+
+                elif config_tag == 'ocsp_signing':
+                    subsystem.config['ca.ocsp_signing.defaultSigningAlgorithm'] = signingalgorithm
+
+            elif subsystem.name == 'ocsp':
+                if config_tag == 'signing':
+                    subsystem.config['ocsp.signing.defaultSigningAlgorithm'] = signingalgorithm
+
+            elif subsystem.name == 'kra':
+                if config_tag == 'transport':
+                    subsystem.config['kra.transportUnit.signingAlgorithm'] = signingalgorithm
+
             # TODO: move more system cert params here
 
+        admin_dn = deployer.mdict['pki_admin_subject_dn']
+        subsystem.config['preop.cert.admin.dn'] = admin_dn
+
         # If specified in the deployment parameter, add generic CA signing cert
         # extension parameters into the CS.cfg. Generic extension for other
         # system certs can be added directly into CS.cfg after before the
         # configuration step.
 
         if subsystem.type == 'CA':
+
+            signing_nickname = subsystem.config['ca.signing.nickname']
+            subsystem.config['ca.signing.certnickname'] = signing_nickname
+            subsystem.config['ca.signing.cacertnickname'] = signing_nickname
+
+            ocsp_signing_nickname = subsystem.config['ca.ocsp_signing.nickname']
+            subsystem.config['ca.ocsp_signing.certnickname'] = ocsp_signing_nickname
+            subsystem.config['ca.ocsp_signing.cacertnickname'] = ocsp_signing_nickname
+
             if deployer.configuration_file.add_req_ext:
 
                 subsystem.config['preop.cert.signing.ext.oid'] = \
@@ -275,36 +352,41 @@ class PkiScriptlet(pkiscriptlet.AbstractBasePkiScriptlet):
                     deployer.configuration_file.req_ski
 
         if subsystem.type == 'KRA':
-            if deployer.configuration_file.clone:
 
-                storage_subsystem = subsystem.config['preop.cert.storage.subsystem']
-                storage_nickname = subsystem.config['preop.cert.storage.nickname']
-                transport_nickname = subsystem.config['preop.cert.transport.nickname']
+            storage_nickname = subsystem.config['kra.storage.nickname']
+            storage_token = subsystem.config['kra.storage.tokenname']
 
-                token = subsystem.config['preop.module.token']
-                if pki.nssdb.normalize_token(token):
+            if pki.nssdb.normalize_token(storage_token):
+                subsystem.config['kra.storageUnit.hardware'] = storage_token
+                subsystem.config['kra.storageUnit.nickName'] = \
+                    storage_token + ':' + storage_nickname
+            else:
+                subsystem.config['kra.storageUnit.nickName'] = \
+                    storage_nickname
 
-                    subsystem.config['%s.storageUnit.hardware' % storage_subsystem] = token
-                    subsystem.config['%s.storageUnit.nickName' % storage_subsystem] = \
-                        token + ':' + storage_nickname
-                    subsystem.config['%s.transportUnit.nickName' % storage_subsystem] = \
-                        token + ':' + transport_nickname
+            transport_nickname = subsystem.config['kra.transport.nickname']
+            transport_token = subsystem.config['kra.transport.tokenname']
 
-                else:
-                    subsystem.config['%s.storageUnit.nickName' % storage_subsystem] = \
-                        storage_nickname
-                    subsystem.config['%s.transportUnit.nickName' % storage_subsystem] = \
-                        transport_nickname
+            if pki.nssdb.normalize_token(transport_token):
+                subsystem.config['kra.transportUnit.nickName'] = \
+                    transport_token + ':' + transport_nickname
+            else:
+                subsystem.config['kra.transportUnit.nickName'] = \
+                    transport_nickname
 
-        if deployer.configuration_file.clone:
+        if subsystem.type == 'OCSP':
 
-            nickname = subsystem.config['%s.audit_signing.nickname' % subsystem.name]
-            token = subsystem.config['%s.audit_signing.tokenname' % subsystem.name]
+            signing_nickname = subsystem.config['ocsp.signing.nickname']
+            subsystem.config['ocsp.signing.certnickname'] = signing_nickname
+            subsystem.config['ocsp.signing.cacertnickname'] = signing_nickname
 
-            if pki.nssdb.normalize_token(token):
-                nickname = token + ':' + nickname
+        audit_nickname = subsystem.config['%s.audit_signing.nickname' % subsystem.name]
+        audit_token = subsystem.config['%s.audit_signing.tokenname' % subsystem.name]
 
-            subsystem.config['log.instance.SignedAudit.signedAuditCertNickname'] = nickname
+        if pki.nssdb.normalize_token(audit_token):
+            audit_nickname = audit_token + ':' + audit_nickname
+
+        subsystem.config['log.instance.SignedAudit.signedAuditCertNickname'] = audit_nickname
 
         subsystem.save()
 
@@ -351,6 +433,88 @@ class PkiScriptlet(pkiscriptlet.AbstractBasePkiScriptlet):
         finally:
             nssdb.close()
 
+    def set_signing_algs_for_pss(self, deployer):
+        # We know that the use pss flag is true.
+        # Only modify SHAXXXwithRSA series of algs for PSS.
+        logger.debug('In security_databases.set_signing_algs_for_pss')
+        if ('pki_admin_key_algorithm' in deployer.mdict):
+            if ('RSA' in deployer.mdict['pki_admin_key_algorithm'] and
+                    'PSS' not in deployer.mdict['pki_admin_key_algorithm']):
+                deployer.mdict['pki_admin_key_algorithm'] = \
+                    deployer.mdict['pki_admin_key_algorithm'] + '/PSS'
+        if ('pki_audit_signing_key_algorithm' in deployer.mdict):
+            if ('RSA' in deployer.mdict['pki_audit_signing_key_algorithm'] and
+                    'PSS' not in deployer.mdict['pki_audit_signing_key_algorithm']):
+                deployer.mdict['pki_audit_signing_key_algorithm'] = \
+                    deployer.mdict['pki_audit_signing_key_algorithm'] + '/PSS'
+        if ('pki_audit_signing_signing_algorithm' in deployer.mdict):
+            if ('RSA' in deployer.mdict['pki_audit_signing_signing_algorithm'] and
+                    'PSS' not in deployer.mdict['pki_audit_signing_signing_algorithm']):
+                deployer.mdict['pki_audit_signing_signing_algorithm'] = \
+                    deployer.mdict['pki_audit_signing_signing_algorithm'] + '/PSS'
+        if ('pki_ca_signing_key_algorithm' in deployer.mdict):
+            if ('RSA' in deployer.mdict['pki_ca_signing_key_algorithm'] and
+                    'PSS' not in deployer.mdict['pki_ca_signing_key_algorithm']):
+                deployer.mdict['pki_ca_signing_key_algorithm'] = \
+                    deployer.mdict['pki_ca_signing_key_algorithm'] + '/PSS'
+        if ('pki_ca_signing_signing_algorithm' in deployer.mdict):
+            if ('RSA' in deployer.mdict['pki_ca_signing_signing_algorithm'] and
+                    'PSS' not in deployer.mdict['pki_ca_signing_signing_algorithm']):
+                deployer.mdict['pki_ca_signing_signing_algorithm'] = \
+                    deployer.mdict['pki_ca_signing_signing_algorithm'] + '/PSS'
+        if ('pki_ocsp_signing_key_algorithm' in deployer.mdict):
+            if ('RSA' in deployer.mdict['pki_ocsp_signing_key_algorithm'] and
+                    'PSS' not in deployer.mdict['pki_ocsp_signing_key_algorithm']):
+                deployer.mdict['pki_ocsp_signing_key_algorithm'] = \
+                    deployer.mdict['pki_ocsp_signing_key_algorithm'] + '/PSS'
+        if ('pki_ocsp_signing_signing_algorithm' in deployer.mdict):
+            if ('RSA' in deployer.mdict['pki_ocsp_signing_signing_algorithm'] and
+                    'PSS' not in deployer.mdict['pki_ocsp_signing_signing_algorithm']):
+                deployer.mdict['pki_ocsp_signing_signing_algorithm'] = \
+                    deployer.mdict['pki_ocsp_signing_signing_algorithm'] + '/PSS'
+        if ('pki_transport_signing_algorithm' in deployer.mdict):
+            if ('pki_transport_signing_algorithm' in deployer.mdict):
+                if ('RSA' in deployer.mdict['pki_transport_signing_algorithm'] and
+                        'PSS' not in deployer.mdict['pki_transport_signing_algorithm']):
+                    deployer.mdict['pki_transport_signing_algorithm'] = \
+                        deployer.mdict['pki_transport_signing_algorithm'] + '/PSS'
+        if ('pki_transport_key_algorithm' in deployer.mdict):
+            if ('pki_transport_key_algorithm' in deployer.mdict):
+                if ('RSA' in deployer.mdict['pki_transport_key_algorithm'] and
+                        'PSS' not in deployer.mdict['pki_transport_key_algorithm']):
+                    deployer.mdict['pki_transport_key_algorithm'] = \
+                        deployer.mdict['pki_transport_key_algorithm'] + '/PSS'
+        if ('pki_sslserver_key_algorithm' in deployer.mdict):
+            if ('RSA' in deployer.mdict['pki_sslserver_key_algorithm'] and
+                    'PSS' not in deployer.mdict['pki_sslserver_key_algorithm']):
+                deployer.mdict['pki_sslserver_key_algorithm'] = \
+                    deployer.mdict['pki_sslserver_key_algorithm'] + '/PSS'
+        if ('pki_sslserver_signing_algorithm' in deployer.mdict):
+            if ('RSA' in deployer.mdict['pki_sslserver_signing_algorithm'] and
+                    'PSS' not in deployer.mdict['pki_sslserver_signing_algorithm']):
+                deployer.mdict['pki_sslserver_signing_algorithm'] = \
+                    deployer.mdict['pki_sslserver_signing_algorithm'] + '/PSS'
+        if ('pki_storage_signing_algorithm' in deployer.mdict):
+            if ('RSA' in deployer.mdict['pki_storage_signing_algorithm'] and
+                    'PSS' not in deployer.mdict['pki_storage_signing_algorithm']):
+                deployer.mdict['pki_storage_signing_algorithm'] = \
+                    deployer.mdict['pki_storage_signing_algorithm'] + '/PSS'
+        if ('pki_storage_key_algorithm' in deployer.mdict):
+            if ('RSA' in deployer.mdict['pki_storage_key_algorithm'] and
+                    'PSS' not in deployer.mdict['pki_storage_key_algorithm']):
+                deployer.mdict['pki_storage_key_algorithm'] = \
+                    deployer.mdict['pki_storage_key_algorithm'] + '/PSS'
+        if ('pki_subsystem_key_algorithm' in deployer.mdict):
+            if ('RSA' in deployer.mdict['pki_subsystem_key_algorithm'] and
+                    'PSS' not in deployer.mdict['pki_subsystem_key_algorithm']):
+                deployer.mdict['pki_subsystem_key_algorithm'] = \
+                    deployer.mdict['pki_subsystem_key_algorithm'] + '/PSS'
+        if ('pki_subsystem_signing_algorithm' in deployer.mdict):
+            if ('RSA' in deployer.mdict['pki_subsystem_signing_algorithm'] and
+                    'PSS' not in deployer.mdict['pki_subsystem_signing_algorithm']):
+                deployer.mdict['pki_subsystem_signing_algorithm'] = \
+                    deployer.mdict['pki_subsystem_signing_algorithm'] + '/PSS'
+
     def update_external_certs_conf(self, external_path, deployer):
         external_certs = pki.server.instance.PKIInstance.read_external_certs(
             external_path)
@@ -372,10 +536,13 @@ class PkiScriptlet(pkiscriptlet.AbstractBasePkiScriptlet):
 
         if deployer.directory.exists(deployer.mdict['pki_client_dir']):
             logger.info('Removing %s', deployer.mdict['pki_client_dir'])
-            pki.util.rmtree(deployer.mdict['pki_client_dir'])
+            pki.util.rmtree(deployer.mdict['pki_client_dir'],
+                            deployer.mdict['pki_force_destroy'])
 
         logger.info('Removing %s', deployer.mdict['pki_server_database_path'])
-        pki.util.rmtree(deployer.mdict['pki_server_database_path'])
+        pki.util.rmtree(deployer.mdict['pki_server_database_path'],
+                        deployer.mdict['pki_force_destroy'])
 
         logger.info('Removing %s', deployer.mdict['pki_shared_password_conf'])
-        pki.util.remove(deployer.mdict['pki_shared_password_conf'])
+        pki.util.remove(deployer.mdict['pki_shared_password_conf'],
+                        deployer.mdict['pki_force_destroy'])
