@@ -22,6 +22,7 @@
 from __future__ import absolute_import
 
 import functools
+import inspect
 import io
 import logging
 import os
@@ -40,8 +41,6 @@ import pki.nssdb
 import pki.util
 import pki.server
 
-SUBSYSTEM_TYPES = ['ca', 'kra', 'ocsp', 'tks', 'tps']
-
 logger = logging.getLogger(__name__)
 
 parser = etree.XMLParser(remove_blank_text=True)
@@ -67,7 +66,6 @@ class PKIInstance(pki.server.PKIServer):
         self.version = version
 
         self.external_certs = []
-        self.subsystems = []
 
         self.default_root_doc_base = os.path.join(
             pki.SHARE_DIR,
@@ -145,10 +143,6 @@ class PKIInstance(pki.server.PKIServer):
     def conf_dir(self):
         return os.path.join(pki.server.PKIServer.CONFIG_DIR, self.name)
 
-    @property
-    def nssdb_dir(self):
-        return os.path.join(self.conf_dir, 'alias')
-
     @property
     def log_dir(self):
         return os.path.join(pki.server.PKIServer.LOG_DIR, self.name)
@@ -159,6 +153,10 @@ class PKIInstance(pki.server.PKIServer):
 
     @property
     def server_cert_nick_conf(self):
+        logger.warning(
+            '%s:%s: The PKIInstance.server_cert_nick_conf() has '
+            'been deprecated (https://www.dogtagpki.org/wiki/PKI_10.9_Python_Changes).',
+            inspect.stack()[1].filename, inspect.stack()[1].lineno)
         return os.path.join(self.conf_dir, 'serverCertNick.conf')
 
     @property
@@ -181,7 +179,13 @@ class PKIInstance(pki.server.PKIServer):
     def unit_file(self):
         return PKIInstance.TARGET_WANTS + '/%s.service' % self.service_name
 
-    def execute(self, command, jdb=False, as_current_user=False, gdb=False):
+    def execute(
+            self, command,
+            as_current_user=False,
+            with_jdb=False,
+            with_gdb=False,
+            with_valgrind=False,
+            agentpath=None):
 
         if command == 'start':
 
@@ -196,44 +200,52 @@ class PKIInstance(pki.server.PKIServer):
                 if current_user != self.user:
                     prefix.extend(['sudo', '-u', self.user])
 
-            cmd = prefix + ['/usr/sbin/pki-server', 'upgrade', '--validate', self.name]
+            cmd = prefix + ['/usr/sbin/pki-server', 'upgrade']
 
             if logger.isEnabledFor(logging.DEBUG):
-                cmd.extend(['--debug'])
+                cmd.append('--debug')
 
             elif logger.isEnabledFor(logging.INFO):
-                cmd.extend(['-v'])
+                cmd.append('--verbose')
+
+            cmd.append(self.name)
 
             logger.debug('Command: %s', ' '.join(cmd))
-
             subprocess.run(cmd, env=self.config, check=True)
 
             cmd = prefix + ['/usr/bin/pkidaemon', 'start', self.name]
-            logger.debug('Command: %s', ' '.join(cmd))
 
+            logger.debug('Command: %s', ' '.join(cmd))
             subprocess.run(cmd, env=self.config, check=True)
 
-        return super(PKIInstance, self).execute(command, jdb=jdb,
-                                                as_current_user=as_current_user,
-                                                gdb=gdb)
+        return super(PKIInstance, self).execute(
+            command,
+            as_current_user=as_current_user,
+            with_jdb=with_jdb,
+            with_gdb=with_gdb,
+            with_valgrind=with_valgrind,
+            agentpath=agentpath)
 
     def create(self, force=False):
 
         super(PKIInstance, self).create(force=force)
 
         conf_link = os.path.join(self.base_dir, 'conf')
+        logger.info('Creating %s', conf_link)
         self.symlink(self.conf_dir, conf_link, force=force)
 
         logs_link = os.path.join(self.base_dir, 'logs')
+        logger.info('Creating %s', logs_link)
         self.symlink(self.log_dir, logs_link, force=force)
 
+        logger.info('Creating %s', self.registry_dir)
         self.makedirs(self.registry_dir, force=force)
 
+        logger.info('Creating %s', self.registry_file)
         self.copyfile(
             PKIInstance.REGISTRY_FILE,
             self.registry_file,
             params={
-                'PKI_WEB_SERVER_TYPE': 'tomcat',
                 'PKI_USER': self.user,
                 'PKI_GROUP': self.group,
                 'PKI_INSTANCE_NAME': self.name,
@@ -241,6 +253,7 @@ class PKIInstance(pki.server.PKIServer):
                 'TOMCAT_PIDFILE': '/var/run/pki/tomcat/' + self.name + '.pid'
             })
 
+        logger.info('Creating %s', self.unit_file)
         self.symlink(PKIInstance.UNIT_FILE, self.unit_file, force=force)
 
     def create_libs(self, force=False):
@@ -281,8 +294,13 @@ class PKIInstance(pki.server.PKIServer):
 
         dependencies = project.findall('{%s}dependencies/{%s}dependency' % (xmlns, xmlns))
 
+        logger.info('Creating %s', self.lib_dir)
         self.makedirs(self.lib_dir, force=force)
+
+        logger.info('Creating %s', self.common_dir)
         self.makedirs(self.common_dir, force=force)
+
+        logger.info('Creating %s', self.common_lib_dir)
         self.makedirs(self.common_lib_dir, force=force)
 
         for dependency in dependencies:
@@ -313,7 +331,6 @@ class PKIInstance(pki.server.PKIServer):
                 'jss4.jar',
                 'ldapjdk.jar',
                 'pki-cmsutil.jar',
-                'pki-nsutil.jar',
                 'pki-tomcat.jar',
                 'symkey.jar',
                 'tomcatjss.jar']:
@@ -324,15 +341,6 @@ class PKIInstance(pki.server.PKIServer):
             logger.info('Linking %s to %s', dest, source)
             self.symlink(source, dest, force=force)
 
-    def create_nssdb(self, force=False):
-
-        super(PKIInstance, self).create_nssdb(force=force)
-
-        nssdb_link = os.path.join(self.base_dir, 'alias')
-        logger.info('Creating NSS database link: %s', nssdb_link)
-
-        self.symlink(self.nssdb_dir, nssdb_link, force=force)
-
     def load(self):
 
         super(PKIInstance, self).load()
@@ -349,62 +357,51 @@ class PKIInstance(pki.server.PKIServer):
                 m = re.search('^PKI_USER=(.*)$', line)
                 if m:
                     self.user = m.group(1)
+                    logger.info('- user: %s', self.user)
 
                 m = re.search('^PKI_GROUP=(.*)$', line)
                 if m:
                     self.group = m.group(1)
+                    logger.info('- group: %s', self.group)
 
         self.load_external_certs(self.external_certs_conf)
 
-        # load subsystems
-        for subsystem_name in SUBSYSTEM_TYPES:
-
-            subsystem_dir = os.path.join(self.base_dir, subsystem_name)
-            if not os.path.exists(subsystem_dir):
-                continue
-
-            subsystem = pki.server.subsystem.PKISubsystemFactory.create(self, subsystem_name)
-            subsystem.load()
-            self.subsystems.append(subsystem)
-
     def load_external_certs(self, conf_file):
         for external_cert in PKIInstance.read_external_certs(conf_file):
             self.external_certs.append(external_cert)
 
     def remove(self, force=False):
 
+        logger.info('Removing %s', self.unit_file)
         pki.util.unlink(self.unit_file, force=force)
+
+        logger.info('Removing %s', self.registry_file)
         pki.util.remove(self.registry_file, force=force)
+
+        logger.info('Removing %s', self.registry_dir)
         pki.util.rmtree(self.registry_dir, force=force)
 
         logs_link = os.path.join(self.base_dir, 'logs')
+        logger.info('Removing %s', logs_link)
         pki.util.unlink(logs_link, force=force)
 
         conf_link = os.path.join(self.base_dir, 'conf')
+        logger.info('Removing %s', conf_link)
         pki.util.unlink(conf_link, force=force)
 
         super(PKIInstance, self).remove(force=force)
 
     def remove_libs(self, force=False):
 
-        # remove <instance>/common which is always a folder
+        logger.info('Removing %s', self.common_dir)
         pki.util.rmtree(self.common_dir, force=force)
 
-        # remove <instance>/lib which could be a link or a folder
+        logger.info('Removing %s', self.lib_dir)
         if os.path.islink(self.lib_dir):
             pki.util.unlink(self.lib_dir, force=force)
         else:
             pki.util.rmtree(self.lib_dir, force=force)
 
-    def remove_nssdb(self, force=False):
-
-        nssdb_link = os.path.join(self.base_dir, 'alias')
-        logger.info('Removing NSS database link: %s', nssdb_link)
-
-        pki.util.unlink(nssdb_link, force=force)
-
-        super(PKIInstance, self).remove_nssdb(force=force)
-
     @staticmethod
     def read_external_certs(conf_file):
         external_certs = []
@@ -498,7 +495,23 @@ class PKIInstance(pki.server.PKIServer):
                 shutil.rmtree(tmpdir)
 
     def get_sslserver_cert_nickname(self):
-        with open(self.server_cert_nick_conf) as f:
+
+        # Load SSL server cert nickname from server.xml
+
+        server_config = self.get_server_config()
+        connector = server_config.get_connector('Secure')
+        sslhost = server_config.get_sslhost(connector)
+        sslcert = server_config.get_sslcert(sslhost)
+        nickname = sslcert.get('certificateKeyAlias')
+
+        if nickname:
+            return nickname
+
+        # If not available, load SSL server cert nickname from serverCertNick.conf
+        # TODO: Remove serverCertNick.conf
+
+        server_cert_nick_conf = os.path.join(self.conf_dir, 'serverCertNick.conf')
+        with open(server_cert_nick_conf) as f:
             return f.readline().strip()
 
     def set_sslserver_cert_nickname(self, nickname, token=None):
@@ -506,14 +519,18 @@ class PKIInstance(pki.server.PKIServer):
         if pki.nssdb.normalize_token(token):
             nickname = token + ':' + nickname
 
-        # update serverCertNick.conf
-        with open(self.server_cert_nick_conf, 'w') as f:
+        # Store SSL server cert nickname into serverCertNick.conf
+        # TODO: Remove serverCertNick.conf
+
+        server_cert_nick_conf = os.path.join(self.conf_dir, 'serverCertNick.conf')
+        with open(server_cert_nick_conf, 'w') as f:
             f.write(nickname + '\n')
 
-        os.chown(self.server_cert_nick_conf, self.uid, self.gid)
-        os.chmod(self.server_cert_nick_conf, 0o0660)
+        os.chown(server_cert_nick_conf, self.uid, self.gid)
+        os.chmod(server_cert_nick_conf, pki.server.DEFAULT_FILE_MODE)
+
+        # Store SSL server cert nickname into server.xml
 
-        # update server.xml
         server_config = self.get_server_config()
         connector = server_config.get_connector('Secure')
         sslhost = server_config.get_sslhost(connector)
@@ -521,12 +538,6 @@ class PKIInstance(pki.server.PKIServer):
         sslcert.set('certificateKeyAlias', nickname)
         server_config.save()
 
-    def get_subsystem(self, name):
-        for subsystem in self.subsystems:
-            if name == subsystem.name:
-                return subsystem
-        return None
-
     def banner_installed(self):
         return os.path.exists(self.banner_file)
 
@@ -552,7 +563,7 @@ class PKIInstance(pki.server.PKIServer):
         subsystem_name, cert_tag = pki.server.PKIServer.split_cert_id(cert_id)
 
         if not subsystem_name:
-            subsystem_name = self.subsystems[0].name
+            subsystem_name = self.get_subsystems()[0].name
 
         subsystem = self.get_subsystem(subsystem_name)
 
@@ -588,7 +599,7 @@ class PKIInstance(pki.server.PKIServer):
         # store cert data and request in CS.cfg
         if cert_id == 'sslserver' or cert_id == 'subsystem':
             # Update for all subsystems
-            for subsystem in self.subsystems:
+            for subsystem in self.get_subsystems():
                 subsystem.update_subsystem_cert(cert)
                 subsystem.save()
         else:
@@ -630,7 +641,7 @@ class PKIInstance(pki.server.PKIServer):
         subsystem_name, cert_tag = pki.server.PKIServer.split_cert_id(cert_id)
 
         if not subsystem_name:
-            subsystem_name = self.subsystems[0].name
+            subsystem_name = self.get_subsystems()[0].name
 
         subsystem = self.get_subsystem(subsystem_name)
 
@@ -744,9 +755,10 @@ class PKIInstance(pki.server.PKIServer):
         :raises pki.server.PKIServerException
 
         Either supply both username and password, or supply
-        client_nssdb and client_cert and
-        (client_nssdb_pass or client_nssdb_pass_file).
+        client_cert and (client_nssdb_pass or client_nssdb_pass_file).
 
+        Note that client_nssdb should be specified in either case, as it
+        contains the CA Certificate.
         """
         nssdb = self.open_nssdb()
         tmpdir = tempfile.mkdtemp()
@@ -758,7 +770,7 @@ class PKIInstance(pki.server.PKIServer):
 
                 subsystem_name, cert_tag = pki.server.PKIServer.split_cert_id(cert_id)
                 if not subsystem_name:
-                    subsystem_name = self.subsystems[0].name
+                    subsystem_name = self.get_subsystems()[0].name
                 subsystem = self.get_subsystem(subsystem_name)
 
                 if serial is None:
@@ -800,7 +812,8 @@ class PKIInstance(pki.server.PKIServer):
                 logger.info('Trying to setup a secure connection to CA subsystem.')
                 if username and password:
                     connection = pki.server.PKIServer.setup_password_authentication(
-                        username, password, subsystem_name='ca', secure_port=secure_port)
+                        username, password, subsystem_name='ca', secure_port=secure_port,
+                        client_nssdb=client_nssdb)
                 else:
                     if not client_cert:
                         raise pki.server.PKIServerException('Client cert nick name required.')
@@ -865,10 +878,23 @@ class PKIServerFactory(object):
             instance_type = parts[0]
             instance_name = parts[1]
 
+        sysconfig_file = os.path.join('/etc/sysconfig', instance_name)
+
+        if os.path.isfile(sysconfig_file):
+
+            with open(sysconfig_file) as f:
+                nuxwdog_status = re.search('^USE_NUXWDOG=\"(.*)\"', f.read(), re.MULTILINE)
+
+                # Check if the regex was matched and then check if nuxwdog is enabled.
+                if nuxwdog_status and nuxwdog_status.group(1) == "true":
+                    instance_type += '-nuxwdog'
+
+        logger.info("Loading instance type: %s", instance_type)
+
         if instance_type == 'tomcat':
             return pki.server.PKIServer(instance_name)
 
-        if instance_type == 'pki-tomcatd':
-            return PKIInstance(instance_name)
+        if instance_type.startswith('pki-tomcatd'):
+            return PKIInstance(instance_name, instance_type=instance_type)
 
         raise Exception('Unsupported instance type: %s' % instance_type)
