@@ -18,38 +18,27 @@
 # All rights reserved.
 #
 
-# System Imports
 from __future__ import absolute_import
 from __future__ import print_function
+import fileinput
+import ldap
 import logging
+import os
+import requests
 import sys
 import signal
 import subprocess
+import traceback
 
-if not hasattr(sys, "hexversion") or sys.hexversion < 0x020700f0:
-    print("Python version %s.%s.%s is too old." % sys.version_info[:3])
-    print("Please upgrade to at least Python 2.7.0.")
-    sys.exit(1)
-try:
-    import fileinput
-    import ldap
-    import os
-    import requests
-    import traceback
-    import pki
-    from pki.server.deployment import pkiconfig as config
-    from pki.server.deployment import pkimanifest as manifest
-    from pki.server.deployment.pkiparser import PKIConfigParser
-    from pki.server.deployment import pkilogging
-    from pki.server.deployment import pkimessages as log
-except ImportError:
-    print("""\
-There was a problem importing one of the required Python modules. The
-error was:
+import pki
+import pki.server
+import pki.server.instance
 
-    %s
-""" % sys.exc_info()[1], file=sys.stderr)
-    sys.exit(1)
+from pki.server.deployment import pkiconfig as config
+from pki.server.deployment import pkimanifest as manifest
+from pki.server.deployment.pkiparser import PKIConfigParser
+from pki.server.deployment import pkilogging
+from pki.server.deployment import pkimessages as log
 
 logger = logging.getLogger('pkispawn')
 
@@ -377,6 +366,12 @@ def main(argv):
 
             else:
                 while True:
+                    ca_cert = os.path.join(deployer.mdict['pki_server_database_path'], "ca.crt")
+                    if not os.path.exists(ca_cert):
+                        parser.read_text('Security Domain CA Root Certificate',
+                                         deployer.subsystem_name,
+                                         'pki_cert_chain_path')
+
                     parser.read_text('Hostname',
                                      deployer.subsystem_name,
                                      'pki_security_domain_hostname')
@@ -513,7 +508,9 @@ def main(argv):
               deployer.subsystem_name.lower())
         sys.exit(1)
 
-    start_logging()
+    if args.pki_verbosity > 1:
+        logger.warning('The -%s option has been deprecated. Use --debug instead.',
+                       'v' * args.pki_verbosity)
 
     # Read the specified PKI configuration file.
     rv = parser.read_pki_configuration_file()
@@ -532,6 +529,21 @@ def main(argv):
     create_master_dictionary(parser)
     deployer.init()
 
+    instance = pki.server.instance.PKIInstance(
+        deployer.mdict['pki_instance_name'],
+        user=deployer.mdict['pki_user'],
+        group=deployer.mdict['pki_group'])
+
+    log_dir = config.PKI_DEPLOYMENT_LOG_ROOT
+    log_name = "pki" + "-" + \
+               deployer.subsystem_name.lower() + \
+               "-" + "spawn" + "." + \
+               deployer.log_timestamp + "." + "log"
+    log_file = os.path.join(log_dir, log_name)
+    print('Installation log: %s' % log_file)
+
+    pkilogging.enable_pki_logger(log_file, 'pkispawn')
+
     if not interactive and \
             not config.str2bool(parser.mdict['pki_skip_configuration']):
         check_ds()
@@ -558,7 +570,8 @@ def main(argv):
                 fromlist=[scriptlet_name])
 
             scriptlet = scriptlet_module.PkiScriptlet()
-
+            scriptlet.deployer = deployer
+            scriptlet.instance = instance
             scriptlet.spawn(deployer)
 
     except subprocess.CalledProcessError as e:
@@ -568,7 +581,7 @@ def main(argv):
         if e.output:
             print(e.output)
         print()
-        print('Please check pkispawn logs in %s/%s' % (config.pki_log_dir, config.pki_log_name))
+        print('Please check pkispawn logs in %s' % log_file)
         sys.exit(1)
 
     except requests.HTTPError as e:
@@ -679,21 +692,6 @@ def sanitize_user_deployment_cfg(cfg):
         print(line)
 
 
-def start_logging():
-    # Enable 'pkispawn' logging.
-    config.pki_log_dir = config.PKI_DEPLOYMENT_LOG_ROOT
-    config.pki_log_name = "pki" + "-" + \
-                          deployer.subsystem_name.lower() + \
-                          "-" + "spawn" + "." + \
-                          deployer.log_timestamp + "." + "log"
-    print('Installation log: %s/%s' % (config.pki_log_dir, config.pki_log_name))
-
-    pkilogging.enable_pki_logger(config.pki_log_dir,
-                                 config.pki_log_name,
-                                 config.pki_log_level,
-                                 "pkispawn")
-
-
 def create_master_dictionary(parser):
 
     # Read in the PKI slots configuration file.
@@ -702,9 +700,6 @@ def create_master_dictionary(parser):
     # Combine the various sectional dictionaries into a PKI master dictionary
     parser.compose_pki_master_dictionary()
 
-    parser.mdict['pki_spawn_log'] = \
-        config.pki_log_dir + "/" + config.pki_log_name
-
     logger.debug(log.PKI_DICTIONARY_MASTER)
     logger.debug(pkilogging.log_format(parser.mdict))
 
