@@ -22,9 +22,8 @@
 from __future__ import absolute_import
 
 import functools
-import io
+import json
 import logging
-import operator
 import os
 import pwd
 import re
@@ -36,12 +35,12 @@ import xml.etree.ElementTree as ET
 
 import ldap
 import ldap.filter
-import six
 
 import pki
 import pki.nssdb
 import pki.util
 import pki.server
+import pki.system
 
 SELFTEST_CRITICAL = 'critical'
 
@@ -63,8 +62,11 @@ class PKISubsystem(object):
 
         self.conf_dir = os.path.join(self.base_dir, 'conf')
         self.cs_conf = os.path.join(self.conf_dir, 'CS.cfg')
+        self.registry_conf = os.path.join(self.conf_dir, 'registry.cfg')
 
         self.config = {}
+        self.registry = {}
+
         self.type = None  # e.g. CA, KRA
         self.prefix = None  # e.g. ca, kra
 
@@ -116,32 +118,21 @@ class PKISubsystem(object):
 
     def load(self):
 
-        logger.info('Loading subsystem: %s', self.name)
-
         self.config.clear()
 
         if os.path.exists(self.cs_conf):
-
             logger.info('Loading subsystem config: %s', self.cs_conf)
-
-            lines = open(self.cs_conf).read().splitlines()
-
-            for index, line in enumerate(lines):
-
-                if not line or line.startswith('#'):
-                    continue
-
-                parts = line.split('=', 1)
-                if len(parts) < 2:
-                    raise Exception('Missing delimiter in %s line %d' % (self.cs_conf, index + 1))
-
-                name = parts[0]
-                value = parts[1]
-                self.config[name] = value
+            pki.util.load_properties(self.cs_conf, self.config)
 
             self.type = self.config['cs.type']
             self.prefix = self.type.lower()
 
+        self.registry.clear()
+
+        if os.path.exists(self.registry_conf):
+            logger.info('Loading subsystem registry: %s', self.registry_conf)
+            pki.util.load_properties(self.registry_conf, self.registry)
+
     def find_system_certs(self):
 
         cert_ids = self.config['%s.cert.list' % self.name].split(',')
@@ -164,6 +155,8 @@ class PKISubsystem(object):
     def get_subsystem_cert(self, cert_id):
 
         cert = self.get_cert_info(cert_id)
+
+        # If nickname is empty, then cert cannot be queried for in NSSDB
         if not cert['nickname']:
             return cert
 
@@ -175,7 +168,7 @@ class PKISubsystem(object):
 
     def get_cert_info(self, cert_id):
 
-        logger.info('Getting %s cert info for %s from CS.cfg', cert_id, self.name)
+        logger.info('Getting %s cert info from CS.cfg', cert_id)
 
         nickname = self.config.get('%s.%s.nickname' % (self.name, cert_id))
         token = self.config.get('%s.%s.tokenname' % (self.name, cert_id))
@@ -195,27 +188,23 @@ class PKISubsystem(object):
 
     def get_nssdb_cert_info(self, cert_id):
 
-        logger.info('Getting %s cert info for %s from NSS database', cert_id, self.name)
+        logger.info('Getting %s cert info from NSS database', cert_id)
 
         nickname = self.config.get('%s.%s.nickname' % (self.name, cert_id))
         token = self.config.get('%s.%s.tokenname' % (self.name, cert_id))
 
-        nssdb = self.instance.open_nssdb(token)
+        nssdb = self.instance.open_nssdb()
         try:
-            return nssdb.get_cert_info(nickname)
+            return nssdb.get_cert_info(nickname, token=token)
         finally:
             nssdb.close()
 
     def update_subsystem_cert(self, cert):
         cert_id = cert['id']
-        self.config['%s.%s.nickname' % (self.name, cert_id)] = (
-            cert.get('nickname', None))
-        self.config['%s.%s.tokenname' % (self.name, cert_id)] = (
-            cert.get('token', None))
-        self.config['%s.%s.cert' % (self.name, cert_id)] = (
-            cert.get('data', None))
-        self.config['%s.%s.certreq' % (self.name, cert_id)] = (
-            cert.get('request', None))
+        self.config['%s.%s.nickname' % (self.name, cert_id)] = cert.get('nickname')
+        self.config['%s.%s.tokenname' % (self.name, cert_id)] = cert.get('token')
+        self.config['%s.%s.cert' % (self.name, cert_id)] = cert.get('data')
+        self.config['%s.%s.certreq' % (self.name, cert_id)] = cert.get('request')
 
     def validate_system_cert(self, cert_id=None):
 
@@ -237,40 +226,44 @@ class PKISubsystem(object):
             cert_id,
             pkcs12_file,
             pkcs12_password_file,
+            no_key=False,
             append=False):
 
         cert = self.get_subsystem_cert(cert_id)
         nickname = cert['nickname']
         token = pki.nssdb.normalize_token(cert['token'])
 
-        nssdb_password = self.instance.get_token_password(token)
+        if token:
+            nickname = token + ':' + nickname
 
         tmpdir = tempfile.mkdtemp()
 
         try:
-            nssdb_password_file = os.path.join(tmpdir, 'password.txt')
-            with open(nssdb_password_file, 'w') as f:
-                f.write(nssdb_password)
-
             # add the certificate, key, and chain
             cmd = [
                 'pki',
                 '-d', self.instance.nssdb_dir,
-                '-C', nssdb_password_file
+                '-f', self.instance.password_conf
             ]
 
-            if token:
-                cmd.extend(['--token', token])
-
             cmd.extend([
                 'pkcs12-cert-import',
                 '--pkcs12-file', pkcs12_file,
                 '--pkcs12-password-file', pkcs12_password_file,
             ])
 
+            if no_key:
+                cmd.extend(['--no-key'])
+
             if append:
                 cmd.extend(['--append'])
 
+            if logger.isEnabledFor(logging.DEBUG):
+                cmd.append('--debug')
+
+            elif logger.isEnabledFor(logging.INFO):
+                cmd.append('-v')
+
             cmd.extend([
                 nickname
             ])
@@ -313,8 +306,8 @@ class PKISubsystem(object):
 
             cmd.extend([
                 'pkcs12-export',
-                '--pkcs12-file', pkcs12_file,
-                '--pkcs12-password-file', pkcs12_password_file,
+                '--pkcs12', pkcs12_file,
+                '--password-file', pkcs12_password_file,
                 nickname
             ])
 
@@ -347,18 +340,12 @@ class PKISubsystem(object):
             shutil.rmtree(tmpdir)
 
     def save(self):
-        sorted_config = sorted(self.config.items(), key=operator.itemgetter(0))
-        with io.open(self.cs_conf, 'w') as f:
-            for key, value in sorted_config:
-                if value is None:
-                    # write None as empty value
-                    f.write(u'{0}=\n'.format(key))
-                elif isinstance(value, six.string_types):
-                    f.write(u'{0}={1}\n'.format(key, value))
-                elif isinstance(value, six.integer_types):
-                    f.write(u'{0}={1:d}\n'.format(key, value))
-                else:
-                    raise TypeError((key, value, type(value)))
+
+        logger.info('Storing subsystem config: %s', self.cs_conf)
+        self.instance.store_properties(self.cs_conf, self.config)
+
+        logger.info('Storing registry config: %s', self.registry_conf)
+        self.instance.store_properties(self.registry_conf, self.registry)
 
     def is_valid(self):
         return os.path.exists(self.conf_dir)
@@ -384,12 +371,15 @@ class PKISubsystem(object):
             protocol = 'http'
             port = server_config.get_unsecure_port()
 
+        # When waiting for a connection to come alive, don't bother verifying
+        # the certificate at this stage.
         connection = pki.client.PKIConnection(
             protocol=protocol,
             hostname=socket.getfqdn(),
             port=port,
             accept='application/xml',
-            trust_env=False)
+            trust_env=False,
+            verify=False)
 
         client = pki.system.SystemStatusClient(connection, subsystem=self.name)
         response = client.get_status(timeout=timeout)
@@ -726,7 +716,7 @@ class PKISubsystem(object):
         logger.debug('Exporting CSR for %s cert', cert_tag)
 
         # Retrieve CSR for cert_id
-        cert_request = self.get_subsystem_cert(cert_tag).get('request', None)
+        cert_request = self.get_subsystem_cert(cert_tag).get('request')
         if cert_request is None:
             raise pki.server.PKIServerException('Unable to find CSR for %s cert' % cert_tag)
 
@@ -743,7 +733,7 @@ class PKISubsystem(object):
         # Retrieve Subject Key Identifier from CA cert
         ca_signing_cert = self.instance.get_subsystem('ca').get_subsystem_cert('signing')
 
-        ca_cert_data = ca_signing_cert.get('data', None)
+        ca_cert_data = ca_signing_cert.get('data')
         if ca_cert_data is None:
             raise pki.server.PKIServerException(
                 'Unable to find certificate data for CA signing certificate.')
@@ -859,6 +849,29 @@ class PKISubsystem(object):
         """
         self.config.update(new_config)
 
+    def import_ldif(self, bind_dn, bind_password, filename):
+
+        # TODO(alee) re-implement this using open_database
+        host = self.config['internaldb.ldapconn.host']
+        port = self.config['internaldb.ldapconn.port']
+        secure = self.config['internaldb.ldapconn.secureConn']
+
+        cmd = [
+            'ldapmodify',
+            '-c',
+            '-D', bind_dn,
+            '-w', bind_password,
+            '-h', host,
+            '-p', port,
+            '-f', filename
+        ]
+
+        if secure.lower() == 'true':
+            cmd.append('-Z')
+
+        logger.debug('Command: %s', ' '.join(cmd))
+        subprocess.check_call(cmd)
+
     def init_database(
             self,
             setup_schema=False,
@@ -931,8 +944,142 @@ class PKISubsystem(object):
 
         self.run(cmd, as_current_user=as_current_user)
 
+    def request_range(self, master_url, range_type, install_token):
+
+        cmd = [
+            'pki',
+            '-d', self.instance.nssdb_dir,
+            '-f', self.instance.password_conf,
+            '-U', master_url,
+            '%s-range-request' % self.name,
+            range_type,
+            '--session', install_token.token,
+            '--output-format', 'json'
+        ]
+
+        if logger.isEnabledFor(logging.DEBUG):
+            cmd.append('--debug')
+
+        elif logger.isEnabledFor(logging.INFO):
+            cmd.append('--verbose')
+
+        logger.debug('Command: %s', ' '.join(cmd))
+        output = subprocess.check_output(cmd)
+
+        return json.loads(output.decode())
+
+    def update_ranges(self, master_url, install_token):
+
+        logger.info('Updating request ID range')
+
+        request_range = self.request_range(master_url, 'request', install_token)
+        self.config['dbs.beginRequestNumber'] = request_range['begin']
+        self.config['dbs.endRequestNumber'] = request_range['end']
+
+        logger.info('Updating serial number range')
+
+        serial_range = self.request_range(master_url, 'serialNo', install_token)
+        self.config['dbs.beginSerialNumber'] = serial_range['begin']
+        self.config['dbs.endSerialNumber'] = serial_range['end']
+
+        logger.info('Updating replica ID range')
+
+        replica_range = self.request_range(master_url, 'replicaId', install_token)
+        self.config['dbs.beginReplicaNumber'] = replica_range['begin']
+        self.config['dbs.endReplicaNumber'] = replica_range['end']
+
+        self.config['dbs.enableSerialManagement'] = 'true'
+
+        self.save()
+
+    def retrieve_config(self, master_url, names, substores, install_token):
+
+        cmd = [
+            'pki',
+            '-d', self.instance.nssdb_dir,
+            '-f', self.instance.password_conf,
+            '-U', master_url,
+            '%s-config-export' % self.name,
+            '--names', ','.join(names),
+            '--substores', ','.join(substores),
+            '--session', install_token.token,
+            '--output-format', 'json'
+        ]
+
+        if logger.isEnabledFor(logging.DEBUG):
+            cmd.append('--debug')
+
+        elif logger.isEnabledFor(logging.INFO):
+            cmd.append('--verbose')
+
+        logger.debug('Command: %s', ' '.join(cmd))
+        output = subprocess.check_output(cmd)
+
+        return json.loads(output.decode())
+
+    def update_config(self, master_url, install_token):
+
+        logger.info('Updating configuration')
+
+        names = [
+            'internaldb.ldapauth.password',
+            'internaldb.replication.password'
+        ]
+
+        substores = [
+            'internaldb',
+            'internaldb.ldapauth',
+            'internaldb.ldapconn'
+        ]
+
+        tags = self.config['preop.cert.list'].split(',')
+        for tag in tags:
+            if tag == 'sslserver':
+                continue
+            substores.append(self.name + '.' + tag)
+
+        if self.name == 'ca':
+            substores.append('ca.connector.KRA')
+        else:
+            names.append('cloning.ca.type')
+
+        config = self.retrieve_config(master_url, names, substores, install_token)
+        properties = config['properties']
+
+        for name in properties:
+
+            if name.startswith('internaldb'):
+                new_name = 'preop.internaldb.master' + name[10:]
+
+            elif name.startswith('cloning.ca'):
+                new_name = 'preop.ca' + name[10:]
+
+            elif name.startswith('cloning'):
+                new_name = 'preop.cert' + name[7:]
+
+            else:
+                new_name = name
+
+            value = properties.get(name)
+            self.config[new_name] = value
+
+        self.config['preop.clone.configuration'] = 'true'
+
+        self.save()
+
+        master_hostname = self.config['preop.internaldb.master.ldapconn.host']
+        master_port = self.config['preop.internaldb.master.ldapconn.port']
+
+        replica_hostname = self.config['internaldb.ldapconn.host']
+        replica_port = self.config['internaldb.ldapconn.port']
+
+        if master_hostname == replica_hostname and \
+                master_port == replica_port:
+            raise Exception('Master and replica must not share LDAP database')
+
     def run(self, args, as_current_user=False):
 
+        java_path = os.getenv('PKI_JAVA_PATH')
         java_home = self.instance.config['JAVA_HOME']
         java_opts = self.instance.config['JAVA_OPTS']
 
@@ -955,8 +1102,16 @@ class PKISubsystem(object):
             if username != self.instance.user:
                 cmd.extend(['sudo', '-u', self.instance.user])
 
+        if os.path.exists(java_path):
+            cmd.extend([java_path])
+        elif os.path.exists(java_home + '/jre/bin/java'):
+            cmd.extend([java_home + '/jre/bin/java'])
+        elif os.path.exists(java_home + '/bin/java'):
+            cmd.extend([java_home + '/bin/java'])
+        else:
+            cmd.extend(['/usr/bin/env', 'java'])
+
         cmd.extend([
-            java_home + '/bin/java',
             '-classpath', os.pathsep.join(classpath),
             '-Djavax.sql.DataSource.Factory=org.apache.commons.dbcp.BasicDataSourceFactory',
             '-Dcatalina.base=' + self.instance.base_dir,
@@ -968,7 +1123,9 @@ class PKISubsystem(object):
         ])
 
         if java_opts:
-            cmd.extend(java_opts.split(' '))
+            opts = java_opts.split(' ')
+            non_empty_opts = [opt for opt in opts if opt]
+            cmd.extend(non_empty_opts)
 
         cmd.extend(['org.dogtagpki.server.cli.PKIServerCLI'])
 
@@ -988,14 +1145,6 @@ class CASubsystem(PKISubsystem):
     def __init__(self, instance):
         super(CASubsystem, self).__init__(instance, 'ca')
 
-    def load_profile_registry(self):
-
-        registry = {}
-        registry_conf = os.path.join(self.conf_dir, 'registry.cfg')
-        pki.util.load_properties(registry_conf, registry)
-
-        return registry
-
     def get_profile_configs(self):
 
         profiles_dir = os.path.join(self.base_dir, 'profiles')
