@@ -32,6 +32,7 @@ import javax.servlet.ServletOutputStream;
 import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpServletResponse;
 
+import org.dogtag.util.cert.CertUtil;
 import org.dogtagpki.legacy.policy.IPolicyProcessor;
 import org.dogtagpki.server.authentication.AuthToken;
 import org.dogtagpki.server.authentication.IAuthSubsystem;
@@ -1339,7 +1340,7 @@ public class EnrollServlet extends CMSServlet {
                                     ILogger.SIGNED_AUDIT_ACCEPTANCE,
                                     issuedCerts[i]));
                     }
-                } catch (IOException ex) {
+                } catch (Exception ex) {
                     cmsReq.setStatus(ICMSRequest.ERROR);
 
                     // (automated "agent" cert request processed - "rejected")
@@ -1515,8 +1516,7 @@ public class EnrollServlet extends CMSServlet {
         }
     }
 
-    protected void renderServerEnrollResult(CMSRequest cmsReq) throws
-            IOException {
+    protected void renderServerEnrollResult(CMSRequest cmsReq) throws Exception {
         HttpServletResponse httpResp = cmsReq.getHttpResp();
 
         httpResp.setContentType("text/html");
@@ -1552,7 +1552,7 @@ public class EnrollServlet extends CMSServlet {
             X509CertImpl certs[] =
                     cmsReq.getIRequest().getExtDataInCertArray(IRequest.ISSUED_CERTS);
 
-            out.println(CertUtils.getEncodedCert(certs[0]));
+            out.println(CertUtil.toPEM(certs[0]));
             out.println("</PRE>");
             out.println("<P>");
             out.println("<!HTTP_OUTPUT REQUEST_CREATION_TIME=" +
@@ -1562,7 +1562,7 @@ public class EnrollServlet extends CMSServlet {
             out.println("<!HTTP_OUTPUT REQUEST_ID=" +
                     cmsReq.getIRequest().getRequestId().toString() + ">");
             out.println("<!HTTP_OUTPUT X509_CERTIFICATE=" +
-                    CertUtils.getEncodedCert(certs[0]) + ">");
+                    CertUtil.toPEM(certs[0]) + ">");
         } else if (cmsReq.getIRequest().getRequestStatus().equals(RequestStatus.PENDING)) {
             out.println("<H1>");
             out.println("PENDING");
