@@ -36,6 +36,7 @@ import javax.servlet.ServletOutputStream;
 import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpServletResponse;
 
+import org.dogtag.util.cert.CertUtil;
 import org.dogtagpki.server.authentication.AuthToken;
 import org.dogtagpki.server.authentication.IAuthManager;
 import org.dogtagpki.server.authentication.IAuthSubsystem;
@@ -598,7 +599,7 @@ public class HashEnrollServlet extends CMSServlet {
             try {
                 renderServerEnrollResult(cmsReq);
                 cmsReq.setStatus(ICMSRequest.SUCCESS); // no default render
-            } catch (IOException ex) {
+            } catch (Exception ex) {
                 cmsReq.setStatus(ICMSRequest.ERROR);
             }
             return;
@@ -1016,8 +1017,7 @@ public class HashEnrollServlet extends CMSServlet {
         }
     }
 
-    protected void renderServerEnrollResult(CMSRequest cmsReq) throws
-            IOException {
+    protected void renderServerEnrollResult(CMSRequest cmsReq) throws Exception {
         HttpServletResponse httpResp = cmsReq.getHttpResp();
 
         httpResp.setContentType("text/html");
@@ -1053,7 +1053,7 @@ public class HashEnrollServlet extends CMSServlet {
             X509CertImpl certs[] =
                     cmsReq.getIRequest().getExtDataInCertArray(IRequest.ISSUED_CERTS);
 
-            out.println(CertUtils.getEncodedCert(certs[0]));
+            out.println(CertUtil.toPEM(certs[0]));
             out.println("</PRE>");
             out.println("<P>");
             out.println("<!HTTP_OUTPUT REQUEST_CREATION_TIME=" +
@@ -1063,7 +1063,7 @@ public class HashEnrollServlet extends CMSServlet {
             out.println("<!HTTP_OUTPUT REQUEST_ID=" +
                     cmsReq.getIRequest().getRequestId().toString() + ">");
             out.println("<!HTTP_OUTPUT X509_CERTIFICATE=" +
-                    CertUtils.getEncodedCert(certs[0]) + ">");
+                    CertUtil.toPEM(certs[0]) + ">");
         } else if (cmsReq.getIRequest().getRequestStatus().equals(RequestStatus.PENDING)) {
             out.println("<H1>");
             out.println("PENDING");
