@@ -54,36 +54,24 @@ public class LDAPConfigurator {
     public final static Logger logger = LoggerFactory.getLogger(LDAPConfigurator.class);
 
     LDAPConnection connection;
+    LDAPConfig config;
+    String instanceID;
 
-    Map<String, String> params = new HashMap<>();
-
-    public LDAPConfigurator(LDAPConnection connection) throws Exception {
+    public LDAPConfigurator(LDAPConnection connection, LDAPConfig config) throws Exception {
         this.connection = connection;
+        this.config = config;
     }
 
-    public LDAPConfigurator(LDAPConnection connection, String instanceId, LDAPConfig ldapConfig) throws Exception {
+    public LDAPConfigurator(LDAPConnection connection, LDAPConfig config, String instanceID) throws Exception {
         this.connection = connection;
-
-        params.put("instanceId", instanceId);
-
-        String baseDN = ldapConfig.getBaseDN();
-        params.put("rootSuffix", baseDN);
-
-        String database = ldapConfig.getDatabase();
-        params.put("database", database);
-
-        String dbuser = ldapConfig.getDBUser("uid=pkidbuser,ou=people," + baseDN);
-        params.put("dbuser", dbuser);
+        this.config = config;
+        this.instanceID = instanceID;
     }
 
     public LDAPConnection getConnection() {
         return connection;
     }
 
-    public String getParam(String name) {
-        return params.get(name);
-    }
-
     public void configureDirectory() throws Exception {
         logger.info("Configuring directory");
         importFile("/usr/share/pki/server/conf/database.ldif", true);
@@ -95,8 +83,11 @@ public class LDAPConfigurator {
     }
 
     public void setupSchema() throws Exception {
-        logger.info("Setting up schema");
+        logger.info("Setting up PKI schema");
         importSchemaFile("/usr/share/pki/server/conf/schema.ldif");
+
+        logger.info("Setting up ACME schema");
+        importSchemaFile("/usr/share/pki/acme/database/ldap/schema.ldif");
     }
 
     public void createContainers(String subsystem) throws Exception {
@@ -357,6 +348,18 @@ public class LDAPConfigurator {
 
         logger.info("Creating " + tmpFile);
 
+        Map<String, String> params = new HashMap<>();
+        params.put("instanceId", instanceID);
+
+        String baseDN = config.getBaseDN();
+        params.put("rootSuffix", baseDN);
+
+        String database = config.getDatabase();
+        params.put("database", database);
+
+        String dbuser = config.getDBUser("uid=pkidbuser,ou=people," + baseDN);
+        params.put("dbuser", dbuser);
+
         try (BufferedReader in = new BufferedReader(new FileReader(file));
                 PrintWriter out = new PrintWriter(new FileWriter(tmpFile))) {
 
@@ -701,11 +704,13 @@ public class LDAPConfigurator {
         return "";
     }
 
-    public void createChangeLog(String dir) throws Exception {
+    public void createChangeLog() throws Exception {
 
         String dn = "cn=changelog5,cn=config";
         logger.info("Adding " + dn);
 
+        String dir = getInstanceDir() + "/changelogs";
+
         LDAPAttributeSet attrs = new LDAPAttributeSet();
         attrs.add(new LDAPAttribute("objectClass", "top"));
         attrs.add(new LDAPAttribute("objectClass", "extensibleObject"));
@@ -729,8 +734,10 @@ public class LDAPConfigurator {
         }
     }
 
-    public int enableReplication(String replicaDN, String bindUser, String baseDN, int id) throws Exception {
+    public boolean createReplicaObject(String bindUser, int id) throws Exception {
 
+        String baseDN = config.getBaseDN();
+        String replicaDN = "cn=replica,cn=\"" + baseDN + "\",cn=mapping tree,cn=config";
         String bindDN = "cn=" + LDAPUtil.escapeRDNValue(bindUser) + ",ou=csusers,cn=config";
 
         logger.info("Adding " + replicaDN);
@@ -762,7 +769,7 @@ public class LDAPConfigurator {
 
                 entry = connection.read(replicaDN);
                 LDAPAttribute attr = entry.getAttribute("nsDS5ReplicaBindDN");
-                attr.addValue("cn=" + LDAPUtil.escapeRDNValue(bindUser) + ",ou=csusers,cn=config");
+                attr.addValue(bindDN);
 
                 LDAPModification mod = new LDAPModification(LDAPModification.REPLACE, attr);
 
@@ -773,27 +780,27 @@ public class LDAPConfigurator {
                     logger.warn("Unable to modify " + replicaDN + ": " + ee.getMessage(), ee);
                 }
 
-                return id;
+                return false;
 
             } else {
                 logger.warn("Unable to add " + replicaDN + ": " + e.getMessage(), e);
-                return id;
+                return false;
             }
         }
 
-        return id + 1;
+        return true;
     }
 
     public void createReplicationAgreement(
-            String replicaDN,
             String name,
             String replicaHostname,
             int replicaPort,
-            String replicaPassword,
-            String baseDN,
             String bindUser,
+            String replicaPassword,
             String replicationSecurity) throws Exception {
 
+        String baseDN = config.getBaseDN();
+        String replicaDN = "cn=replica,cn=\"" + baseDN + "\",cn=mapping tree,cn=config";
         String dn = "cn=" + LDAPUtil.escapeRDNValue(name) + "," + replicaDN;
         String bindDN = "cn=" + LDAPUtil.escapeRDNValue(bindUser) + ",ou=csusers,cn=config";
 
@@ -853,8 +860,39 @@ public class LDAPConfigurator {
         }
     }
 
-    public void initializeConsumer(String replicaDN, String agreementName) throws Exception {
+    public boolean setupReplicationAgreement(
+            String agreementName,
+            String bindUser,
+            String bindPassword,
+            String peerHostname,
+            int peerPort,
+            String peerBindUser,
+            String peerBindPassword,
+            String security,
+            int replicaID)
+            throws Exception {
 
+        createSystemContainer();
+        createReplicationManager(bindUser, bindPassword);
+        createChangeLog();
+
+        boolean created = createReplicaObject(bindUser, replicaID);
+
+        createReplicationAgreement(
+                agreementName,
+                peerHostname,
+                peerPort,
+                peerBindUser,
+                peerBindPassword,
+                security);
+
+        return created;
+    }
+
+    public void initializeConsumer(String agreementName) throws Exception {
+
+        String baseDN = config.getBaseDN();
+        String replicaDN = "cn=replica,cn=\"" + baseDN + "\",cn=mapping tree,cn=config";
         String dn = "cn=" + LDAPUtil.escapeRDNValue(agreementName) + "," + replicaDN;
         logger.info("Initializing consumer " + dn);
 
