@@ -41,7 +41,6 @@ import org.apache.commons.lang.StringUtils;
 import org.dogtagpki.server.ca.ICertificateAuthority;
 import org.mozilla.jss.CryptoManager;
 import org.mozilla.jss.crypto.CryptoToken;
-import org.mozilla.jss.crypto.ObjectNotFoundException;
 import org.mozilla.jss.crypto.PrivateKey;
 import org.mozilla.jss.crypto.Signature;
 import org.mozilla.jss.crypto.SignatureAlgorithm;
@@ -97,7 +96,7 @@ import com.netscape.cmscore.usrgrp.UGSubsystem;
 import com.netscape.cmscore.util.Debug;
 import com.netscape.cmsutil.crypto.CryptoUtil;
 import com.netscape.cmsutil.password.IPasswordStore;
-import com.netscape.cmsutil.password.NuxwdogPasswordStore;
+import com.netscape.cmsutil.util.NuxwdogUtil;
 
 import netscape.ldap.LDAPConnection;
 import netscape.ldap.LDAPException;
@@ -126,7 +125,6 @@ public class CMSEngine {
     private boolean mAutoSD_Restart = false;
     private int mAutoSD_RestartMax = 3;
     private int mAutoSD_RestartCount = 0;
-    private String mSAuditCertNickName = null;
     private PrivateKey mSigningKey = null;
     private byte[] mSigningData = null;
     private long mStartupTime = 0;
@@ -185,34 +183,16 @@ public class CMSEngine {
         return instanceDir;
     }
 
-    public boolean startedByNuxwdog() {
-        String wdPipeName = System.getenv("WD_PIPE_NAME");
-        if (StringUtils.isNotEmpty(wdPipeName)) {
-            return true;
-        }
-        return false;
-    }
-
     public synchronized IPasswordStore getPasswordStore() throws EBaseException {
         if (mPasswordStore == null) {
-            String pwdClass = null;
-            String pwdPath = null;
-
-            if (startedByNuxwdog()) {
-                pwdClass = NuxwdogPasswordStore.class.getName();
-                // note: pwdPath is expected to be null in this case
-            } else {
-                pwdClass = mConfig.getString("passwordClass");
-                pwdPath = mConfig.getString("passwordFile", null);
-            }
-
             try {
-                mPasswordStore = (IPasswordStore) Class.forName(pwdClass).newInstance();
-                mPasswordStore.init(pwdPath);
-                mPasswordStore.setId(instanceId);
+                /* mConfig.getProperties() is O(n), but we cache the returned
+                 * password store so this is fine */
+                mPasswordStore =
+                    IPasswordStore.getPasswordStore(instanceId, mConfig.getProperties());
             } catch (Exception e) {
-                logger.error("Cannot get password store: " + e);
-                throw new EBaseException(e);
+                throw new EBaseException(
+                    "Failed to initialise password store: " + e.getMessage(), e);
             }
         }
         return mPasswordStore;
@@ -467,7 +447,10 @@ public class CMSEngine {
         loadSubsystems();
         initSubsystems();
 
-        configureAutoShutdown();
+        if (!isPreOpMode()) {
+            configureAutoShutdown();
+        }
+
         configureServerCertNickname();
         configureExcludedLdapAttrs();
 
@@ -824,70 +807,53 @@ public class CMSEngine {
         ss.init(ssConfig);
     }
 
-    public void configureAutoShutdown() throws EBaseException {
+    public void configureAutoShutdown() throws Exception {
 
         logger.info("CMSEngine: Configuring auto shutdown");
 
-        try {
-            /*
-             * autoShutdown.allowed=false
-             * autoShutdown.crumbFile=[PKI_INSTANCE_PATH]/logs/autoShutdown.crumb
-             * autoShutdown.restart.enable=false
-             * autoShutdown.restart.max=3
-             * autoShutdown.restart.count=0
-             */
+        /*
+         * autoShutdown.allowed=false
+         * autoShutdown.crumbFile=[PKI_INSTANCE_PATH]/logs/autoShutdown.crumb
+         * autoShutdown.restart.enable=false
+         * autoShutdown.restart.max=3
+         * autoShutdown.restart.count=0
+         */
 
-            mAutoSD_Restart = mConfig.getBoolean("autoShutdown.restart.enable", false);
-            logger.debug("CMSEngine: restart at autoShutdown: " + mAutoSD_Restart);
+        mAutoSD_Restart = mConfig.getBoolean("autoShutdown.restart.enable", false);
+        logger.debug("CMSEngine: restart at autoShutdown: " + mAutoSD_Restart);
 
-            if (mAutoSD_Restart) {
-                mAutoSD_RestartMax = mConfig.getInteger("autoShutdown.restart.max", 3);
-                logger.debug("CMSEngine: restart max: " + mAutoSD_RestartMax);
+        if (mAutoSD_Restart) {
+            mAutoSD_RestartMax = mConfig.getInteger("autoShutdown.restart.max", 3);
+            logger.debug("CMSEngine: restart max: " + mAutoSD_RestartMax);
 
-                mAutoSD_RestartCount = mConfig.getInteger("autoShutdown.restart.count", 0);
-                logger.debug("CMSEngine: current restart count: " + mAutoSD_RestartCount);
+            mAutoSD_RestartCount = mConfig.getInteger("autoShutdown.restart.count", 0);
+            logger.debug("CMSEngine: current restart count: " + mAutoSD_RestartCount);
 
-            } else { //!mAutoSD_Restart
-                mAutoSD_CrumbFile = mConfig.getString("autoShutdown.crumbFile",
-                    instanceDir + "/logs/autoShutdown.crumb");
-                logger.debug("CMSEngine: autoShutdown crumb file path: " + mAutoSD_CrumbFile);
+        } else { //!mAutoSD_Restart
 
-                File crumb = new File(mAutoSD_CrumbFile);
-                try {
-                    if (crumb.exists()) {
-                        logger.debug("CMSEngine: delete autoShutdown crumb file");
-                        crumb.delete();
-                    }
-                } catch (Exception e) {
-                    logger.warn("Delete autoShutdown crumb file failed: " + e.getMessage(), e);
-                    logger.warn("Continue with initialization");
-                }
+            mAutoSD_CrumbFile = mConfig.getString("autoShutdown.crumbFile",
+                instanceDir + "/logs/autoShutdown.crumb");
+            logger.info("CMSEngine: auto-shutdown crumb file: " + mAutoSD_CrumbFile);
+
+            File crumb = new File(mAutoSD_CrumbFile);
+            if (crumb.exists()) {
+                logger.info("CMSEngine: deleting auto-shutdown crumb file: " + mAutoSD_CrumbFile);
+                crumb.delete();
             }
-
-            /*
-             * establish signing key reference using audit signing cert
-             * for HSM failover detection
-             */
-            mSAuditCertNickName = mConfig.getString(PROP_SIGNED_AUDIT_CERT_NICKNAME);
-            logger.debug("CMSEngine: about to look for cert for auto-shutdown support:" + mSAuditCertNickName);
-
-            CryptoManager mManager = CryptoManager.getInstance();
-            org.mozilla.jss.crypto.X509Certificate cert = null;
-            try {
-                cert = mManager.findCertByNickname(mSAuditCertNickName);
-            } catch (ObjectNotFoundException as) {
-                logger.warn("CMSEngine: Unable to support auto-shutdown: " + as.getMessage());
-            }
-
-            if (cert != null) {
-                logger.debug("CMSEngine: found cert:" + mSAuditCertNickName);
-                mSigningKey = mManager.findPrivKeyByCert(cert);
-                mSigningData = cert.getPublicKey().getEncoded();
-            }
-
-        } catch (Exception e) {
-            logger.warn("CMSEngine: Unable to configure auto-shutdown: " + e.getMessage(), e);
         }
+
+        /*
+         * establish signing key reference using audit signing cert
+         * for HSM failover detection
+         */
+        String mSAuditCertNickName = mConfig.getString(PROP_SIGNED_AUDIT_CERT_NICKNAME);
+        logger.debug("CMSEngine: audit signing cert: " + mSAuditCertNickName);
+
+        CryptoManager mManager = CryptoManager.getInstance();
+        org.mozilla.jss.crypto.X509Certificate cert = mManager.findCertByNickname(mSAuditCertNickName);
+
+        mSigningKey = mManager.findPrivKeyByCert(cert);
+        mSigningData = cert.getPublicKey().getEncoded();
     }
 
     public void configureServerCertNickname() throws EBaseException {
@@ -1189,7 +1155,7 @@ public class CMSEngine {
             cmds = new String[3];
             cmds[0] = "/usr/bin/systemctl";
             cmds[1] = cmd;
-            if (startedByNuxwdog()) {
+            if (NuxwdogUtil.startedByNuxwdog()) {
                 cmds[2] = "pki-tomcatd-nuxwdog@" + instanceId + ".service";
             } else {
                 cmds[2] = "pki-tomcatd@" + instanceId + ".service";
