@@ -19,6 +19,7 @@ package com.netscape.cmscore.cert;
 
 import java.io.ByteArrayInputStream;
 import java.io.FileInputStream;
+import java.io.FileNotFoundException;
 import java.io.FileOutputStream;
 import java.io.IOException;
 import java.math.BigInteger;
@@ -34,9 +35,19 @@ import java.security.cert.X509CRL;
 import java.security.cert.X509Certificate;
 import java.util.Arrays;
 import java.util.Date;
+import java.util.Enumeration;
+import java.util.Iterator;
 import java.util.Locale;
+import java.util.Properties;
+import java.util.Set;
 import java.util.StringTokenizer;
 
+import javax.ws.rs.core.MultivaluedMap;
+
+import org.apache.commons.lang.ArrayUtils;
+import org.apache.commons.lang.StringUtils;
+import org.dogtag.util.cert.CertUtil;
+import org.dogtagpki.server.ca.ICertificateAuthority;
 import org.mozilla.jss.CertificateUsage;
 import org.mozilla.jss.CryptoManager;
 import org.mozilla.jss.asn1.SEQUENCE;
@@ -56,24 +67,39 @@ import org.mozilla.jss.netscape.security.x509.CertificateIssuerName;
 import org.mozilla.jss.netscape.security.x509.CertificateSerialNumber;
 import org.mozilla.jss.netscape.security.x509.CertificateValidity;
 import org.mozilla.jss.netscape.security.x509.CertificateVersion;
+import org.mozilla.jss.netscape.security.x509.Extension;
 import org.mozilla.jss.netscape.security.x509.X500Name;
 import org.mozilla.jss.netscape.security.x509.X509CRLImpl;
 import org.mozilla.jss.netscape.security.x509.X509CertImpl;
 import org.mozilla.jss.netscape.security.x509.X509CertInfo;
 import org.mozilla.jss.netscape.security.x509.X509Key;
 import org.mozilla.jss.pkix.crmf.CertReqMsg;
+import org.xml.sax.SAXException;
 
 import com.netscape.certsrv.base.EBaseException;
+import com.netscape.certsrv.base.EPropertyNotFound;
+import com.netscape.certsrv.base.IConfigStore;
+import com.netscape.certsrv.base.MetaInfo;
+import com.netscape.certsrv.client.PKIClient;
+import com.netscape.certsrv.dbs.certdb.ICertRecord;
+import com.netscape.certsrv.dbs.certdb.ICertificateRepository;
 import com.netscape.certsrv.logging.AuditEvent;
 import com.netscape.certsrv.logging.ILogger;
 import com.netscape.certsrv.logging.LogEvent;
 import com.netscape.certsrv.profile.EProfileException;
+import com.netscape.certsrv.request.IRequest;
+import com.netscape.certsrv.request.IRequestQueue;
+import com.netscape.certsrv.request.RequestId;
+import com.netscape.certsrv.request.RequestStatus;
 import com.netscape.cms.logging.Logger;
 import com.netscape.cms.logging.SignedAuditLogger;
+import com.netscape.cms.profile.common.EnrollProfile;
+import com.netscape.cms.servlet.csadmin.CertInfoProfile;
 import com.netscape.cmscore.apps.CMS;
 import com.netscape.cmscore.apps.CMSEngine;
 import com.netscape.cmscore.apps.EngineConfig;
 import com.netscape.cmsutil.crypto.CryptoUtil;
+import com.netscape.cmsutil.xml.XMLObject;
 
 /**
  * Utility class with assorted methods to check for
@@ -97,38 +123,6 @@ public class CertUtils {
     public static final String END_CRL_HEADER =
             "-----END CERTIFICATE REVOCATION LIST-----";
 
-    public static String normalizeCertReq(String s) {
-
-        if (s == null) {
-            return s;
-        }
-
-        s = s.replaceAll(org.mozilla.jss.netscape.security.util.Cert.REQUEST_HEADER, "");
-        s = s.replaceAll("-----BEGIN NEW CERTIFICATE REQUEST-----", "");
-        s = s.replaceAll(org.mozilla.jss.netscape.security.util.Cert.REQUEST_FOOTER, "");
-        s = s.replaceAll("-----END NEW CERTIFICATE REQUEST-----", "");
-
-        StringBuffer sb = new StringBuffer();
-        StringTokenizer st = new StringTokenizer(s, "\r\n ");
-
-        while (st.hasMoreTokens()) {
-            String nextLine = st.nextToken();
-
-            nextLine = nextLine.trim();
-            if (nextLine.equals(org.mozilla.jss.netscape.security.util.Cert.REQUEST_HEADER))
-                continue;
-            if (nextLine.equals("-----BEGIN NEW CERTIFICATE REQUEST-----"))
-                continue;
-            if (nextLine.equals(org.mozilla.jss.netscape.security.util.Cert.REQUEST_FOOTER))
-                continue;
-            if (nextLine.equals("-----END NEW CERTIFICATE REQUEST-----"))
-                continue;
-            sb.append(nextLine);
-        }
-
-        return sb.toString();
-    }
-
     public static DerInputStream parseKeyGen(Locale locale, String certreq) throws Exception {
         byte[] data = Utils.base64decode(certreq);
         return new DerInputStream(data);
@@ -225,8 +219,7 @@ public class CertUtils {
 
         logger.debug(certreq);
 
-        String creq = normalizeCertReq(certreq);
-        byte[] data = Utils.base64decode(creq);
+        byte[] data = CertUtil.parseCSR(certreq);
 
         CMSEngine engine = CMS.getCMSEngine();
         EngineConfig cs = engine.getConfig();
@@ -274,10 +267,9 @@ public class CertUtils {
             throw new EProfileException(CMS.getUserMessage(locale, "CMS_PROFILE_INVALID_REQUEST"));
         }
 
-        String creq = normalizeCertReq(certreq);
+        byte[] data = CertUtil.parseCSR(certreq);
 
         try {
-            byte[] data = Utils.base64decode(creq);
             ByteArrayInputStream crmfBlobIn = new ByteArrayInputStream(data);
             SEQUENCE crmfMsgs = (SEQUENCE) new SEQUENCE.OF_Template(
                     new CertReqMsg.Template()).decode(crmfBlobIn);
@@ -743,16 +735,6 @@ public class CertUtils {
         return crl;
     }
 
-    public static String getEncodedCert(X509Certificate cert) {
-        try {
-            return Cert.HEADER + "\n" +
-                    Utils.base64encode(cert.getEncoded(), true) +
-                    Cert.FOOTER + "\n";
-        } catch (Exception e) {
-            return null;
-        }
-    }
-
     public static String normalizeCertStr(String s) {
         StringBuffer val = new StringBuffer();
 
@@ -959,6 +941,399 @@ public class CertUtils {
         return tmp.toString();
     }
 
+    // Dynamically apply the SubjectAlternativeName extension to a
+    // remote PKI instance's request for its SSL Server Certificate.
+    //
+    // Since this information may vary from instance to
+    // instance, obtain the necessary information from the
+    // 'service.sslserver.san' value(s) in the instance's
+    // CS.cfg, process these values converting each item into
+    // its individual SubjectAlternativeName components, and
+    // build an SSL Server Certificate URL extension consisting
+    // of this information.
+    //
+    // 03/27/2013 - Should consider removing this
+    //              "buildSANSSLserverURLExtension()"
+    //              method if it becomes possible to
+    //              embed a certificate extension into
+    //              a PKCS #10 certificate request.
+    //
+    public static void buildSANSSLserverURLExtension(IConfigStore config, MultivaluedMap<String, String> content)
+           throws Exception {
+
+        logger.debug("CertUtils: buildSANSSLserverURLExtension() " +
+                  "building SAN SSL Server Certificate URL extension . . .");
+
+        if (config == null) {
+            throw new EBaseException("injectSANextensionIntoRequest: parameter config cannot be null");
+        }
+
+        String sanHostnames = config.getString("service.sslserver.san");
+        String sans[] = StringUtils.split(sanHostnames, ",");
+
+        int i = 0;
+        for (String san : sans) {
+            logger.debug("CertUtils: buildSANSSLserverURLExtension() processing " +
+                      "SAN hostname: " + san);
+            // Add the DNSName for all SANs
+            content.putSingle("req_san_pattern_" + i, san);
+            i++;
+        }
+
+        content.putSingle("req_san_entries", "" + i);
+
+        logger.debug("CertUtils: buildSANSSLserverURLExtension() " + "placed " +
+                  i + " SAN entries into SSL Server Certificate URL.");
+    }
+
+    /*
+     * create requests so renewal can work on these initial certs
+     */
+    public static IRequest createLocalRequest(
+            IRequestQueue queue,
+            CertInfoProfile profile,
+            X509CertInfo info,
+            X509Key x509key,
+            String[] sanHostnames,
+            boolean installAdjustValidity)
+            throws Exception {
+
+        //        RequestId rid = new RequestId(serialNum);
+        // just need a request, no need to get into a queue
+        //        IRequest r = new EnrollmentRequest(rid);
+
+        logger.info("CertUtils: Creating local request");
+
+        IRequest req = queue.newRequest("enrollment");
+
+        req.setExtData("profile", "true");
+        req.setExtData("requestversion", "1.0.0");
+        req.setExtData("req_seq_num", "0");
+
+        req.setExtData(EnrollProfile.REQUEST_CERTINFO, info);
+        req.setExtData(EnrollProfile.REQUEST_EXTENSIONS, new CertificateExtensions());
+
+        req.setExtData("requesttype", "enrollment");
+        req.setExtData("requestor_name", "");
+        req.setExtData("requestor_email", "");
+        req.setExtData("requestor_phone", "");
+        req.setExtData("profileRemoteHost", "");
+        req.setExtData("profileRemoteAddr", "");
+        req.setExtData("requestnotes", "");
+        req.setExtData("isencryptioncert", "false");
+        req.setExtData("profileapprovedby", "system");
+
+        if (sanHostnames != null) {
+
+            logger.info("CertUtils: Injecting SAN extension:");
+
+            // Dynamically inject the SubjectAlternativeName extension to a
+            // local/self-signed master CA's request for its SSL Server Certificate.
+            //
+            // Since this information may vary from instance to
+            // instance, obtain the necessary information from the
+            // 'service.sslserver.san' value(s) in the instance's
+            // CS.cfg, process these values converting each item into
+            // its individual SubjectAlternativeName components, and
+            // inject these values into the local request.
+
+            int i = 0;
+            for (String sanHostname : sanHostnames) {
+                logger.info("CertUtils: - " + sanHostname);
+                req.setExtData("req_san_pattern_" + i, sanHostname);
+                i++;
+            }
+        }
+
+        req.setExtData("req_key", x509key.toString());
+
+        String origProfileID = profile.getID();
+        int idx = origProfileID.lastIndexOf('.');
+        if (idx > 0) {
+            origProfileID = origProfileID.substring(0, idx);
+        }
+
+        // store original profile id in cert request
+        req.setExtData("origprofileid", origProfileID);
+
+        // store mapped profile ID for use in renewal
+        req.setExtData("profileid", profile.getProfileIDMapping());
+        req.setExtData("profilesetid", profile.getProfileSetIDMapping());
+
+        if (installAdjustValidity) {
+            /*
+             * (applies to non-CA-signing cert only)
+             * installAdjustValidity tells ValidityDefault to adjust the
+             * notAfter value to that of the CA's signing cert if needed
+             */
+            req.setExtData("installAdjustValidity", "true");
+        }
+
+        // mark request as complete
+        logger.debug("CertUtils: calling setRequestStatus");
+        req.setRequestStatus(RequestStatus.COMPLETE);
+
+        return req;
+    }
+
+    /**
+     * update local cert request with the actual request
+     * called from CertRequestPanel.java
+     * @throws EBaseException
+     * @throws EPropertyNotFound
+     */
+    public static void updateLocalRequest(
+            String reqId,
+            byte[] certReq,
+            String reqType,
+            String subjectName
+            ) throws Exception {
+
+        logger.debug("CertUtils: updateLocalRequest(" + reqId + ")");
+
+        CMSEngine engine = CMS.getCMSEngine();
+        ICertificateAuthority ca = (ICertificateAuthority) engine.getSubsystem(ICertificateAuthority.ID);
+        IRequestQueue queue = ca.getRequestQueue();
+
+        IRequest req = queue.findRequest(new RequestId(reqId));
+
+        if (certReq != null) {
+            logger.debug("CertUtils: updating cert request");
+            String certReqs = CryptoUtil.base64Encode(certReq);
+            String certReqf = CryptoUtil.reqFormat(certReqs);
+            req.setExtData("cert_request", certReqf);
+        }
+
+        req.setExtData("cert_request_type", reqType);
+
+        if (subjectName != null) {
+            logger.debug("CertUtils: updating request subject: " + subjectName);
+            req.setExtData("subject", subjectName);
+            new X500Name(subjectName); // check for errors
+        }
+
+        queue.updateRequest(req);
+    }
+
+    public static X509CertInfo createCertInfo(
+            String dn,
+            String issuerdn,
+            String keyAlgorithm,
+            X509Key x509key,
+            String type) throws Exception {
+
+        logger.info("CertUtils: Creating certificate info for " + dn);
+
+        Date date = new Date();
+
+        CMSEngine engine = CMS.getCMSEngine();
+        ICertificateAuthority ca = (ICertificateAuthority) engine.getSubsystem(ICertificateAuthority.ID);
+        ICertificateRepository cr = ca.getCertificateRepository();
+        BigInteger serialNo = cr.getNextSerialNumber();
+
+        X509CertInfo info;
+
+        if (type.equals("selfsign")) {
+
+            logger.debug("CertUtils: Creating self-signed certificate");
+            CertificateIssuerName issuerdnObj = new CertificateIssuerName(new X500Name(dn));
+            info = CryptoUtil.createX509CertInfo(x509key, serialNo, issuerdnObj, dn, date, date, keyAlgorithm);
+
+        } else {
+
+            logger.debug("CertUtils: Creating CA-signed certificate");
+            CertificateIssuerName issuerdnObj = ca.getIssuerObj();
+
+            if (issuerdnObj != null) {
+
+                logger.debug("CertUtils: Reusing CA's CertificateIssuerName to preserve the DN encoding");
+                info = CryptoUtil.createX509CertInfo(x509key, serialNo, issuerdnObj, dn, date, date, keyAlgorithm);
+
+            } else {
+
+                logger.debug("CertUtils: Creating new CertificateIssuerName");
+                issuerdnObj = new CertificateIssuerName(new X500Name(issuerdn));
+                info = CryptoUtil.createX509CertInfo(x509key, serialNo, issuerdnObj, dn, date, date, keyAlgorithm);
+            }
+        }
+
+        logger.info("CertUtils: Cert info:\n" + info);
+        return info;
+    }
+
+    public static void createCertRecord(
+            IRequest request,
+            CertInfoProfile profile,
+            X509CertImpl cert) throws Exception {
+
+        logger.debug("CertUtils: createCertRecord(" +
+                cert.getSerialNumber() + ", " +
+                cert.getSubjectDN() + ")");
+
+        CMSEngine engine = CMS.getCMSEngine();
+        ICertificateAuthority ca = (ICertificateAuthority) engine.getSubsystem(ICertificateAuthority.ID);
+        ICertificateRepository cr = ca.getCertificateRepository();
+
+        MetaInfo meta = new MetaInfo();
+        meta.set(ICertRecord.META_REQUEST_ID, request.getRequestId().toString());
+        meta.set(ICertRecord.META_PROFILE_ID, profile.getProfileIDMapping());
+
+        ICertRecord record = cr.createCertRecord(cert.getSerialNumber(), cert, meta);
+        cr.addCertificateRecord(record);
+    }
+
+    public static void createCertRecord(
+            IRequest request,
+            CertInfoProfile profile,
+            X509Certificate cert) throws Exception {
+
+        X509CertImpl certImpl = new X509CertImpl(cert.getEncoded());
+        createCertRecord(request, profile, certImpl);
+    }
+
+    public static X509CertImpl createLocalCert(
+            IRequest req,
+            CertInfoProfile profile,
+            X509CertInfo info,
+            java.security.PrivateKey signingPrivateKey,
+            String caSigningKeyAlgo) throws Exception {
+
+        profile.populate(req, info);
+
+        X509CertImpl cert = CryptoUtil.signCert(signingPrivateKey, info, caSigningKeyAlgo);
+
+        createCertRecord(req, profile, cert);
+
+        // update request with cert
+        req.setExtData(EnrollProfile.REQUEST_ISSUED_CERT, cert);
+
+        return cert;
+    }
+
+    public static X509CertImpl createRemoteCert(
+            PKIClient client,
+            MultivaluedMap<String, String> content)
+            throws Exception {
+
+        logger.debug("CertUtils: Calling profileSubmit");
+        logger.debug("CertUtils: content: " + content);
+
+        String c = client.post("/ca/ee/ca/profileSubmit", content);
+
+        if (c == null) {
+            logger.error("CertUtils: Missing CA response");
+            throw new Exception("Missing CA response");
+        }
+
+        ByteArrayInputStream bis = new ByteArrayInputStream(c.getBytes());
+        XMLObject parser;
+        try {
+            parser = new XMLObject(bis);
+        } catch (SAXException e) {
+            logger.error("Response: " + c);
+            logger.error("CertUtils: Unable to parse XML response: " + e, e);
+            throw e;
+        }
+
+        String status = parser.getValue("Status");
+        logger.debug("CertUtils: status: " + status);
+
+        if (!status.equals("0")) {
+            String error = parser.getValue("Error");
+            logger.error("CertUtils: error: " + error);
+            throw new IOException(error);
+        }
+
+        String b64 = parser.getValue("b64");
+        logger.debug("CertUtils: cert: " + b64);
+
+        b64 = CryptoUtil.normalizeCertAndReq(b64);
+        byte[] b = CryptoUtil.base64Decode(b64);
+
+        return new X509CertImpl(b);
+    }
+
+    public static boolean isAlgorithmValid(String signingKeyType, String algorithm) {
+       return ((signingKeyType.equals("rsa") && algorithm.contains("RSA")) ||
+               (signingKeyType.equals("ecc") && algorithm.contains("EC"))  ||
+               (signingKeyType.equals("dsa") && algorithm.contains("DSA")));
+    }
+
+    /**
+     * reads from the admin cert profile caAdminCert.profile and determines the algorithm as follows:
+     *
+     * 1.  First gets list of allowed algorithms from profile (constraint.params.signingAlgsAllowed)
+     *     If entry does not exist, uses entry "ca.profiles.defaultSigningAlgsAllowed" from CS.cfg
+     *     If that entry does not exist, uses basic default
+     *
+     * 2.  Gets default.params.signingAlg from profile.
+     *     If entry does not exist or equals "-", selects first algorithm in allowed algorithm list
+     *     that matches CA signing key type
+     *     Otherwise returns entry if it matches signing CA key type.
+     *
+     * @throws EBaseException
+     * @throws IOException
+     * @throws FileNotFoundException
+     */
+
+    public static String getAdminProfileAlgorithm(
+            String caSigningKeyType,
+            String profileFilename,
+            String defaultSigningAlgsAllowed) throws Exception {
+
+        Properties props = new Properties();
+        props.load(new FileInputStream(profileFilename));
+
+        Set<String> keys = props.stringPropertyNames();
+        Iterator<String> iter = keys.iterator();
+        String defaultAlg = null;
+        String[] algsAllowed = null;
+
+        while (iter.hasNext()) {
+            String key = iter.next();
+            if (key.endsWith("default.params.signingAlg")) {
+                defaultAlg = props.getProperty(key);
+            }
+            if (key.endsWith("constraint.params.signingAlgsAllowed")) {
+                algsAllowed = StringUtils.split(props.getProperty(key), ",");
+            }
+        }
+
+        if (algsAllowed == null) { //algsAllowed not defined in profile, use a global setting
+            algsAllowed = StringUtils.split(defaultSigningAlgsAllowed, ",");
+        }
+
+        if (ArrayUtils.isEmpty(algsAllowed)) {
+            throw new EBaseException("No allowed signing algorithms defined.");
+        }
+
+        if (StringUtils.isNotEmpty(defaultAlg) && !defaultAlg.equals("-")) {
+            // check if the defined default algorithm is valid
+            if (! isAlgorithmValid(caSigningKeyType, defaultAlg)) {
+                throw new EBaseException("Administrator cert cannot be signed by specfied algorithm." +
+                                         "Algorithm incompatible with signing key");
+            }
+
+            for (String alg : algsAllowed) {
+                if (defaultAlg.trim().equals(alg.trim())) {
+                    return defaultAlg;
+                }
+            }
+            throw new EBaseException(
+                    "Administrator Certificate cannot be signed by the specified algorithm " +
+                    "as it is not one of the allowed signing algorithms.  Check the admin cert profile.");
+        }
+
+        // no algorithm specified.  Pick the first allowed algorithm.
+        for (String alg : algsAllowed) {
+            if (isAlgorithmValid(caSigningKeyType, alg)) return alg;
+        }
+
+        throw new EBaseException(
+                "Admin certificate cannot be signed by any of the specified possible algorithms." +
+                "Algorithm is incompatible with the CA signing key type" );
+    }
+
     public static void verifySystemCertValidityByNickname(String nickname) throws Exception {
 
         logger.info("CertUtils: Validating certificate " + nickname);
@@ -1269,6 +1644,104 @@ public class CertUtils {
         return null;
     }
 
+    public static void printRequestContent(IRequest request) {
+        String method = "CertUtils.printRequestContent: ";
+        logger.debug(method + "Content of request: ");
+        Enumeration<String> ereq = request.getExtDataKeys();
+        while (ereq.hasMoreElements()) {
+            String reqKey = ereq.nextElement();
+            String reqVal = request.getExtDataInString(reqKey);
+            if (reqVal != null) {
+                logger.debug("  req entry - " + reqKey + ": " + reqVal);
+            } else {
+                logger.debug("  req entry - " + reqKey + ": no value");
+            }
+        }
+    }
+
+    /*
+     * addCTpoisonExt adds the Certificate Transparency V1 poison extension
+     * to the Ceritificate Info
+     *
+     * @param certinfo X509CertInfo where the poison extension is to be added
+     *
+     * @author cfu
+     */
+    public static final String CT_POISON_OID = "1.3.6.1.4.1.11129.2.4.3";
+    public static final boolean CT_POISON_CRITICAL = true;
+    public static final byte CT_POISON_DATA[] =  new byte[] { 0x05, 0x00 };
+
+    public static void addCTv1PoisonExt(X509CertInfo certinfo)
+                throws CertificateException, IOException, EBaseException {
+        String method = "CryptoUtil:addCTv1PoisonExt: ";
+        ObjectIdentifier ct_poison_oid = new ObjectIdentifier(CT_POISON_OID);
+        Extension ct_poison_ext = null;
+        CertificateExtensions exts =  null;
+
+        exts = (CertificateExtensions)
+                certinfo.get(X509CertInfo.EXTENSIONS);
+        if (exts == null) {
+            logger.debug(method + " X509CertInfo.EXTENSIONS not found inf cetinfo");
+            throw new EBaseException(CMS.getUserMessage("CMS_BASE_INTERNAL_ERROR", " X509CertInfo.EXTENSIONS not found inf cetinfo"));
+        }
+        DerOutputStream out = new DerOutputStream();
+        out.putOctetString(CT_POISON_DATA);
+        ct_poison_ext = new Extension(ct_poison_oid, CT_POISON_CRITICAL, out.toByteArray());
+        //System.out.println(method + " ct_poison_ext id = " +
+        //        ct_poison_ext.getExtensionId().toString());
+        certinfo.set(X509CertInfo.EXTENSIONS, exts);
+
+        exts.set(CT_POISON_OID, ct_poison_ext);
+        certinfo.delete(X509CertInfo.EXTENSIONS);
+        certinfo.set(X509CertInfo.EXTENSIONS, exts);
+    }
+
+    /*
+     * for debugging
+     */
+    public static void printExtensions(CertificateExtensions exts) {
+
+        String method = "CryptoUtil.printExtensions: ";
+        System.out.println(method + "begins");
+        try {
+            if (exts == null)
+                return;
+
+            Enumeration<String> e = exts.getNames();
+            while (e.hasMoreElements()) {
+                String n = e.nextElement();
+                Extension ext = (Extension) exts.get(n);
+
+                System.out.println(" ---- " + ext.getExtensionId().toString());
+            }
+        } catch (Exception e) {
+            System.out.println(method + e.toString());
+        }
+        System.out.println(method + "ends");
+    }
+
+    /**
+     * Write the int as a big-endian byte[] of fixed width (in bytes).
+     */
+    public static byte[] intToFixedWidthBytes(int n, int width) {
+        byte[] out = new byte[width];
+        for (int i = 0; i < width; i++) {
+            out[i] = (byte) (n >> ((width - i - 1) * 8));
+        }
+        return out;
+    }
+
+    /*
+     * from byte array to hex in String
+     */
+    public static String bytesToHex(byte[] bytes) {
+        final StringBuilder sb = new StringBuilder();
+        for(byte b : bytes) {
+            sb.append(String.format("%02x", b));
+        }
+        return sb.toString();
+    }
+
     /**
      * Signed Audit Log
      * This method is called to store messages to the signed audit log.
@@ -1282,4 +1755,14 @@ public class CertUtils {
     protected void audit(LogEvent event) {
         signedAuditLogger.log(event);
     }
+
+    public static boolean certInCertChain(X509Certificate[] certChain, X509Certificate cert) {
+
+        for (X509Certificate c : certChain) {
+            if (!cert.equals(c)) continue;
+            return true;
+        }
+
+        return false;
+    }
 }
