@@ -19,41 +19,47 @@ package org.dogtagpki.server.rest;
 
 import java.security.KeyPair;
 import java.security.Principal;
-import java.security.PublicKey;
 
 import org.apache.commons.lang.StringUtils;
 import org.dogtagpki.server.ca.ICertificateAuthority;
 import org.mozilla.jss.CryptoManager;
 import org.mozilla.jss.crypto.CryptoToken;
 import org.mozilla.jss.crypto.ObjectNotFoundException;
-import org.mozilla.jss.crypto.PrivateKey;
 import org.mozilla.jss.crypto.X509Certificate;
+import org.mozilla.jss.netscape.security.pkcs.PKCS10;
 import org.mozilla.jss.netscape.security.util.Utils;
 import org.mozilla.jss.netscape.security.x509.X509CertImpl;
+import org.mozilla.jss.netscape.security.x509.X509CertInfo;
+import org.mozilla.jss.netscape.security.x509.X509Key;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
 import com.netscape.certsrv.base.BadRequestException;
 import com.netscape.certsrv.base.PKIException;
+import com.netscape.certsrv.request.IRequest;
+import com.netscape.certsrv.request.IRequestQueue;
+import com.netscape.certsrv.request.RequestId;
 import com.netscape.certsrv.system.AdminSetupRequest;
 import com.netscape.certsrv.system.AdminSetupResponse;
 import com.netscape.certsrv.system.CertificateSetupRequest;
 import com.netscape.certsrv.system.CloneSetupRequest;
-import com.netscape.certsrv.system.ConfigurationRequest;
 import com.netscape.certsrv.system.DatabaseSetupRequest;
 import com.netscape.certsrv.system.DatabaseUserSetupRequest;
 import com.netscape.certsrv.system.FinalizeConfigRequest;
 import com.netscape.certsrv.system.SecurityDomainSetupRequest;
 import com.netscape.certsrv.system.SystemCertData;
 import com.netscape.certsrv.system.SystemConfigResource;
+import com.netscape.cms.profile.common.EnrollProfile;
 import com.netscape.cms.servlet.base.PKIService;
 import com.netscape.cms.servlet.csadmin.Cert;
+import com.netscape.cms.servlet.csadmin.CertInfoProfile;
 import com.netscape.cms.servlet.csadmin.Configurator;
 import com.netscape.cms.servlet.csadmin.SystemCertDataFactory;
 import com.netscape.cmscore.apps.CMS;
 import com.netscape.cmscore.apps.CMSEngine;
 import com.netscape.cmscore.apps.EngineConfig;
 import com.netscape.cmscore.apps.PreOpConfig;
+import com.netscape.cmscore.cert.CertUtils;
 import com.netscape.cmsutil.crypto.CryptoUtil;
 
 /**
@@ -92,44 +98,6 @@ public class SystemConfigService extends PKIService implements SystemConfigResou
         configurator = engine.createConfigurator();
     }
 
-    /* (non-Javadoc)
-     * @see com.netscape.cms.servlet.csadmin.SystemConfigurationResource#configure(com.netscape.cms.servlet.csadmin.data.ConfigurationData)
-     */
-    @Override
-    public void configure(ConfigurationRequest request) throws Exception {
-
-        logger.info("SystemConfigService: configuring subsystem");
-
-        try {
-            validatePin(request.getPin());
-
-            if (csState.equals("1")) {
-                throw new BadRequestException("System already configured");
-            }
-
-            // configure security domain
-            logger.debug("=== Security Domain Configuration ===");
-            configurator.configureSecurityDomain(request);
-
-            logger.debug("=== Configure CA Cert Chain ===");
-            configurator.configureCACertChain(request);
-
-            cs.commit(false);
-
-        } catch (PKIException e) { // normal responses
-            logger.error("Configuration failed: " + e.getMessage()); // log the response
-            throw e;
-
-        } catch (Exception e) { // unexpected exceptions
-            logger.error("Configuration failed: " + e.getMessage(), e); // show stack trace for troubleshooting
-            throw e;
-
-        } catch (Error e) { // system errors
-            logger.error("Configuration failed: " + e.getMessage(), e); // show stack trace for troubleshooting
-            throw e;
-        }
-    }
-
     @Override
     public void setupClone(CloneSetupRequest request) throws Exception {
 
@@ -191,14 +159,6 @@ public class SystemConfigService extends PKIService implements SystemConfigResou
                 throw new BadRequestException("System already configured");
             }
 
-            PreOpConfig preopConfig = cs.getPreOpConfig();
-
-            boolean enable = preopConfig.getBoolean("cert." + tag + ".enable", true);
-            if (!enable) {
-                logger.info("SystemConfigService: " + tag + " certificate is disabled");
-                return null;
-            }
-
             SystemCertData certData = request.getSystemCert();
 
             if (certData == null) {
@@ -206,25 +166,6 @@ public class SystemConfigService extends PKIService implements SystemConfigResou
                 throw new BadRequestException("Missing certificate: " + tag);
             }
 
-            boolean generateServerCert = !request.getGenerateServerCert().equalsIgnoreCase("false");
-            if (!generateServerCert && tag.equals("sslserver")) {
-                logger.info("SystemConfigService: not generating " + tag + " certificate");
-                updateConfiguration(certData, "sslserver");
-                return null;
-            }
-
-            boolean generateSubsystemCert = request.getGenerateSubsystemCert();
-            if (!generateSubsystemCert && tag.equals("subsystem")) {
-                logger.info("SystemConfigService: not generating " + tag + " certificate");
-
-                // update the details for the shared subsystem cert here.
-                updateConfiguration(certData, "subsystem");
-
-                // get parameters needed for cloning
-                updateCloneConfiguration(certData, "subsystem");
-                return null;
-            }
-
             KeyPair keyPair = processKeyPair(certData);
 
             Cert cert = processCert(request, keyPair, certData);
@@ -411,8 +352,6 @@ public class SystemConfigService extends PKIService implements SystemConfigResou
         CryptoToken token = CryptoUtil.getKeyStorageToken(tokenName);
 
         String keytype = preopConfig.getString("cert." + tag + ".keytype");
-        String keyalgorithm = preopConfig.getString("cert." + tag + ".keyalgorithm");
-        String signingalgorithm = preopConfig.getString("cert." + tag + ".signingalgorithm");
 
         // support injecting SAN into server cert
         if (tag.equals("sslserver") && certData.getServerCertSAN() != null) {
@@ -425,7 +364,9 @@ public class SystemConfigService extends PKIService implements SystemConfigResou
                 logger.debug("SystemConfigService: san_server_cert not found");
             }
         }
+
         cs.commit(false);
+
         KeyPair pair;
 
         try {
@@ -452,9 +393,6 @@ public class SystemConfigService extends PKIService implements SystemConfigResou
             }
         }
 
-        logger.debug("SystemConfigService: storing key pair into CS.cfg");
-        configurator.storeKeyPair(tag, pair);
-
         return pair;
     }
 
@@ -476,11 +414,12 @@ public class SystemConfigService extends PKIService implements SystemConfigResou
         String nickname = preopConfig.getString("cert." + tag + ".nickname");
         String dn = preopConfig.getString("cert." + tag + ".dn");
         String subsystem = preopConfig.getString("cert." + tag + ".subsystem");
+        String type = preopConfig.getString("cert." + tag + ".type");
 
         Cert cert = new Cert(tokenName, nickname, tag);
         cert.setDN(dn);
         cert.setSubsystem(subsystem);
-        cert.setType(preopConfig.getString("cert." + tag + ".type"));
+        cert.setType(type);
 
         String fullName;
         if (!CryptoUtil.isInternalToken(tokenName)) {
@@ -510,7 +449,7 @@ public class SystemConfigService extends PKIService implements SystemConfigResou
                 || "kra".equals(subsystem) && (request.isExternal() || request.getStandAlone())
                 || "ocsp".equals(subsystem) && (request.isExternal()  || request.getStandAlone())) {
 
-            logger.info("SystemConfigService: loading existing " + tag + " certificate");
+            logger.info("SystemConfigService: Loading existing " + tag + " certificate");
 
             byte[] bytes = x509Cert.getEncoded();
             String b64 = CryptoUtil.base64Encode(bytes);
@@ -519,15 +458,15 @@ public class SystemConfigService extends PKIService implements SystemConfigResou
 
             cert.setCert(bytes);
 
-            configurator.updateConfig(cert);
+            cs.commit(false);
 
-            logger.debug("SystemConfigService: loading existing cert request");
-            byte[] binRequest = configurator.loadCertRequest(subsystem, tag);
-            String b64Request = CryptoUtil.base64Encode(binRequest);
+            logger.info("SystemConfigService: Loading existing " + tag + " cert request");
 
-            logger.debug("SystemConfigService: request: " + b64Request);
+            String certreqStr = cs.getString(subsystem + "." + tag + ".certreq");
+            logger.debug("SystemConfigService: request: " + certreqStr);
 
-            cert.setRequest(binRequest);
+            byte[] certreqBytes = CryptoUtil.base64Decode(certreqStr);
+            cert.setRequest(certreqBytes);
 
             // When importing existing self-signed CA certificate, create a
             // certificate record to reserve the serial number. Otherwise it
@@ -555,17 +494,51 @@ public class SystemConfigService extends PKIService implements SystemConfigResou
             }
 
             logger.debug("SystemConfigService: cert issued by existing CA, create record");
-            configurator.createCertRecord(cert);
+
+            CMSEngine engine = CMS.getCMSEngine();
+            ICertificateAuthority ca = (ICertificateAuthority) engine.getSubsystem(ICertificateAuthority.ID);
+
+            String profileName = preopConfig.getString("cert." + tag + ".profile");
+            logger.debug("SystemConfigService: profile: " + profileName);
+
+            String instanceRoot = cs.getInstanceDir();
+            String configurationRoot = cs.getString("configurationRoot");
+            CertInfoProfile profile = new CertInfoProfile(instanceRoot + configurationRoot + profileName);
+
+            PKCS10 pkcs10 = new PKCS10(certreqBytes);
+            X509Key x509key = pkcs10.getSubjectPublicKeyInfo();
+
+            X509CertImpl certImpl = new X509CertImpl(bytes);
+            X509CertInfo info = certImpl.getInfo();
+
+            IRequest req = configurator.createRequest(tag, profile, x509key, info);
+
+            req.setExtData(EnrollProfile.REQUEST_ISSUED_CERT, certImpl);
+            req.setExtData("cert_request", certreqBytes);
+            req.setExtData("cert_request_type", "pkcs10");
+
+            IRequestQueue queue = ca.getRequestQueue();
+            queue.updateRequest(req);
+
+            RequestId reqId = req.getRequestId();
+            preopConfig.putString("cert." + tag + ".reqId", reqId.toString());
+
+            CertUtils.createCertRecord(req, profile, certImpl);
 
             return cert;
         }
 
         // generate and configure other system certificate
         logger.info("SystemConfigService: generating new " + tag + " certificate");
-        configurator.configCert(request, keyPair, cert);
+        X509CertImpl certImpl = configurator.configCert(request, keyPair, cert);
 
-        String certStr = cs.getString(subsystem + "." + tag + ".cert" );
-        cert.setCert(CryptoUtil.base64Decode(certStr));
+        byte[] certBin = certImpl.getEncoded();
+        String certStr = CryptoUtil.base64Encode(certBin);
+        cert.setCert(certBin);
+
+        String subsystemName = preopConfig.getString("cert." + tag + ".subsystem");
+        cs.putString(subsystemName + "." + tag + ".cert", certStr);
+        cs.commit(false);
 
         logger.debug("SystemConfigService: cert: " + certStr);
 
@@ -575,73 +548,6 @@ public class SystemConfigService extends PKIService implements SystemConfigResou
         return cert;
     }
 
-    private void updateCloneConfiguration(
-            SystemCertData cdata,
-            String tag) throws Exception {
-
-        PreOpConfig preopConfig = cs.getPreOpConfig();
-
-        String tokenName = cdata.getToken();
-        if (StringUtils.isEmpty(tokenName)) {
-            tokenName = preopConfig.getString("module.token", null);
-        }
-
-        // TODO - some of these parameters may only be valid for RSA
-        CryptoManager cryptoManager = CryptoManager.getInstance();
-        String nickname;
-        if (!CryptoUtil.isInternalToken(tokenName)) {
-            logger.debug("SystemConfigService:updateCloneConfiguration: tokenName=" + tokenName);
-            nickname = tokenName + ":" + cdata.getNickname();
-        } else {
-            logger.debug("SystemConfigService:updateCloneConfiguration: tokenName empty; using internal");
-            nickname = cdata.getNickname();
-        }
-
-        boolean isECC = false;
-        String keyType = preopConfig.getString("cert." + tag + ".keytype");
-
-        logger.debug("SystemConfigService:updateCloneConfiguration: keyType: " + keyType);
-        if("ecc".equalsIgnoreCase(keyType)) {
-            isECC = true;
-        }
-        X509Certificate cert = cryptoManager.findCertByNickname(nickname);
-        PublicKey pubk = cert.getPublicKey();
-        byte[] exponent = null;
-        byte[] modulus = null;
-
-        if (isECC == false) {
-            exponent = CryptoUtil.getPublicExponent(pubk);
-            modulus = CryptoUtil.getModulus(pubk);
-            cs.putString("preop.cert." + tag + ".pubkey.modulus", CryptoUtil.byte2string(modulus));
-            cs.putString("preop.cert." + tag + ".pubkey.exponent", CryptoUtil.byte2string(exponent));
-        }
-
-        PrivateKey privk = cryptoManager.findPrivKeyByCert(cert);
-
-        preopConfig.putString("cert." + tag + ".privkey.id", CryptoUtil.encodeKeyID(privk.getUniqueID()));
-    }
-
-    private void updateConfiguration(SystemCertData cdata, String tag) throws Exception {
-
-        PreOpConfig preopConfig = cs.getPreOpConfig();
-
-        String tokenName = cdata.getToken();
-        if (StringUtils.isEmpty(tokenName)) {
-            tokenName = preopConfig.getString("module.token", null);
-        }
-
-        if (CryptoUtil.isInternalToken(tokenName)) {
-            cs.putString(csSubsystem + ".cert." + tag + ".nickname", cdata.getNickname());
-        } else {
-            cs.putString(csSubsystem + ".cert." + tag + ".nickname", tokenName +
-                    ":" + cdata.getNickname());
-        }
-
-        cs.putString(csSubsystem + "." + tag + ".nickname", cdata.getNickname());
-        cs.putString(csSubsystem + "." + tag + ".tokenname", StringUtils.defaultString(tokenName));
-        cs.putString(csSubsystem + "." + tag + ".dn", cdata.getSubjectDN());
-    }
-
     private void validatePin(String pin) throws Exception {
 
         if (pin == null) {
